C///////////////////////////////////////////////////////////////////////
C   DISK I-O ROUTINE
c        subroutines to read from and write to disk
c           standard ppp input and output is fits file (.fits files)
c          with block lenght 2880 one or more fits header blocks are allowed.
c           When data are read in, they are recorded in a PPPDIR.DAT file
c           For reading data, if the file does not exist in PPPDIR, a search
c           of the disk is made and PPPDIR is automatically updated.
c           
c           if a .fits file does not exist for a PPPDIR image, the second
c           default format is the old .PIC file (unformatted direct acess
c           record length=1 line of the image), and it will automatically
c           read such a file if it exists.
c
c           When an image is read into PPP core, a pointer to its file name
c           is established so that the header information is available.
c           Writing a file into the disk involves creating a fits header
c           using, in order of over-ride, the following information:
c                  (1) a new OBJ line to be input from the terminal
c                  (2) the current picture size
c                  (3) the header file of the parent fits image if exist.
c
C///////////////////////////////////////////////////////////////////////
c/////////////////////////////////////////////
c
c      subroutine to write core picture to disk
C        IREP=0 WRITE to DISK in .fits format
c             1 write to disk in .pic format
C        IREP=2 WRITE to DISK in .fits format, CORE DIR COMMENT ACCEPTED
C        IREP=3 WRITE to DISK in .pic format, CORE DIR COMMENT ACCEPTED
C        IREP=4 OVERWRITE PICTURE WITH EXISTING NAME IN DIR, in .fits format
C        IREP=5 OVERWRITE PICTURE WITH EXISTING NAME IN DIR, in .pic format
c        irep=6 write to disk in .fits using common header block as header
c        irep=7 write to disk in .pic using common header block for .fdh
C
	SUBROUTINE wdkfitsold(*,NFR,FILENC,IREP,IX,NU,NV,KF,IR)
c	INTEGER*2 IX(NU,NV,KF),NFLAG,NID,NJD
	INTEGER IX(NU,NV,KF),NFLAG,NID,NJD
	CHARACTER*24 NFILEN,FILEN
	CHARACTER*24 FILENC,FILENF,fitsname*24,filenh,objnam*36
	CHARACTER*36 NCOM,NCOMC,imdir*55
	character headblk*2880,hdr(360)*80,size*20,fdate*24,nyes*1
     *            ,itest1*3,blank*20,headbst*2880
	INCLUDE 'picdir.inc'
	INCLUDE 'ppparm.inc'
	common/imagedir/imdir
	COMMON/PICC/ NCOMC(15),fitsname(15)
	COMMON/DIR/ nof,NCOM(254),NFILEN(254),NFLAG(5,254),
     *              NID(254),NJD(254),nextend(254)
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	common/header/bzero,bscale,headblk(20),nhb
	equivalence (headblk(1)(1:1),hdr(1)(1:1))
	CALL QUERYNFR(NFR,KF,0,*900,*20001)
C decode filename
20001	IR=0
	blank='                   '
	IF (FILENC.EQ.' ') WRITE(6,1)
	CALL RDFILND(FILENC,NC,IPF)
	IF (IPF.LE.-1) THEN
C  name too long or aborted
	  IR=-1
	  RETURN1
	END IF              
C  get name for checking with directory
	FILEN=FILENC(1:NC)
C  get name for OPEN statement
	IF (IPF.EQ.0) THEN
C  no qualifier, append 
	  if (mod(irep,2).eq.1) then
	    FILENF=FILENC(1:NC)//'.pic'
	    filenh=filenc(1:nc)//'.fhd'
	  else
	    FILENF=FILENC(1:NC)//'.fits'
	  end if
	ELSE                           
C	  qualifier exits
	  FILENF=FILENC
	END IF
C
 1	FORMAT('  Enter output .fits file name for image: ',$)
C
C  Write into disk directory from disk, first check for duplicate
C 
C	  over write, dir taken care by calling prog
	IF (IREP.eq.4.or.irep.eq.5) GO TO 71
 79	IF (IREP.EQ.0.or.irep.eq.1) CALL DIREC(FILEN,FILENF,NFR,IP,2,IDF)
	IF (IREP.EQ.2.or.irep.eq.3.or.irep.eq.6.or.irep.eq.7) THEN 
	  CALL DIREC(FILEN,FILENF,NFR,IP,3,IDF)
	  IP=0
	END IF
C a duplicate name is found in DIR  
	IF (IP.EQ.1) THEN   
 600	  WRITE(6,2) FILEN
 2	FORMAT(' --- ',A9,' --- ALREADY EXIST, DO YOU WANT TO'
     *         ,' DELETE? (y/n): ',$)
	  call readstr(1,nyes,*900,*600)
	  IF (NYES.EQ.'y') THEN
C	      delete original
 	    CALL DIREC(FILEN,FILENF,NFR,IP,0,IDF)
	    GO TO 79
	  ELSE
	    WRITE(6,*) ' ...NO FILE WRITTEN.'
	    RETURN1
	  END IF
C   directory full
	ELSE  IF (IP.EQ.2) THEN
	  WRITE(6,*) ' *** NO FILE WRITTEN *** '
	  IR=-1
	  RETURN1
	ELSE IF (IP.EQ.-1) THEN
	  RETURN1
	END IF
C
C  start write:
C
 71	NI=NII(NFR)
	NJ=NJJ(NFR)
c  writing fits
	if (mod(irep,2).eq.0) then
	  open(unit=20,file=imdir(1:lnblnk(imdir))//filenf
     *        ,status='new',access='direct',recl=2880,err=9001)
	else
	  open(unit=20,file=imdir(1:lnblnk(imdir))//filenh
     *        ,status='new',access='direct',recl=2880,err=9001)
	end if
 777	format(1x,a18)
c  write header
C
C  A) write directly from headblk common block
C     NOTE:  this will only write one header block maximum
c	if (irep.eq.6.or.irep.eq.7) then
c	  ihb=1
c  	  write(20,rec=ihb) headblk
c	  goto 133
c	end if
c
c A) copy header block exactly  (for irep=6, 7)
	if (irep.eq.6.or.irep.eq.7) then
	  if (fitsname(nfr).ne.' ') then
	    open(unit=21,file=imdir(1:lnblnk(imdir))//fitsname(nfr),
     *            status='old',access='direct',recl=2880,err=155)
	      
	    do 117 ihb=1,10
c             READ (unit=21,rec=ihb,ERR=155,END=155) HEADBLK
              READ (unit=21,rec=ihb,ERR=155) HEADBLK
	      if (ihb.eq.1) then
c  for frames with fitsheader from uncertain source, check bitpix and scale
	        if (fitsname(nfr)(1:4).eq.'temp') then
	          if (hdr(2)(1:3).eq.'BIT') hdr(2)(29:30)='16'
                  DO 1181 I=1,36
                    IF (hdr(i)(1:3).eq.'BSC') 
     *               hdr(i)(11:30)='          1.00000000'
                    IF (hdr(i)(1:3).eq.'BZE') 
     *               hdr(i)(11:30)='          0.00000000'
 1181	          continue
	        end if
	      end if	          
 	      write(20,rec=ihb) headblk
c check for end card
              DO 118 I=1,36
                IF (hdr(i)(1:3).eq.'END')  goto 133
 118	      continue
 117	    continue
c  end card not found, make new header
	    write(6,*) '  *** header has no end card, making new one **'
	    close(unit=20,status='delete')
	    if (mod(irep,2).eq.0) then
	      open(unit=20,file=imdir(1:lnblnk(imdir))//filenf
     *          ,status='new',access='direct',recl=2880,err=9001)
	    else
	      open(unit=20,file=imdir(1:lnblnk(imdir))//filenh
     *           ,status='new',access='direct',recl=2880,err=9001)
	    end if
	  end if
c  make new header
	  goto 156
	end if
c
c  B) create header: first check if there is an existing header:
	if (fitsname(nfr).ne.' ') then
	  open(unit=21,file=imdir(1:lnblnk(imdir))//fitsname(nfr),
     *         status='old',access='direct',recl=2880,err=155)
c	  go to 1552
c 1551	  open(unit=21,file=imdir(1:lnblnk(imdir))//
c     *       fitsname(nfr)(1:lnblnk(fitsname(nfr)))//'.B',
c     *         status='old',access='direct',recl=2880,err=155)
c  read header and copy to output file
 1552	  ihb=1
c         READ (unit=21,rec=ihb,ERR=155,END=155) HEADBLK
          READ (unit=21,rec=ihb,ERR=155) HEADBLK
c  replace size : line 4 and 5
	  istatus=encode(size,float(ni),1,0,idum)
	  hdr(4)='NAXIS1  ='//BLANK(1:21-lnblnk(size))//size(1:
     *         lnblnk(size))//'  /'
	  istatus=encode(size,float(nj),1,0,idum)
	  hdr(5)='NAXIS2  ='//BLANK(1:21-lnblnk(size))//size(1:
     *         lnblnk(size))//'  /'
502     FORMAT (10X,I20)
c  read remaining header
c change OBJECT field, make sure bscale and bzero are 1 and 0, and
c  add a header card with the new time and ppp history
          KNH=6
	  imore=0
 150      IF (IHB.GT.1) KNH=1
          DO 110 I=KNH,36
            IF (hdr(i).EQ.' ') GO TO 110
            ITEST1=hdr(i)(1:3)
            IF (ITEST1.EQ.'OBJ')  then
c	        READ(hdr(i)(12:46),677) NCOMC(NFR)
c  610	        WRITE(6,512) hdr(i)(11:50) 
c 512	format('  Old Object field: ',a40/'  Enter new object name: ')
c	        read(5,513,err=610,end=900) hdr(i)(12:78)
c 513	format(a67)
c	        call fstr(hdr(i),80,ib,ie,ns)
c	        hdr(i)=hdr(i)(1:10)//' '''//hdr(i)(12:ie(ns))//'''/'
	        hdr(i)=hdr(i)(1:10)//' '''//ncomc(nfr)//'''/'
	    else if  (ITEST1.EQ.'BSC') then
	      READ(hdr(i)(11:30),*) BSCALE
	      if ((abs(bscale)-1.).gt.0.001) then
c  set bscale to 1
	        istatus=encode(size,1.0,1,1,idum)
	        hdr(i)=hdr(i)(1:10)//size//hdr(i)(31:80)
	      end if
            else if (ITEST1.EQ.'BZE') then
	      READ(hdr(i)(11:30),*) BZERO
	      if (abs(bzero).gt.0.001) then
c  set bzero to 0
	        istatus=encode(size,0.0,1,1,idum)
	        hdr(i)=hdr(i)(1:10)//size//hdr(i)(31:80)
	      end if
            else IF(ITEST1.EQ.'END') then
c  add ppp card
	      if (i.ne.36) then
	        hdr(i+1)=hdr(i)
	        hdr(i)=' HISTORY = ''Saved from ppp '//fdate()//' ''/'
	        GO TO 111
	      else
	        hdr(i)=' HISTORY = ''Saved from ppp '//fdate()//' ''/'
	        imore=1
	        go to 1110
	      end if
	    end if
 110      CONTINUE
 677    FORMAT(A20)
C   "END" is not reached, write out current headerblock and read another 
 1110 	  write(20,rec=ihb) headblk
         IHB=IHB+1
	 if (imore.eq.1) then
	   hdr(1)='END'
	   do 1111 k=2,36
	     hdr(k)=' '
 1111      continue
	   go to 111
	 end if
c        READ (21,rec=ihb,ERR=155,END=155) HEADBLK
         READ (21,rec=ihb,ERR=155) HEADBLK
         GO TO 150
c write out last header block
 111 	  write(20,rec=ihb) headblk
	  close(unit=21)
	else
c  case II no header block available, make a header block
	  go to 156
 155	  close(unit=21) 
	  write(6,*) '  .... can''t open fits header or read error',  
     *               ' -- creating new one...'
 156	  continue
c	  WRITE(6,515) 
 515	format('  Enter new object name: ')
	  objnam=ncomc(nfr) 
	  call makefhd(nfr,nii(nfr),njj(nfr),0,objnam)
	  ihb=1
	  write(20,rec=1) headblk
	end if
c  write out data 
c   case 1: for fits:
 133	if (mod(irep,2).eq.0) then
	  call fitsdataout(20,ihb,ix(1,1,nfr),nu,nv,
     *                    nii(nfr),njj(nfr),ibyte)
	  fitsname(nfr)=filenf
	else
c  case 2: for .pic:
c    first close .fhd file
	  close(unit=20)
c   open .pic file
	write(6,*) ' 3 '
	write(6,777) filenf
	  open(unit=20,file=imdir(1:lnblnk(imdir))//filenf,
     *          status='new',access='direct',recl=2*ni,err=9001)
	  fitsname(nfr)=filenh
          DO 102 J=1,NJ
	    call wrpic(ix(1,j,nfr),ni,j)
 102	  CONTINUE
	end if  
c  close loose ends
	if (irep.ne.4.and.irep.ne.5) then
	  ncomc(nfr)=objnam
	  nf(nfr)=nof
	end if
	CLOSE(UNIT=20)
 	WRITE(6,27) FILENF,ihb,NFR
 27	FORMAT ('  .fits file: ',A18,i4,' blks written to disk from <f>'
     *         ,I3)
	  RETURN1
 9001	WRITE(6,*) '  ****ILLEGAL FILE NAME***'
	close(unit=20)
	IR=-1
 900	RETURN1
	END
c//////////////////////////////////////////////////
c
c  subroutine to write out fits data (data only)
c
	subroutine fitsdataout1(idev,nblk,ix,nu,nv,ni,nj,ibyte)
c	integer*2 ix(nu,nv),ixt(1440)
	integer ix(nu,nv)
	integer*2 ixt(1440)
	i=1
	j=1
 201	nblk=nblk+1
	do 202 k=1,1440
	  if (i.gt.ni) then
c  go to next line
	    j=j+1
	    if (j.gt.nj) then
c  pad with zeros
	      do 203, l=k,1440
	        ixt(l)=0
 203	      continue
	      if (ibyte.eq.1) call byte_swap(ixt,2880)
	      write(idev,rec=nblk) ixt
	      return
	    end if
	    i=1
	  end if
	  ixt(k)=ix(i,j)
	  i=i+1
 202	continue
	if (ibyte.eq.1) call byte_swap(ixt,2880)
	write(idev,rec=nblk) ixt
	go to 201
	end
c/////////////////////////////////////////////////////////////////////
c
c  Subroutine to read data from disk either .pic or .fits file
c    if it is a .pic file, a .fhd (fits header file) is assumed to 
c    accompany the data
c
C///////////////////////////////////////////////////////////////////////
C   DISK I-O ROUTINE
C        IREP=1 READ FROM DISK
C        IREP=3 READ FROM DISK WITH NAME SUPPLIED FROM
C               THE DIRECTORY DIRECTLY
c
c        return IR=-1 read not successful
c               IR= 4 picture is in fits format
c               IR= 5 picture is in .pic format
c                       (the last 2 output is for MFO operations)
C
C///////////////////////////////////////////////////////////////////////
	SUBROUTINE rdkfitsold(*,NFR,FILENC,IREP,IX,NU,NV,KF,IR)
c	INTEGER*2 IX(NU,NV,KF),NFLAG,NID,NJD
	INTEGER IX(NU,NV,KF),NFLAG,NID,NJD
	CHARACTER*24 NFILEN,FILEN
	CHARACTER FILENC*24,FILENF*24,filenh*18,fitsname*24
	character headblk*2880,hdr(36)*80,headbst*2880
	CHARACTER*36 NCOM,NCOMC,objnam,imdir*55
	logical ex
	INCLUDE 'picdir.inc'
	INCLUDE 'mosaic.inc'
	include 'imagepar.inc'
	COMMON/PICC/ NCOMC(15),fitsname(15)
	common/imagedir/imdir
	COMMON/DIR/ nof,NCOM(254),NFILEN(254),NFLAG(5,254),
     *              NID(254),NJD(254),nextend(254)
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	common/header/bzero,bscale,headblk(20),nhb
	include 'ppparm.inc'
	call stthead(nfr)
	CALL QUERYNFR(NFR,KF,0,*999,*20001)
C decode filename
20001	IR=0
	IF (FILENC.EQ.' ') WRITE(6,10)
 10	FORMAT('  Enter NAME of file to be READ: ',$)
c	CALL RDFILN(FILENC,NC,IPF)
	CALL RDFILND(FILENC,NC,IPF)
C  name too long or aborted
	IF (IPF.LE.-1) go to 991
C  get name for checking with directory
	FILEN=FILENC(1:NC)
	IF (IPF.EQ.0) THEN
C	get name for OPEN statement
C  no qualifier, append .fits
	  FILENF=FILENC(1:NC)//'.fits'
	ELSE                           
C  qualifier exits
	  FILENF=FILENC
	END IF
C
C  first check for name in directory, IP=0 if not exist
C
 	CALL DIREC(FILEN,FILENF,NFR,IP,1,IDF)
C
C  File name does not exit in pppdir; 
	IF (IP.EQ.0) then
	  WRITE(6,301) FILENF
 301	FORMAT(' ++ ',A20,' ++ does not exist in ppp DIRECTORY'
     *        /'    attempting to locate unlisted .pic/.fits file....')
	else
	  WRITE(6,22) FILEN,ncom(idf),nid(idf),njd(idf)
	end if
c
c  PART 1 for .fits file
c  attempt to open file
 3011	open(unit=20,file=imdir(1:lnblnk(imdir))//filenf,
     *       access='direct',recl=2880,status='old',err=302)
c .fits file successfully opened: read
	if (ipppfile.gt.1) then
	   write (6,*) 
     *   '++++ warning: unsigned integer fits file, image divided by '
     *    ,ipppfile
	  nfl(6,nfr)=1
	end if
	call fitsin(20,ix(1,1,nfr),nu,nv,ni,nj,objnam,ifitshead,ibyte,
     *          *999)
	iftype=0
	if (ip.eq.0) WRITE(6,22) FILEN,objnam,NI,NJ
	fitsname(nfr)=filenf
	IR=4
	go to 305
 302	close(unit=20)
c   try to find compressed version
c  for unix version -- first try for compressed version
	inquire(file=imdir(1:lnblnk(imdir))//filenf(1:lnblnk(filenf))
     *               //'.Z',exist=ex)
	if (ex.eqv..true.) then
c  uncompress
	  write(idout,*) '   +++ uncompressing .Z version ... '
	  idum=system('uncompress '//imdir(1:lnblnk(imdir))//
     *         filenf(1:lnblnk(filenf))//'.Z')
	  go to 3011
c  else look for gz file
	else
	  inquire(file=imdir(1:lnblnk(imdir))//filenf(1:lnblnk(filenf))
     *               //'.gz',exist=ex)
	  if (ex.eqv..true.) then
c gunzip 
	    write(idout,*) '   +++ gunzipping .gz version ... '
	    idum=system('gunzip '//imdir(1:lnblnk(imdir))//
     *         filenf(1:lnblnk(filenf))//'.gz')
	    go to 3011
	  end if
	end if
c
c..................................................
c  PART 2: for .pic files
c
c  can't find .fits file trying  .pic file	
	IF (IPF.EQ.0) THEN
C	get name for OPEN statement
C  no qualifier, append .pic
	  FILENF=FILENC(1:NC)//'.pic'
	ELSE                           
C  qualifier exits
	  FILENF=FILENC
	END IF
	if (filenc(nc-3:nc).ne.'.pic') then
	  filenh=filenc(1:nc)//'.fhd'	
	else 
	  filenh=filenc(1:nc-4)//'.fhd'
	end if
	if (ip.eq.0) then
c  if the name does not exist in pppdir, see if there is a header file
	  open(unit=21,file=imdir(1:lnblnk(imdir))//filenh,
     *          access='direct',recl=2880,status='old',err=303)
c read header to get ni and nj
	  read(21,rec=1) headblk
	  read(hdr(4),15,err=995) ni
	  read(hdr(5),15,err=995) nj
 15	format(10x,i20)
c read object name
	  do 100 i=ihc,36
	    if (hdr(i)(1:3).eq.'OBJ') then
	      read(hdr(i),16) objnam
 16	format(11x,a35)
	      go to 101
	    end if
 100	  continue
 101	  iftype=2
	  go to 304
c   --- no header, ask for size
 303	  write(6,21) 
 21	format('  ++ no .fits nor .pic header file.'/
     *          '   Enter I,J size to read, or n(=dflt) to give up: ',$)
	  call readi4(2,ni,nj,idum,*999,*998,*998,*304) 
	  iftype=3
	  filenh='none'
	else
c  picture is in pppdir
	  NI=NID(IDF)
	  NJ=NJD(IDF)
c	  WRITE(6,22) FILEN,NCOM(IDF),NI,NJ
 22	FORMAT(1X,A18,' : ''',A35,''' size: ',I4,' X ',I4)
c  look for .fhd file
	  open(unit=21,file=imdir(1:lnblnk(imdir))//filenh,
     *          access='direct',recl=2880,status='old',err=306)
	  close(unit=21)
	  go to 304
 306	  iftype=4
	  close(unit=21)
	end if 
 304	IF (NI.GT.NU.OR.NJ.GT.NV) GO TO 996
c  file is found, open it
	NREC=ni*2
 	OPEN (UNIT=20,FILE=imdir(1:lnblnk(imdir))//filenh,
     *       status='OLD',ACCESS='DIRECT',recl=NREC,ERR=998)
c  open .pic successfully -- now read
	fitsname(nfr)=filenh
        DO 102 J=1,NJ
	  call rdpic(ix(1,j,nfr),ni,j)
 102	CONTINUE
	IR=5
 	CLOSE(UNIT=20)
c.......................................................
c  reading finished, now tidy up the ppp and core directory
c
 305	NII(NFR)=NI
	NJJ(NFR)=NJ
	call headtst(nfr)
c get values of key words from header
	call getkeyval(*309,nfr,kf)
 309	continue
C 1) file exists in ppp directory
 	if (ip.ne.0) then
C       change core directory
	  NF(NFR)=IDF
	  NCOMC(NFR)=NCOM(IDF)
	  DO 233 K=1,5
 233	    NFL(K,NFR)=NFLAG(K,IDF)
	  if (ipppfile.le.1) then
	    nfl(6,nfr)=0
	  else
	    nfl(6,nfr)=1
	  end if
          WRITE(6,270) FILENF,NFR
  270 FORMAT ('  Image ',A18,' read from DISK into FRAME',I3)
	  if (iftype.eq.4) then
c  create a .fhd file
	    objnam=ncom(idf)
	    call makefhd(idf,ni,nj,0,objnam)
	    write(6,85) filenh
	    open(unit=21,file=imdir(1:lnblnk(imdir))//filenh,
     *         access='direct',recl=2880,status='new',err=979)
	    write(21,rec=1) headblk
	    close(unit=21)
	  end if
 85	format(3x,a18,' file created for .pic file...')
 	  RETURN1
	else 	
C 2) file does not exist in directory -- up date ppp directory
C
	  DO 230 M=1,5
 230	    NFL(M,NFR)=0
c a)  it was a fits or a .pic file with a header 
	  if (iftype.le.2) then
	    call direc(filen,filenf,nfr,ip,4,idf)
c add header to ncom
	    ncom(nof)=objnam
	  else if (iftype.eq.3) then
	    CALL DIREC(FILEN,FILENF,NFR,IP,2,IDF)
c  create a header file for the .pic picture
	    call makefhd(nof,ni,nj,0,objnam)
	    fitsname(nfr)=filen(1:lnblnk(filen))//'.fhd'
	    open(unit=21,file=imdir(1:lnblnk(imdir))//fitsname(nfr),
     *         access='direct',recl=2880,status='new',err=979)
	    write(21,rec=1) headblk
	    close(unit=21)
	    write(6,85) fitsname(nfr) 
	  end if
	  IF (IP.EQ.2) THEN
	    NF(NFR)=0 
   	    WRITE(6,*) ' **** NO NEW DIRECTORY ENTRY IS MADE ****'
	  ELSE	     
	    NF(NFR)=NOF
	    WRITE(6,*) ' ...NEW ENTRY to ppp DIRECTORY has been made'
	  END IF
	  return1
	END IF
 997	WRITE(6,19) NFILENF
 19	FORMAT(' **** OPEN ERROR ****',A18,' NO such file, or incrorre'
     *    ,'ct SIZE')
	IR=-1
	RETURN1 
c
c  ***error messages***
 991	WRITE(6,*) '  ****ILLEGAL FILE NAME***'
	IR=-1
	RETURN1
 996	WRITE(6,18) NI,NJ
 18	FORMAT(1X,' ***** PICTURE SIZE: ',I4,' X',I4,' TOO LARGE FOR',
     *         ' CURRENT CONFIGURATION***')
	return1
 995	write(6,*) '  **** error in .fhd file ****'
	return1
 979	write(6,*) '  ** error in creating .fdh file, already exist **'
	return1
 998	write(6,*) '   +++ image not read +++'
 999	RETURN1
	END
c///////////////////////////////////////////////
	subroutine stthead(nfr)
	character headbst*2880,headblk*2880
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	common/header/bzero,bscale,headblk(20),nhb
	bzero=bzest(nfr)
	bscale=bscst(nfr)
	nhb=nhbst(nfr)
	do 100 i=1,10
	  headblk(i)=headbst(i,nfr)
 100	continue
	return
	end
c///////////////////////////////////////////////
	subroutine headtst(nfr)
	character headbst*2880,headblk*2880
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	common/header/bzero,bscale,headblk(20),nhb
	bzest(nfr)=bzero
	bscst(nfr)=bscale
	nhbst(nfr)=nhb
	do 100 i=1,20
	  headbst(i,nfr)=headblk(i)
 100	continue
	return
	end
C
C****************************************************************************
c  alternate reading routine: not found to be any faster than the one being used.
C
C
c        SUBROUTINE fitsin1(idev,ix,nu,nv,ni,nj,objnam,ipar,ifh,*)
c        INTEGER*2 IX(NU,NV),ixy(1000000)

C
C  read data
c	i=1
c	j=1	
c	write(6,*) '  bscale ',bscale,'  bzero ',bzero
c	ntot=nint((float(ni*nj)+1440.)/1440.)
c	m=1
c	write(6,*) krec,ntot
c	do 220 k=krec+1,ntot
c	  read(idev,rec=k,err=985,end=984) ixy(m)
c	  m=m+1440
c 220	continue
c	k=0
c	if (abs(1.-bscale).lt.0.0001.and.abs(bzero).lt.0.01) then
c	  do 221 j=1,nj
c	    do 222 i=1,ni
c	     k=k+1
c	     ix(i,j)=ixy(k)
c 222	   continue
c 221	  continue 
c	else
c	  do 231 j=1,nj
c	    do 232 i=1,ni
c	     k=k+1
c	     ix(i,j)=nint(ixy(k)*bscale-bzero)
c 232	   continue
c 231	  continue 
c	end if
c	return

C
c////////////////////////////////////////////////////////
C   Subroutine to read pictures in FITS format on disk to the core
C****************************************************************************
C
C
        SUBROUTINE fitsin2(idev,ix,nu,nv,ni,nj,objnam,ipar,ifh,ibyte,*)
c       INTEGER*2 IX(NU,NV),ixy(1440)
        INTEGER IX(NU,NV)
	integer*2 ixy(1440)
        INTEGER NPAX(2),bitpix
c       REAL*8 BSCALE,BZERO
        CHARACTER*20 ITEST1
        CHARACTER*36 objnam
        CHARACTER HEADBLK*2880,HDR(360)*80
	common/header/bzero,bscale,headblk(20),nhb
	equivalence (headblk(1)(1:1),hdr(1)(1:1))
	krec=1
c         READ (unit=idev,rec=krec,ERR=986,END=986) HEADBLK
          READ (unit=idev,rec=krec,ERR=986) HEADBLK
C  go through header block and extract needed info
          NHEAD=1
          IHB=0
          IF (hdr(1)(30:30).NE.'T') then
 	    WRITE(6,*) '  **** NOT SIMPLE FITS, image not read  ***'
            RETURN1
	  end if	
C  Get "Number-of-bits-per-data-point"
          READ (hdr(2),502,err=990) bitpix
	  if (bitpix.ne.16) then
	    write(6,*) '  *** cannot read fits not in short integer ***'
	    return1
	  end if	
502     FORMAT (10X,I20)
C  Get number of axes
          READ (hdr(3),502,err=990) NAXIS
	  if (naxis.gt.2.or.naxis.lt.1) then
	    write(6,8) naxis
 8	format('  NAXIS=',i2,
     *      '  ** can''t deal with higher than 2 dimension **')
	    return1
	  else if (naxis.eq.1) then
	    npax(2)=1
	  end if
C  Get number of data points along first axis, etc.
          DO 107 I=1,NAXIS
            READ (hdr(3+i),502,err=990) NPAX(I)
 107      CONTINUE
          IF (NPAX(1).GT.NU.OR.NPAX(2).GT.NV) THEN
            WRITE(6,*)' +++ Picture size too large for configuration'
            RETURN1
          END IF
          NI=NPAX(1)
          NJ=NPAX(2)
c	  bscale=1.
c	  bzero=0.
	  if (ifh.eq.1) then
	    do 108 k=1,2+i
 108	      write(6,30) hdr(k)
	  end if
c  read remaining header
C  get bscale and bzero
C  find objectname and put it in the comment field
          KNH=3+i
 150      IF (IHB.GE.1) KNH=1
          DO 110 i=knh,36
            IF (hdr(i).EQ.' ') GO TO 110
            ITEST1=hdr(i)(1:3)
	    if (ifh.eq.1) then
	      if (hdr(i)(1:6).eq.'FILTER') then
	        write(6,30) hdr(i)
	      else if (hdr(i)(4:7).eq.'TIME') then
	        write(6,30) hdr(i)
	      else if (hdr(i)(1:2).eq.'UT') then
	        write(6,30) hdr(i)
	      else if (hdr(i)(1:4).eq.'DATE') then
	        write(6,30) hdr(i)
	      end if
	    end if
            IF (itest1.EQ.'OBJ')  then
	        READ(hdr(i)(11:45),503) objnam
	      if (ifh.eq.1) write(6,30) hdr(i)
	    else if (ITEST1.EQ.'BSC') then
	      READ(hdr(i)(11:30),*,err=990) BSCALE
	      if (ifh.eq.1) write(6,30) hdr(i)
            else if (ITEST1.EQ.'BZE') then
	      READ(hdr(i)(11:30),*,err=990) BZERO
	      if (ifh.eq.1) write(6,30) hdr(i)
            else IF(ITEST1.EQ.'END') then
	      if (ifh.eq.2) then 
	        do 109 k=1,i
 109	          write(6,30) hdr(k)(1:79)
	      end if
	      GO TO 111
	    end if
 503     FORMAT(A20)
 110      CONTINUE
C   "END" is not reached, read another header block
	 if (ifh.eq.2) then
	   do 112 k=1,36
 112	     write(6,30) hdr(k)
	 end if
 30	format(1x,a79)
	 krec=krec+1
c        READ(idev,rec=krec,ERR=986,END=986) HEADBLK
         READ(idev,rec=krec,ERR=986) HEADBLK
         IHB=IHB+1
         GO TO 150
111       CONTINUE
C
C  read data
	i=1
	j=1	
c	write(6,*) '  bscale ',bscale,'  bzero ',bzero
	if (abs(1.-bscale).lt.0.0001.and.abs(bzero).lt.0.01) then
 124	  krec=krec+1
c	  read(idev,rec=krec,err=985,end=984) ixy
	  if (i.eq.ni+1.and.j.eq.nj) return
	  read(idev,rec=krec,err=985) ixy
	  if (ibyte.eq.1) call byte_swap(ixy,2880)
           do 122 k=1,1440
	     if (i.gt.ni) then
	       j=j+1
	       if (j.gt.nj) go to 123
	       i=1
	     end if
	     ix(i,j)=ixy(k)
	     i=i+1
 122	   continue
	  go to 124
	else
 125	  krec=krec+1
c	  read(idev,rec=krec,err=985,end=984) ixy
	  if (i.eq.ni+1.and.j.eq.nj) return
	  read(idev,rec=krec,err=985) ixy
	  if (ibyte.eq.1) call byte_swap(ixy,2880)
           do 126 k=1,1440
	     if (i.gt.ni) then
	       j=j+1
	       if (j.gt.nj) go to 123
	       i=1
	     end if
c	     if (imul.le.1) then
	       ix(i,j)=nint(ixy(k)*bscale+bzero)
c	     else
c	       ttemp=(ixy(k)*bscale+bzero)/float(imul)
c	       ix(i,j)=nint(ttemp)
c	     end if
	     i=i+1
 126	   continue
	  go to 125
	end if
 123	return
 984    if (i.eq.ni+1.and.j.eq.nj) then
c  when the last block is filled exactly, normal return
	  return
	else
	  WRITE(6,*) '  +++ DATA ENDED PREMATURELY +++ at ',I,J
	  return
	end if
 990	write(6,*) '  *** error reading header ****'
	return1
 986    WRITE(6,*) '  **** CAN''T READ HEADER *** ',IHB+1
        RETURN1
 985    WRITE(6,*) '  **** ERR READING DATA *** at ',I,J
	return
 999    RETURN1
        END

C****************************************************************************
d  copied from diskio of starsub.f of ppplt
C
c  Aug 13 2003:  rewrote header block reading
C
        SUBROUTINE fitsin(idev,ix,nu,nv,ni,nj,objnam,ifh,ibyte,*)
        integer IX(NU,NV)
        INTEGER NPAX(2),bitpix
	integer*2 ixy(1440)
c       REAL*8 BSCALE,BZERO
        CHARACTER*20 ITEST1
        CHARACTER*36 objnam
        CHARACTER headblk*2880,HDR(360)*80
 	common/header/bzero,bscale,headblk(20),nhb
	equivalence (headblk(1)(1:1),hdr(1)(1:1))
	include 'picdir.inc'
c read the header blocks, and search for "END"
c	ifh=0
c	ifh=1
	do 210 krec=1,20
          READ (unit=idev,rec=krec,ERR=986) HEADBLK(krec)
c  36 cards per headblk
	  do 211 kn=(krec-1)*36+1,krec*36
	    if (hdr(kn).eq.'END') then
	      write(6,*) ' Total header blk: ',krec, ' total cards: ',kn
	      goto 212
	    end if
 211	  continue
 210	continue
 212	nhb=krec
C  go through header block and extract needed info
c   first, always assume the first 3 cards to contain data type and size
c     info
        IF (hdr(1)(30:30).NE.'T') then
 	  WRITE(6,*) '  **** NOT SIMPLE FITS, image not read  ***'
          RETURN1
	end if	
C  Get "Number-of-bits-per-data-point"
        READ (hdr(2),502) bitpix
	if (bitpix.ne.16) then
	  write(6,*) '  *** cannot read fits not in short integer ***'
	  return1
	end if	
502     FORMAT (10X,I20)
C  Get number of axes
        READ (hdr(3),502,err=990) NAXIS
	if (naxis.gt.2.or.naxis.lt.1) then
	  write(6,8) naxis
 8	format('  NAXIS=',i2,
     *      '  ** can''t deal with higher than 2 dimension **')
	    return1
	else if (naxis.eq.1) then
	  npax(2)=1
	end if
C  Get number of data points along first axis, etc.
        DO 207 kn0=1,NAXIS
          READ (hdr(3+kn0),502,err=990) NPAX(kn0)
 207    CONTINUE
        IF (NPAX(1).GT.NU.OR.NPAX(2).GT.NV) THEN
          WRITE(6,*)' +++ Picture size too large for configuration'
          RETURN1
        END IF
        NI=NPAX(1)
        NJ=NPAX(2)
c default values for bscale and bzero
	bscale=1.
	bzero=0.
c	  if (ifh.eq.1) then
c	    do 108 k=1,2+i
c 108	      write(6,30) hdr(k)
c	  end if
c  read remaining header
C  get bscale and bzero
C  find objectname and put it in the comment field
	do 220 i=3+kn0,kn
          IF (hdr(i).EQ.' ') GO TO 220
          ITEST1=hdr(i)(1:3)
	    if (ifh.eq.1) then
	      if (hdr(i)(1:6).eq.'FILTER') then
	        write(6,30) hdr(i)
	      else if (hdr(i)(4:7).eq.'TIME') then
	        write(6,30) hdr(i)
	      else if (hdr(i)(1:2).eq.'UT') then
	        write(6,30) hdr(i)
	      else if (hdr(i)(1:4).eq.'DATE') then
	        write(6,30) hdr(i)
	      end if
 30     format(1x,a79)
	    end if
c	ifh=1
            IF (hdr(i)(1:6).EQ.'OBJECT')  then
	        READ(hdr(i)(11:30),'(a20)') objnam
	      if (ifh.eq.1) write(6,30) hdr(i)
	    else if (ITEST1.EQ.'BSC') then
	      READ(hdr(i)(11:30),*,err=990) BSCALE
	      if (ifh.eq.1) write(6,30) hdr(i)
            else if (ITEST1.EQ.'BZE') then
	      READ(hdr(i)(11:30),*,err=990) BZERO
	      if (ifh.eq.1) write(6,30) hdr(i)
            else IF(ITEST1.EQ.'END') then
c	      if (ifh.eq.2) then 
c	        do 109 k=1,i
c 109	          write(6,30) hdr(k)(1:79)
c	      end if
	      GO TO 111
	    end if
c look for parameters to keep
	    if (ifh.eq.2) then
	      if (itest1.eq.'AIR')  then
 	        read(hdr(i)(23:34),*,err=990) airmassobs
 	        write(6,*) 'airmass',airmassobs
	      else if (hdr(i)(1:7).eq.'DATE-OB') then
c	      read(hdr(i)(12:30),'(a21)') datestring
 	      write(6,*) 'datestring'
	      else if (itest1.eq.'MJD') then
 	      write(6,*) 'mjuldate'
c	      read(hdr(i)(12:30),*,err=990) mjuldate
	    else if (itest1.eq.'GAI') then
 	      write(6,*) 'CCDgain'
c	      read(hdr(i)(20:30),*,err=990) ccdgain
	    else if (itest1.eq.'RDN') then
 	      write(6,*) 'readnoise'
c	      read(hdr(i)(20:30),*,err=990) readnoise
	    else if (itest1.eq.'SAT') then
 	      write(6,*) 'saturation'
c	      read(hdr(i)(20:30),*,err=990) isaturate
	    else if (hdr(i)(1:5).eq.'EXPTI') then
 	      write(6,*) 'exptime'
c	      read(hdr(i)(20:30),*,err=990) exptime 
	    else if (hdr(i)(1:6).eq.'PIXSCA') then
 	      write(6,*) 'pixscale'
c	      read(hdr(i)(20:30),*,err=990) pixscale
 	    else if (hdr(i)(1:2).eq.'RA') then
c 	      read(hdr(i)(11:27),*) raobj
 	    else if (hdr(i)(1:3).eq.'DEC') then
c 	      read(hdr(i)(11:27),*) decobj
 	    else if (hdr(i)(1:5).eq.'EPOCH') then
c 	      read(hdr(i)(20:30),*) epoch 
	    end if
	    end if
 220     CONTINUE
 111     CONTINUE
C
C  read data
	i=1
	j=1	
 	write(6,*) '  bscale ',bscale,'  bzero ',bzero,' ni,nj',ni,nj
	if (abs(1.-bscale).lt.0.0001.and.abs(bzero).lt.0.01) then
 124	  krec=krec+1
	  read(idev,rec=krec,err=985) ixy
  	  if (ibyte.eq.1) call byte_swap(ixy,2880)
           do 122 k=1,1440
	     if (i.gt.ni) then
	       j=j+1
	       if (j.gt.nj) go to 123
	       i=1
	     end if
	     ix(i,j)=ixy(k)
	     i=i+1
 122	   continue
	  go to 124
	else
 125	  krec=krec+1
	  read(idev,rec=krec,err=985) ixy
 	  if (ibyte.eq.1) call byte_swap(ixy,2880)
           do 126 k=1,1440
	     if (i.gt.ni) then
	       j=j+1
	       if (j.gt.nj) go to 123
	       i=1
	     end if
	     ix(i,j)=nint(ixy(k)*bscale+bzero)
	     i=i+1
 126	   continue
	  go to 125
	end if
 123	return
 984    if (i.eq.ni+1.and.j.eq.nj) then
c  when the last block is filled exactly, normal return
	  return
	else
	  WRITE(6,*) '  +++ DATA ENDED PREMATURELY +++ at ',I,J
	  return 
	end if
 990	write(6,*) '  *** error reading header ****'
	return1
 986    WRITE(6,*) '  **** CAN''T READ HEADER *** ',IHB+1
        RETURN1
 985    WRITE(6,*) '  **** ERR READING DATA *** at ',I,J
 999    RETURN1
        END
c
c//////////////////////////////////////////////////////////////
c
c  subroutine to make a simple .fits header, returning 1 2880 byte
c      this does not use the common headblk
c     block
	subroutine makefhd1(ni,nj,objnam,headblk,bzero,bscale)
	character objnam*36,fdate*24,headblk*2880,hdr(36)*80,size*5
     *            ,blank*20,bzstr*20,headblkt*2880
	equivalence (headblkt(1:1),hdr(1)(1:1))
	blank='                    '
	hdr(1)='SIMPLE  =                    T  /  FITS STANDARD'
	hdr(2)='BITPIX  =                   16  /  FITS BITS/PIXEL'
	hdr(3)='NAXIS   =                    2  /  NUMBER OF AXES'
	istatus=encode(size,float(ni),1,0,idum)
	hdr(4)='NAXIS1  ='//BLANK(1:21-lnblnk(size))//size(1:
     *         lnblnk(size))//'  /'
	istatus=encode(size,float(nj),1,0,idum)
	hdr(5)='NAXIS2  ='//BLANK(1:21-lnblnk(size))//size(1:
     *         lnblnk(size))//'  /'
        istatus=encode(bzstr,bscale,1,6,idum)
        hdr(6)='BSCALE  =    '//bzstr//' / PIX= FITS*BS + BZ'
c *** fix bzero
        istatus=encode(bzstr,bzero,1,6,idum)
        hdr(7)='BZERO   =    '//bzstr//' / PIX= FITS*BS + BZ'
	hdr(8)='OBJECT  = '''//objnam//''''
	hdr(9)='HISTORY = Saved from PPP '//fdate()
	hdr(10)='END'
	do 100 i=11,36
	  hdr(i)=' '
 100	continue
	headblk=headblkt
	return
	end
c
	subroutine wrpic(ixt,ni,j)
c	integer*2 ixt(ni)
	integer ixt(ni)
 	WRITE (20,rec=j) ixt
	return
	end
c
	subroutine rdpic(ixt,ni,j)
c	integer*2 ixt(ni)
	integer ixt(ni)
	    READ (20,rec=J) ixt
	return
	end
C
C///////////////////////////////////////////////////////////////////////
C
C  SUBROUTINE TO READ DISK PICTURE OF A GIVEN PPPDIR FILE NUMBER
C
C///////////////////////////////////////////////////////////////////////
	SUBROUTINE RDIRFILold(*,NFR,NFIL,IX,NU,NV,KF)
c	INTEGER*2 IX(NU,NV,KF),NFLAG,NID,NJD
	INTEGER IX(NU,NV,KF),NFLAG,NID,NJD
	CHARACTER*24 NFILEN,NFILDUM
	CHARACTER*36 NCOM,NCOMC,fitsname*24
	INCLUDE 'picdir.inc'
	COMMON/PICC/ NCOMC(15),fitsname(15)
	COMMON/DIR/ nof,NCOM(254),NFILEN(254),NFLAG(5,254),
     *              NID(254),NJD(254),nextend(254)
	CALL QUERYNFR(NFR,KF,0,*999,*20001)
20001	IF (NFIL.EQ.0) THEN
 600	  WRITE(6,10)
 10	FORMAT('  Enter PPPDIR file number to be read: ',$)
	  call read1i(nfil,*999,*600)
	END IF
	IF (NFIL.LT.1.OR.NFIL.GT.NOF) THEN
	  WRITE(6,*) ' **** File number too large or <0 ****'
	  RETURN1
	ELSE
	  NFILDUM=' '
	  CALL rdkfits(*80,NFR,NFILEN(NFIL)//NFILDUM,3,IX,NU,NV,KF,IR) 
 80	  CONTINUE
	END IF
 999	RETURN1
	END
C///////////////////////////////////////////////////////////////////////
c
c subroutine to output header of data file
c
	SUBROUTINE headout(*,nfr,filenc,ifitshead,kf)
	CHARACTER*18 FILENC,filenf,fitsname*24
	CHARACTER*36 NCOMC,imdir*55
	COMMON/PICC/ NCOMC(15),fitsname(15)
	common/imagedir/imdir
c check input
	if (nfr.ne.0.or.filenc.eq.' ') then
	  CALL QUERYNFR(NFR,KF,0,*999,*20001)
20001	  filenc=fitsname(nfr)
	end if
c	CALL RDFILN(FILENC,NC,IPF)
 	CALL RDFILND(FILENC,NC,IPF)
c	write(6,*) filenc,ipf
C decode filename
C  name too long or aborted
	IF (IPF.LE.-1) go to 991
	IF (IPF.EQ.0) THEN
C	get name for OPEN statement
C  no qualifier, append .fits
	  FILENF=FILENC(1:NC)//'.fits'
	ELSE                           
C  qualifier exits
	  FILENF=FILENC
	END IF
c
c  PART 1 for .fits file
c  attempt to open file
 150	open(unit=20,file=imdir(1:lnblnk(imdir))//filenf,
     *        access='direct',recl=2880,status='old',err=302)
	go to 151
c try .fhd file
 302	close(unit=20)
	IF (IPF.EQ.0) THEN
	  FILENF=FILENC(1:NC)//'.fhd'
	ELSE 
	  filenf=filenc(1:nc-4)//'.fhd'
	end if
 	open(unit=20,file=imdir(1:lnblnk(imdir))//filenf,
     *        access='direct',recl=2880,status='old',err=997)
 151	call whead(20,ifitshead)
 152 	CLOSE(UNIT=20)
	return1
c  ***error messages***
 997	WRITE(6,19) NFILENF
 19	FORMAT(' **** OPEN ERROR ****',A18,' NO such file')
	goto 152
c
 991	WRITE(6,*) '  ****ILLEGAL FILE NAME***'
 999	RETURN1
	END

c
c  subroutine to output file header
c
	subroutine whead(idev,ifh)
	character headblk*2880,hdr(36)*80,itest1*7,idum*1
	equivalence (headblk(1:1),hdr(1)(1:1))
	ihb=1
c	  read(idev,rec=ihb,end=998) headblk
	  read(idev,rec=ihb) headblk
c write out the first 5 card
	  do 105 i=1,5
 105	    write(6,30) hdr(i)(1:79)
	  nhd=6
	  iwt=6
 106	  DO 110 i=nhd,36
            IF (hdr(i).EQ.' ') GO TO 110
            ITEST1=hdr(i)(1:7)
            IF (itest1(1:3).EQ.'OBJ')  then
	      if (ifh.eq.1) write(6,30) hdr(i)
	    else if (ITEST1(1:3).EQ.'BSC') then
	      if (ifh.eq.1) write(6,30) hdr(i)
            else if (ITEST1(1:3).EQ.'BZE') then
	      if (ifh.eq.1) write(6,30) hdr(i)
	    else if (itest1(1:6).eq.'FILTER') then
	      if (ifh.eq.1) write(6,30) hdr(i)
	    else if (itest1(4:7).eq.'TIME') then
	      if (ifh.eq.1) write(6,30) hdr(i)
	    else if (itest1(1:2).eq.'UT') then
	      if (ifh.eq.1) write(6,30) hdr(i)
	    else if (itest1(1:4).eq.'DATE') then
	      if (ifh.eq.1) write(6,30) hdr(i)
            else IF(ITEST1.EQ.'END') then
	      if (ifh.eq.2) then 
	        do 109 k=nhd,i
	          iwt=iwt+1
	          if (iwt.eq.21) then
 600	            write(6,'(''  ... <cr> to continue: '',$)')
	            read(5,'(a1)',end=600) idum
	            iwt=0
	          end if
 109	          write(6,30) hdr(k)(1:79)
	      end if
	      return
	    end if
 110      CONTINUE
C   "END" is not reached, read another header block
	 if (ifh.eq.2) then
	   do 112 k=nhd,36
	     iwt=iwt+1
	     if (iwt.eq.21) then
 601	       write(6,'(''  ... <cr> to continue: '',$)')
	       read(5,'(a1)',end=601) idum
	       iwt=0
	     end if
 112	     write(6,30) hdr(k)
	 end if
 30	format(1x,a79)
	ihb=ihb+1
c	read(idev,rec=ihb,end=998) headblk
	read(idev,rec=ihb) headblk
	nhd=1
	go to 106
 998	write(6,1) ihb-1
 1	format('  +++ error in header: no END card.',I4,
     *         ' header block read...')
	return
	end
