C//////////////////////////////////////////////////////////////
c  additional subroutines developed for RCS pipeline
c
c...............................................................
c  RCS2 pipeline command:
c     n input fits file names, n=number of filter, 1st name=primary filter
c
c     (1) perform object finding using weigthed average of specified 
c           fits images  (currently max of 2)
c     (2) photometry for each filter (.phm and .ast files)
c     (3) generate mspos2 .hdr, spawn mspos2 to generate .cat file
c     (4) repeat for next chip
c
c   ipar=1: object finding only
c   ipar=2: photometry only
c   ipar=3: object finding and photometry
c
c  nch1, nch2: chip number range; if 0, all chips
	subroutine rcspipe(*,nch1,nch2,ipar,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,hdrname*60,
     *            maskstr*60,cext*2,imdir*55,dumstr*8,errmsg*30
	integer ib(6),ie(6),ibm(6),iem(6),status,rmode
	integer ix(nu,nv,kf)
	include 'pppcore.inc'
	include 'ppparm.inc'
	include 'pipeparm.inc'
	common/imagedir/imdir
	common/bstarxy/nbstar,bstx(10),bsty(10),bsrad(10),bsmag(10)
c temporary set for pipeparm
c	iapmaxpipe=49
c	npadpipe=1
c	ismtypepipe=1
c  set ircspipe=1 when starting
	ircspipe=1
c for now, the primary filter is always the first one on the input string
	nfilref=1
c filestr is a string containing the root names of images for each filter
	write(6,1)
 1	format('  Enter fits file names (.fits.*/fz implicit), max 6:'
     *          /': ',$)
	read(5,'(a60)',end=999) filestr
c save the string in the common block for later use
	pfiltstr=filestr
c  parse the file string for getting names of the files
	call fstr(filestr,60,ib,ie,nfilt)
	if (nfilt.le.0) then
	  write(6,*) ' **error in image name string ***'
	  ircspipe=0
	  return1
	end if
c determine the number of chips	using the first image (all others
c    must have the same number of chips!)
	filen=filestr(ib(1):ie(1))
c   .hdr file name root: the patch name with ^cext added
c
c   the separation between filter and pointing name must be 
c     denoted by "_"
	do 161 k=1,lnblnk(filen)
	  if (filen(k:k).eq.'_') goto 162
 161	continue
 162	hdrname=filestr(ib(1):k-1)
	call imageex(filen,filenc,iimex)
	if (iimex.lt.0) then
	  ircspipe=0
	  return1
	end if
c  attempt to open file  (cfitsio rwmode: 0=read, 1=write)
	rmode=0
c	wmode=1
	status=0
	call ftgiou(lun,status)    
	call ftnopn(lun,imdir(1:lnblnk(imdir))//
     *      filenc(1:lnblnk(filenc))//'[0]',rmode, status)
	call ftgkyj(lun,'NEXTEND',nextendt,dumstr,status)
c  if not found, set nextend to 1
	if (status.ne.0) nextendt=1
	write(6,*) ' # of extension: ',nextendt,status
	call ftgerr(status,errmsg)
	call ftclos(lun, status)
	call ftfiou(lun, status)
	if (nch1.eq.0) then
	  jc1=1
	  jc2=nextendt
	else if (nch2.eq.0) then
	  jc1=nch1
	  jc2=nch1
	else
	  jc1=nch1
	  jc2=min0(nextendt,nch2)
	end if
	write(6,*) ' +++ doing chips ',jc1,' to ',jc2
	maskstr=' '
	write(6,2)
 2	format('  Enter corresponding mask fits file names:'
     *          /': ',$)
	read(5,'(a60)',end=999) maskstr
c parse the file names
	if (maskstr.ne.' ') then
	  call fstr(maskstr,80,ibm,iem,nfilm)
	  if (maskstr(ibm(1):iem(1)).eq.'auto') then
c for automatic mask creation
	    nfilm=-1
	  else 
	    if (nfilm.ne.nfilt) then
	      write(6,*) ' ** # of mask files not eq image file, aborted'
	      return1
	    end if
	    write(6,*) ' Images to be processed (image/mask file names):'
	    do 151 i=1,nfilt
	      write(6,3) i,filestr(ib(i):ie(i)),maskstr(ibm(i):iem(i))
 3	format(' image ',i2,' :  ',a24,2x,a24)
 151	    continue
	  end if
	else 
	  write(6,*) ' ++ no mask file names, assuming no bad pix masks'
	  nfilm=-1
	  write(6,*) ' Images to be processed (no masks images):'
	  do 152 i=1,nfilt
	    write(6,3) i,filestr(ib(i):ie(i))
 152	  continue
	end if
c  parameters:  All normal input parameters are passed to subroutines
c    using a common block: pipeparm.  The parameters must be set
c    before calling the pipeline command using: setp
c
c  object finding frames (need to make the read statement here
c     more intelligent so that 1 number can be entered
c   nobf1 and nobf2 are *filter* numbers, as ordered in filestr
	if (ipar.eq.1.or.ipar.eq.3) then
	  nobf1=1
	  nobf2=2
 601	  write(6,10) nobf1,nobf2
 10	format(' Enter 2 image (filter) numbers from list for ',
     *     'object finding'/ 
     *   '  (enter 0 for the 2nd one if not used) [',2i2,']: ',$)
	  call read2i(nobf1,nobf2,*999,*601)
	end if
c.........................
c   compute shifts between filters for different chips
c   if there is only one filter, then it is always nfilref (no need to shift)
	if (nfilt.gt.1) then
c  for object finding, and only uses one chip, then don''t do shift
	  if (ipar.eq.2.and.nobf2.eq.0) goto 701
c  normal mode: chip shifts computed before doing anything else
c for ichipsh 2 and 3, the shifts are done while doing photometry
	  if (ichipsh.eq.1) then
	    call mulshift(jc1,jc2,nextendt,                
     *            filestr,nfilm,ie,ib,iem,ibm,ix,nu,nv,kf)    
	  end if
	end if
 701	continue
c................................
c for each chip, do object finding and/or photometry
	do 100 jj=jc1,jc2
c extension string
	  istatus=encode(cext,float(jj),1,0,idum)
c  this is the parameter that is passed to the pipeparm parameter
	  jchip=jj
	  write(6,*) '   >>>>> Doing chip ',cext,' <<<<<<<'
c object finding
	  if (ipar.eq.1.or.ipar.eq.3) then
	    write(6,*) ' object finding for chip ',cext
            call pipeobjf(nobf1,nobf2,filestr,maskstr,
     *          nfilm,ib,ie,ibm,iem,ix,nu,nv,kf,cext)
	  end if
c photometry
	  if (ipar.eq.2.or.ipar.eq.3) then
	    write(6,*) ' photometry for chip ',cext
	    call pipephot(filestr,maskstr,nfilm,ib,ie,ibm,iem,
     *                    ix,nu,nv,kf,cext)
c make .hdr file 
	    call mkhdr(hdrname,cext)
c make msposin file (to be written)
c	    call mkmspbatch(hdrname,cext)
c spawn mspos2
c	    call system('nice '//pppath(1:lnblnk(pppath))//
c    *         '/mspos2 < '//mspos2in.fil//' > '//mspos2out.fil
	  end if
 100    continue
c old bad chipfix
        if (ichipsh.eq.3) then
	  if ((jc2-jc1).gt.2) then
	    write(6,*) '  +++ checking/redoing bad WCS chips ++++'
	    call photbadchips(jc1,jc2,
     *         filestr,maskstr,nfilm,ie,ib,iem,ibm,ix,nu,nv,kf)
	  else
	    write(6,*) '  +++ fewer than 3 chips, no checking',
     *          ' for redoing bad WCS chips '
	  end if
          return
	end if
	ircspipe=0
 999	return1
	end
c///////////////////////////////////////////////////////////////////////
c  subroutine to get key word values from the header
c
	subroutine getkeyval(*,nfr,kf)
	real*8 ratemp,dectemp
	character headbst*2880,headtem*2880,hdr(36)*80,temfil*20,
     *            headblk*2880,ncomc*36
	common/header/bzero,bscale,headblk(20),nhb
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	COMMON/PICC/ ncomc(15),fitsname(15)
c	include 'picdir.inc'
	include 'imagepar.inc'
	equivalence (headtem(1:1),hdr(1)(1:1))
	call QUERYNFR(NFR,KF,0,*999,*2001)
c go through header block to look for key words
 2001	continue
	if (nhbst(nfr).le.0) then
	  write(6,*) '  *** 0 header block for image, no parameters '
	  return1
	end if
c  reasonable default values, in case the don''t exit in header
	ncombine(nfr)=1
c	nextend(nfr)=1
c going through each card
	do 100 j=1,nhbst(nfr)
	  headtem=headbst(j,nfr)
c	if (j.eq.1) write(6,*) (hdr(kk),kk=1,36)
	  do 101 i=1,36 
	    if (hdr(i)(1:3).eq.'END') then
c header done
	      go to 998
	    else if (hdr(i)(1:5).eq.'BZERO') then
	      read(hdr(i)(11:30),*,err=999) bzest(nfr)
	      bzero=bzest(nfr)
	    else if (hdr(i)(1:5).eq.'BSCAL') then
	      read(hdr(i)(11:30),*,err=999) bscst(nfr)
	      bscale=bscst(nfr)
	    else if (hdr(i)(1:7).eq.'NEXTEND') then
	      read(hdr(i)(11:30),*,err=999) nextendh(nfr)
c	write(6,*) hdr(i)(1:30)
c	write(6,*) nextend(nfr)
	    else if (hdr(i)(1:7).eq.'FILTER ') then
	      read(hdr(i)(11:30),*) temfil
	      filter(nfr)=temfil(1:1)
	    else if (hdr(i)(1:6).eq.'OBJECT') then
	      read(hdr(i)(11:30),*) ncomc(nfr)
	    else if (hdr(i)(1:7).eq.'OBSTIME'.or.hdr(i)(1:2).eq.'UT'.
     *              or.hdr(i)(1:5).eq.'UTIME') then
	      read(hdr(i)(11:30),*) obstime(nfr)
	    else if (hdr(i)(1:8).eq.'DATE-OBS') then
	      read(hdr(i)(11:30),*) obsdate(nfr)
	    else if (hdr(i)(1:7).eq.'EXPTIME') then
	      read(hdr(i)(11:30),*,err=990) exptime(nfr)
            else if (hdr(i)(1:7).eq.'AIRMASS')  then
                read(hdr(i)(23:34),*,err=990) airmassobs(nfr)
            else if (hdr(i)(1:3).eq.'MJD') then
              read(hdr(i)(11:30),*,err=990) mjuldate(nfr)
            else if (hdr(i)(1:5).eq.'GAIN ') then
              read(hdr(i)(11:30),*,err=990) ccdgn(nfr)
            else if (hdr(i)(1:8).eq.'RDNOISE ') then
              read(hdr(i)(11:30),*,err=990) readnoise(nfr)
            else if (hdr(i)(1:8).eq.'SATURATE') then
              read(hdr(i)(11:30),*,err=990) saturate(nfr)
            else if (hdr(i)(1:8).eq.'NCOMBINE') then
              read(hdr(i)(11:30),*,err=990) ncombine(nfr)
            else if (hdr(i)(1:6).eq.'PIXSCA') then
              read(hdr(i)(11:30),*,err=990) pixscale(nfr)
            else if (hdr(i)(1:3).eq.'RA ') then
              read(hdr(i)(11:30),*) rastr(nfr)
            else if (hdr(i)(1:4).eq.'DEC ') then
              read(hdr(i)(11:30),*) decstr(nfr)
            else if (hdr(i)(1:5).eq.'EPOCH') then
              read(hdr(i)(20:30),*,err=990) epoch(nfr)
            else if (hdr(i)(1:8).eq.'LATITUDE') then
              read(hdr(i)(20:30),*,err=990) rlat(nfr)
            else if (hdr(i)(1:8).eq.'LONGITUD') then
              read(hdr(i)(20:30),*,err=990) rlong(nfr)
            else if (hdr(i)(1:6).eq.'CRPIX1') then
              read(hdr(i)(11:31),*,err=990) crpix1(nfr)
              iwcs=1
              write(6,*) '  wcs found;  CRPIX1 ',crpix1(nfr),iwcs
            else if (hdr(i)(1:6).eq.'CRPIX2') then
              read(hdr(i)(11:31),*,err=990) crpix2(nfr)
            else if (hdr(i)(1:6).eq.'CRVAL1') then
              read(hdr(i)(11:31),*,err=990) crval1(nfr)
            else if (hdr(i)(1:6).eq.'CRVAL2') then
              read(hdr(i)(11:31),*,err=990) crval2(nfr)
            else if (hdr(i)(1:5).eq.'CD1_1') then
              read(hdr(i)(11:31),*,err=990) cd1_1(nfr)
            else if (hdr(i)(1:5).eq.'CD1_2') then
              read(hdr(i)(11:31),*,err=990) cd1_2(nfr)
            else if (hdr(i)(1:5).eq.'CD2_1') then
              read(hdr(i)(11:31),*,err=990) cd2_1(nfr)
            else if (hdr(i)(1:5).eq.'CD2_2') then
              read(hdr(i)(11:31),*,err=990) cd2_2(nfr)
	    else if (hdr(i)(1:7).eq.'PHOT_C ') then
              read(hdr(i)(11:30),*,err=990) zeropt(nfr)
	    else if (hdr(i)(1:7).eq.'PHOT_X ') then
              read(hdr(i)(11:30),*,err=990) clrterm(nfr)
	    else if (hdr(i)(1:7).eq.'PHOT_K ') then
              read(hdr(i)(11:30),*,err=990) airmterm(nfr)
	    end if
	    goto 101
 990	    write(6,3) j,i,hdr(i)(1:10)
 3	format('  +++  header card read error, blk ',i2,' card ',
     *         i2,5x,a10)
 101	  continue
 100	continue
c if wcs exists, compute co-ordinate of chip center
 998	if (iwcs.eq.1) then
	  call astrom(nfr,1.0,1.0,ratemp,dectemp)
	  ra11(nfr)=ratemp
	  dec11(nfr)=dectemp
	write(6,*) ' RA Dec of (1,1) ',ra11(nfr),dec11(nfr)
	end if
 999	return1
	end 
c//////////////////////////////////////////////////////////////
c subroutine to type out fits header values (for checking)
c
	subroutine fitsvals(*,nfr,kf)
	character headbst*2880
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	include 'imagepar.inc'
	include 'picdir.inc'
	call QUERYNFR(NFR,KF,0,*999,*2001)
 2001	write(6,1) nii(nfr),njj(nfr),bzest(nfr),bscst(nfr)
     *     ,nextendh(nfr),obsdate(nfr)(1:12),
     *       obstime(nfr)(1:12),mjuldate(nfr),rastr(nfr)(1:12),
     *    decstr(nfr)(1:12),epoch(nfr),rlat(nfr),
     *    rlong(nfr),exptime(nfr),filter(nfr),airmassobs(nfr),
     *    pixscale(nfr),ccdgn(nfr),readnoise(nfr),saturate(nfr),iwcs
1	format(' Image size ',2i5,'  BZERO=',f10.2,'  BSCALE=',g13.6,
     *  '  N-chip: ',i2,/
     * ' obsdate: ',a12,'  obstime: ',a12,'  mJDate:',f15.7,/
     *   ' RA ',a12,' Dec ',a12,'  Epoch ',f6.1,/
     *   ' obs lat',f10.5,'   obs long ',f10.5,/' exp time(s)',
     *   f10.2,' filter: ',a1,'  Airmass',f5.2,'  pix scale',f7.4,
     *   /' gain',f6.3,'  rdnoise ',f5.2,'  saturation ',f6.0,
     *         '  wcs: ',i1)
	if (iwcs.gt.0) then
c output wcs paramters
	  write(6,2) crpix1(nfr),crpix2(nfr),crval1(nfr),crval2(nfr),
     *       cd1_1(nfr),cd1_2(nfr),cd2_1(nfr),cd2_2(nfr)
 2	format(' WCS parameters:  crpix 1 2:',2f12.5,/
     *         '                  crval 1 2:',2f12.7,/
     *            '        cd1_1,1_2,2_1,2_2:',4g13.5)
	  write(6,3) zeropt(nfr),airmterm(nfr),clrterm(nfr)
 3	format(' Photometric Calibration:',/
     *         '       A_0=',f7.3,'   A_airmass=',f7.3,
     *         '   A_colr=',f7.3)
	end if
 999	return1
	end
c////////////////////////////////////////////////////////////////////
c     driving subroutine to find bright stars on chip and produce
c     approx masks for haloes
	subroutine usnodri(*,nfr,kf,bsmag1,bsmag2)
	real rad,smag(100)
	double precision sra(100),sdec(100),ratem,dectem
	integer nstars,ii
	common/bstarxy/nbstar,bstx(10),bsty(10),bsrad(10),bsmag(10)
	include 'imagepar.inc'
	include 'picdir.inc'
	call QUERYNFR(NFR,KF,0,*999,*2001)
c search size detemine by size of image
 2001	rad=0.5*sqrt(1.*(nii(nfr)*nii(nfr)+njj(nfr)*njj(nfr)))
     *           *pixscale(nfr)/60.
	write(6,*) ' pix scale and search radius',pixscale(nfr),rad
c     ---- Find bright stars in and around frame
c first determine the ra and dec of the center of chip using wcs
	x0=0.5*float(nii(nfr))
	y0=0.5*float(njj(nfr))
	call astrom(nfr,x0,y0,ratem,dectem)
	write(6,*) ' field center ',x0,y0,'  RA,Dec ',ratem,dectem
c	rat2=ratem*15.
	rat2=ratem
	dect2=dectem
	call queryusnob1(rat2,dect2,rad,bsmag1,bsmag2,
     *         nstars,sra,sdec,smag)
      write(6,20) nstars,bsmag1,bsmag2
 20	format(i4,' bright stars between mags ',f6.2,' and ',f6.2)
	nbstar=0
	ns=0
	write(6,17)
 17	format('     List of stars within 30 pix of CCD chip'/ 
     *    ' #     RA         Dec      mag      Ipix    Jpix'
     *     ,' mask radius')
      do ii=1,nstars
c       write(6,15) ii,sra(ii),sdec(ii),smag(ii)
 15	format(i4,2f11.5,2x,f6.2,'  checking RA Dec')
	call xypixpos(nfr,xi,yj,sra(ii),sdec(ii))
	if (xi.gt.-30..and.xi.lt.float(nii(nfr))+30.) then
	  if (yj.gt.-30..and.yj.lt.float(njj(nfr))+30.) then
	    ns=ns+1
	    if (nbstar.le.9) then
	      nbstar=nbstar+1
	      bstx(nbstar)=xi
	      bsty(nbstar)=yj
	      bsmag(nbstar)=smag(ii)
c size of mask: this is just a fake calibration
	      if (smag(ii).le.6.) then
	        bsrad(nbstar)=1500.
	      else if (smag(ii).le.7.) then
	        bsrad(nbstar)=1000.
	      else if (smag(ii).le.8.) then
	        bsrad(nbstar)=650.
	      else if (smag(ii).le.9.) then
	        bsrad(nbstar)=350.
	      else if (smag(ii).le.10.) then
	        bsrad(nbstar)=200.
	      else if (smag(ii).le.11.) then
	        bsrad(nbstar)=110
	      else
	        bsrad(nbstar)=60
	      end if
              write(6,11) ns,sra(ii),sdec(ii),smag(ii),xi,yj,
     *                   bsrad(nbstar)
	    else
	      dumrad=0.0
              write(6,11) ns,sra(ii),sdec(ii),smag(ii),xi,yj,dumrad
 11	format(i3,2f11.6,f6.2,3x,2f7.1,2x,f6.0)
c save the result to the bright star array:
	    end if
	  else
	write(6,*) ' out of y bound'
            write(6,12) sra(ii),sdec(ii),smag(ii),xi,yj
	  end if
	else
          write(6,12) sra(ii),sdec(ii),smag(ii),xi,yj
 12	format(' 0 ',2f11.6,f6.2,3x,2f7.1,' not in chip')
	end if
      enddo
	if (nbstar.lt.ns) write(6,*) 
     *       ' too many bright stars, only first 10 saved'	  
 999	return1
	end
c////////////////////////////////////////////////////////////////////
c  modification of Gilbank''s subroutine for finding bright stars from USNO 
c   need to be connected to the network and running wget
c     ----------------------------------------------------------------
c
      subroutine queryusnob1(ra,dec,radius,m1,m2,nstars,sra,sdec,smag)
      character url*58,cra*8,cdec*8,cradius*5,query*119,junk*8,id*12
      character*133 ar
      real ra,dec,radius,smag(1000),m1,m2,junkreal,smagb(1000),
     *  rmag1,rmag2,bmag1,bmag2,fakemag(4)
      double precision sra(1000),sdec(1000)
      integer nstars
      integer stat,ii,ios,junkint
	nstars=0
c     Search within radius [arcmins] of ra,dec and mag limits: m1-->m2
c     return nstars, with coords sra,sdec and mags smag
c     return r_mag by default. if r is not present use b_mag
c     [no colour correction from b-->r, as yet]
c -- Take RA, Dec (in degrees), search radius and retrieve catalogue of USNO-B1 
c      objects in field
c     This will work on any PC running wget (fairly standard linux)
c     -- check for wget
      call system('wget --version >& /dev/null',stat)
      if(stat.ne.0) then 
        write(6,*) ' ++ you do not have wget installed, search aborted'
        nstars=0
        return
      endif
c     --
      write(cra,fmt='(f8.4)') ra
      write(cdec,fmt='(f8.4)') dec
      write(cradius,fmt='(f5.2)') radius
	write(6,*) ' RA: ',cra,'  Dec ',cdec,'  radius(arcmin): ',radius
	ibadweb=0
 300	continue
	if (ibadweb.eq.0) then
c     -- Build query (for CADC)
      url='http://vizier.hia.nrc.ca/viz-bin/asu-acl?-source=USNO-B1.0'
	else if (ibadweb.eq.1) then
      url='http://vizier.u-strasbg.fr/viz-bin/asu-acl?-source=USNO-B1.0'
	end if
        query=url//'&-c.ra='//cra//'&-c.dec='//cdec
     *            //'&-c.rm='//cradius//'&-out.max=10000'
c      print *,query
c     -- execute call to wget and place output table in /tmp/temp.TAB
       call system("wget '"//query//"' -O /tmp/temp.TAB")
c     --
c     -- Read in catalogue [header is junk. last line of junk starts '--------'
c        then format is ID, RA, Dec,  4*junk, R1mag, B2mag, R2mag, Imag
c
c     this is horrendous due to the formatting of USSO cat!!!!
c
c     need to apply magnitude cuts here
      junk='zzzzzzzz'
      open(unit=11,file='/tmp/temp.TAB',status='old',form='formatted')
c read to find the first line of data
      do while(junk.ne.'--------')
          read(unit=11,fmt=*) junk
c check if the website is working, error message is ****
	  if (junk(2:4).eq.'***') then
	      ibadweb=ibadweb+1
	      if (ibadweb.eq.1) then
c use UVIC mirrored site
	       write(6,*) ' +++ CADC network down, using Strasbg mirror'
	         goto 300
	      else
	         write(6,*) '  **** Both USNO servers down, should abort!!'
c no grace abort to get out from here yet...
	         return1
	      end if
	   end if
        enddo
c     now read real data
      ios=0
      ii=1
	iline=1
      id='XXXXXX'
      do while(ar(1:5).ne.'[EOD]')
         read(11,fmt='(A133)')ar
	 iline=iline+1
c         print *,ar
c         print *,ar(73:77),ar(79:83),ar(85:89),ar(91:95)
         read(ar,fmt='(A12,1X,F10.6,1X,F10.6)') id,sra(ii),sdec(ii)
c  read in all 4 mags. The problem with the format is that if the
c   last fields are blank, they are tabs rather space.  The following
c   reading algorithm should take care of it
	 ifm=0
	 do 120 kk=73,91,6
	   ifm=ifm+1
	   fakemag(ifm)=0.000
	   read(ar(kk:kk+4),33,err=120) fakemag(ifm)
 120	 continue
 33	format(f5.2)
c  assign r mag if exists, otherwise, b mag
	rmag1=fakemag(1)
	bmag1=fakemag(2)
	rmag2=fakemag(3)
	bmag2=fakemag(4)
c	write(6,*) ' line ',iline,rmag1,bmag1,rmag2,bmag1
	if (rmag1.gt.0.01) then
	  smag(ii)=rmag1
	else if (rmag2.gt.0.01) then
	  smag(ii)=rmag2
	else if (bmag1.gt.0.01) then
	  smag(ii)=bmag1
	else if (bmag2.gt.0.01) then
	  smag(ii)=bmag1
	end if
      if( (smag(ii).lt.m1).or.(smag(ii).gt.m2) ) then
         ii=ii-1
c         print *,'rejecting ',smag(ii)
      else
c         print *,ii,iline,' keeping ',smag(ii)
      endif 
      ii=ii+1
 890 	continue
      enddo
      nstars=ii-1
       do ii=1,nstars
          print *,ii,sra(ii),sdec(ii),smag(ii)
       enddo
c      print *,m1,m2,nstars
       close(unit=11)
	return
      end
c////////////////////////////////////////////////////////////////////
c  Gilbank''s subroutine for finding bright stars from USNO 
c   need to be connected to the network and running wget
c     ----------------------------------------------------------------
c
      subroutine queryusnob2(ra,dec,radius,m1,m2,nstars,sra,sdec,smag)
      character url*58,cra*8,cdec*8,cradius*5,query*119,junk*8,id*12
      character*133 ar
      real ra,dec,radius,smag(1000),m1,m2,junkreal,smagb(1000)
      double precision sra(1000),sdec(1000)
      integer nstars
      integer stat,ii,ios,junkint
	nstars=0
c     Search within radius [arcmins] of ra,dec and mag limits: m1-->m2
c     return nstars, with coords sra,sdec and mags smag
c     return r_mag by default. if r is not present use b_mag
c     [no colour correction from b-->r, as yet]
c -- Take RA, Dec (in degrees), search radius and retrieve catalogue of USNO-B1 
c      objects in field
c     This will work on any PC running wget (fairly standard linux)
c     -- check for wget
      call system('wget --version >& /dev/null',stat)
      if(stat.ne.0) then 
        write(6,*) ' ++ you do not have wget installed, search aborted'
        nstars=0
        return
      endif
c     --
c     -- Build query (for CADC)
      url='http://vizier.hia.nrc.ca/viz-bin/asu-acl?-source=USNO-B1.0'
      write(cra,fmt='(f8.4)') ra
      write(cdec,fmt='(f8.4)') dec
      write(cradius,fmt='(f5.2)') radius
	write(6,*) ' RA: ',cra,'  Dec ',cdec,'  radius(arcmin): ',radius
      query=url//'&-c.ra='//cra//'&-c.dec='//cdec//'&-c.rm='//cradius//
     *     '&-out.max=10000'
c      print *,query
c     -- execute call to wget and place output table in /tmp/temp.TAB
      call system("wget '"//query//"' -O /tmp/temp.TAB")
c     --
c     -- Read in catalogue [header is junk. last line of junk starts '--------'
c        then format is ID, RA, Dec,  4*junk, R1mag, B2mag, R2mag, Imag
c
c     this is horrendous due to the formatting of USSO cat!!!!
c
c     need to apply magnitude cuts here
      junk='zzzzzzzz'
      open(unit=11,file='/tmp/temp.TAB',status='old',form='formatted')
      do while(junk.ne.'--------')
         read(unit=11,fmt=*) junk
c         print *,junk
      enddo
c     now read real data
      ios=0
      ii=1
      id='XXXXXX'
      do while(ar(1:5).ne.'[EOD]')
         read(11,fmt='(A133)')ar
c         print *,ar
c         print *,ar(85:90), '---',ar(90:95)
         read(ar,fmt='(A12,1X,F10.6,1X,F10.6)') id,sra(ii),sdec(ii)
c         if(ar(85:85).eq.'1') print *,'not okay'
c     check for valid number in magnitude column
c       start from 85 NOT 86, as mags < 10. will have ' ' at 85!!!!
         if( (ar(86:86).eq.'1').or.
     *        (ar(86:86).eq.'2').or.(ar(86:86).eq.'3').or.
     *        (ar(86:86).eq.'4').or.(ar(86:86).eq.'5').or.
     *        (ar(86:86).eq.'6').or.(ar(86:86).eq.'7').or.
     *        (ar(86:86).eq.'8').or.(ar(86:86).eq.'9') )
     *    read(ar(85:90),fmt='(F5.2)') smagb(ii)
c         if(ar(85:86).ne.'') read(ar(85:90),fmt='(F5.2)') smagb(ii)
c         if (ar(90:95).ne.'') read(ar(90:95),fmt='(F5.2)') smag(ii)
         if( (ar(90:90).eq.'1').or.
     *        (ar(90:90).eq.'2').or.(ar(90:90).eq.'3').or.
     *        (ar(90:90).eq.'4').or.(ar(90:90).eq.'5').or.
     *        (ar(90:90).eq.'6').or.(ar(90:90).eq.'7').or.
     *        (ar(90:90).eq.'8').or.(ar(90:90).eq.'9') ) then
            read(ar(90:95),fmt='(F5.2)') smag(ii)
         else
            smag(ii)=smagb(ii)  ! **** r_mag not present, switch to b
         endif
c         if (ar(90:95).eq.'') then 
c            smag(ii)=smagb(ii)  ! **** r_mag not present, switch to b
c                                ! might want to do some average colour correction
c         endif
c            read(ar,fmt=22) id,sra(ii),sdec(ii),
c     *           junkreal,junkint,junkint,junkreal,junkint,junkint,
c     *       junkint,
c     *           smagb(ii)      !,smag(ii)
c            smag(ii)=smagb(ii) ! **** r_mag not present, switch to b
c                               ! might want to do some average colour correction
c         else
c            read(ar,fmt=23) id,sra(ii),sdec(ii),
c     *           junkreal,junkint,junkint,junkreal,junkint,junkint,
c     *           junkint,
c     *           smagb(ii),smag(ii)
c         endif
cc         print *,id,sra(ii),sdec(ii),smag(ii)
c
cc 22   format(A12,4x,F10.6,5x,F10.6,I3,I3,F6.1,I6,I6,I1,F5.2,F5.2)
c 22   format(
c     +        A12,1X,F10.6,1X,F10.6,1X,F5.2,1X,I3,1X,I3,1X,F6.1,1X,I6,
c     +        1X,I6,1X,I1,1X,F5.2)!,1X,F5.2)
c
c 23   format(
c     +        A12,1X,F10.6,1X,F10.6,1X,F5.2,1X,I3,1X,I3,1X,F6.1,1X,I6,
c     +        1X,I6,1X,I1,1X,F5.2,1X,F5.2)
c      print *,id,sra(ii),sdec(ii),
c     *        junkreal,junkint,junkint,junkreal,junkint,junkint
c     +  I1,1X,I3,1X,I3)
c     1X,I1,1X,I1,1X,I1,1X,A3,2X,F5.2,1X,I1,1X,I1,A1,
c     +  I3,1X,I2,1X,I4,I4,2X,F5.2,1X,I1,1X,I1,A1,I3,1X,I2,1X,I4,I4,2X,
c     +  F5.2,1X,I1,1X,I1,A1,I3,1X,I2,1X,I4,I4,2X,F5.2,1X,I1,1X,I1,A1,
c     +  I3,1X,I2,1X,I4,I4,2X,F5.2,1X,I1,1X,I1,A1,I3,1X,I2,1X,I4,I4)
      if( (smag(ii).lt.m1).or.(smag(ii).gt.m2) ) then
         ii=ii-1
c         print *,'rejecting ',smag(ii)
      else
          print *,ii,'keeping ',smag(ii)
      endif 
      ii=ii+1
      enddo
      nstars=ii-1
       do ii=1,nstars
          print *,sra(ii),sdec(ii),smag(ii)
       enddo
       print *,m1,m2,nstars
       close(unit=11)
	return
      end

c/////////////////////////////////////////////////////////
c  subroutine to pipeline object finding
c     done on extention cext
c     option of: summing 2 frames (nobf1 nobf2), 
c                 in the order of image names in filestr
c                mask frame names in maskstr
c
	subroutine pipeobjf(nobf1,nobf2,filestr,maskstr,
     *            nfilm,ib,ie,ibm,iem,ix,nu,nv,kf,cext)
	character filestr*60,maskstr*60,dumstr*60,maskname*60,cext*2
     *        ,filent*60,filen*24,primname*60
	dimension nfrar(5),ib(6),ie(6),iem(6),ibm(6)
	integer ix(nu,nv,kf)
	common/bstarxy/nbstar,bstx(10),bsty(10),bsrad(10),bsmag(10)
	include 'pppcore.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'ppparm.inc'
	include 'objcat.inc'
	include 'picdir.inc'
c prepare image
	filen=filestr(ib(nobf1):ie(nobf1))
	filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
c save the primary file name for making output names
	primname=filen(1:index(filen,'_')-1)
c  NOTE: nobf1 and nobf2 are the ordinal number of the files in the
c   filestr string; the images to be operated on in ppp is always
c    1 for obj finding, 2 for averaging, and 2 or 3 for masks
	call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,0)
c check if there really is data on the chip, by assume a chip
c  with all zeros is an empty chip
 800	call chzeros(*8001,*999,1,ix,nu,nv,kf,ichzero)
c  fill negatives
 8001	irplim=-2
	irprep=nint(saturate(1))+500
	icomp=0
	call smspars(1,filent)
	call getadusat(adusat,saturate(1),adusatpipe)

	write(6,*) '  ^^^^ rple lim/repl, adusat ',irplim,irprep,adusat
	call searchpix(*8002,1,icomp,' ',' ',1,ix,nu,nv,kf)
	write(6,*) ' ^^^^ randomize saturated pixels'
 8002	call fixsatpix(*801,1,isatpixpipe,ix,nu,nv,kf)
c
c  get mask for primary frame: read or make mask
 801	if (nfilm.gt.0) then
	  maskname=maskstr(ibm(nobf1):iem(nobf1))
 	  call getmasknam(nobf1,ibm,iem,maskstr,maskname)
 	  if (maskname.ne.' ') then
	    call rdkfits(*802,3,maskname,irep,ix,nu,nv,kf,ir,0)
c  no need to fix image!
 802	    write(6,*) '  ^^^ read in mask for primary image'
	  end if
	else if (nfilm.eq.-1) then
	  call createmask(ix(1,1,1),ix(1,1,3),nu,nv,
     *                    nii(1),njj(1),nii(3),njj(3),0)
c 	  call bpfix(*803 ,1,3,0,ix,nu,nv,kf)
 	  write(6,*) ' ^^^^ created mask from primary image '
	else if (nfilm.eq.0) then
c  read in dummy mask, called dummymask.fits
	  dumstr='dummymask'
	  call rdkfits(*803,3,dumstr,irep,ix,nu,nv,kf,ir,0)
 803	  write(6,*) '  ^^^ read in dummy mask for primary image'
	end if
c set mask frame to 3
	imaskpipe=3
c
c  find bright stars using star catalog (wcs solution from frame 1)
c      bsmag1 and bsmag2 are global ppparm set parameters.
c      The positions are stored and masks created when doing sky
 	call usnodri(*808,1,kf,bsmag1,bsmag2)
c	nbstar=0    ! temp disabling bright star search
c
c quick object finding for positional reference (qfnd) (frame 1)
c  used of averaging for obj finding, and also for shifting for photometry
 808	if( ichipsh.eq.2) then
	  write(6,*) ' ^^^ qfnd ','  jlpos ',jlpos,ilpos
	  call FINOBD(*809,1,dumstr,dumstr,0.00,7,IX,NU,NV,KF)
c  search for reference stars from the quickfind list, and store
 809	  write(6,*) ' ^^^ arf '
	  call autoref(1,ix,nu,nv,kf,0,*8091)
 8091	  continue
	end if
c
c  find saturated stars (using frame 1 only)
c   ++ this should be improved by having saturation masks from both frames
c    and then shift and sum
	call mkoutname(primname,1,cext,'ss')
	call mkoutcomm(primname,1,cext,'ss')
	dumstr=poutname(1:lnblnk(poutname))
c set thersky=-100 for saturated stars, save original value in tthresky
	tthresky=thresky
	thresky=-100.
c temporarily increase iir and ior
	iir=iir+4
	ior=ior+10
 	write(6,*) ' ^^^ finding saturated stars for obj finding mask --'
c  create a bleeding column mask if there are very bright stars (m<9.0)
c    added Dec 2005
	mkbsmask=0
	if (nbstar.gt.0) then
	  do 167 kj=1,nbstar
	    if (bsmag(kj).le.9.0) mkbsmask=1
 167	  continue
c make a quick and dirty bright star mask on frame 3
	  if (mkbsmask.eq.1) then
	    write(6,*) '  +++ very bright stars found --making mask'
	    call qkbsmask(nii(1),njj(1),3,cext,ix,nu,nv,kf)
c  fix the image before finding saturated stars
 	    call bpfix(*8092,1,3,0,ix,nu,nv,kf)
 8092	    continue
	  end if
	end if
 	call finobd(*810,1,dumstr,poutcomm,0.00,-2,IX,NU,NV,KF)
c reset values
 810	thresky=tthresky
	iir=iir-4
	ior=ior-10
c
c create average if nobf2=2, read image into 2
	if (nobf2.gt.0) then
	  icomp=0
	  filen=filestr(ib(nobf2):ie(nobf2))
	  filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	  call rdkfits(*804,2,filent,irep,ix,nu,nv,kf,ir,0)
 804	  irplim=-10
	  irprep=nint(saturate(2))
	  call smspars(2,filent)
 	  call searchpix(*805,2,icomp,' ',' ',1,ix,nu,nv,kf)
 805	  continue
c  shift 2nd image
	  if (ichipsh.eq.1) then
c  copy shift from computed shift array
	    dxmean=pmxshift(noblref,jchip)
	    dymean=pmyshift(noblref,jchip)
c computing matching transform between 1 and 2, using ref stars created
	  else if (ichipsh.eq.2) then
c  from the last call of autoref (stored in array).
c
c  first compute roughshift using wcs
 	  if (iwcssh.eq.1.and.iwcs.eq.1) then
	    call wcsshift(1,1,iwcshift,jwcshift,2)
	    iinitpipe=iwcshift
	    jinitpipe=jwcshift
	    write(6,14) iinitpipe,jinitpipe
 14	format('  ^^^ estimated shifts from wcs: ',2i5)
	  else
	    iinitpipe=0
	    jinitpipe=0
	  end if
	  write(6,*) '  ^^^^ computing shifts between filters 1 & 2'
          call simposmat(2,iir,iret,1,1,ix,nu,nv,kf,*811) 
	  end if
c  averaging two frames before object finding
 811	  nfrar(1)=1
	  nfrar(2)=2
c  result is overwritten in 1 (which will be used for object fin
	  nfrar(3)=1
	  nfrar(4)=0
	  nfrar(5)=0
	write(6,*) '  ^^^^ wtav, weighted (unfixed) average '
	  call combine2(*812,nfrar,ix,nu,nv,kf)
c
c  now make a combined frame 1 and frame 2 mask, frame 1 mask is still
c    at 3
c
c  case 1: read in mask
 812	  if (nfilm.gt.0) then
	    call getmasknam(nobf2,ibm,iem,maskstr,maskname)
	    write(6,*) ' ^^^ read in mask for frame 2: ',maskname
 	    call rdkfits(*806,2,maskname,irep,ix,nu,nv,kf,ir,0)
c shift mask and add to frame 3
 806	    call shiftp(*807,2,0.,0.,ix,nu,nv,kf,2)
 807	    call sumpic(*8071,2,3,3,0,ix,nu,nv,kf)
 8071	    continue
	  else if (nfilm.eq.-1) then
c  case 2: 
c shift image 2, and create mask by adding to frame 3 mask
	    call shiftp(*807,2,0.,0.,ix,nu,nv,kf,2)
 8072	    call createmask(ix(2,1,1),ix(1,1,3),nu,nv,
     *               nii(2),njj(2),nii(3),njj(3),1)
	  end if
c note that if dummy= mask, it is already read
	end if
c
c  make bleeding column mask, add to frame 3
	write(6,*) ' ^^^ adding bleeding column map to bp mask '
c  set adusat to .9 saturation to account partially for averaging 
c   note: bleedmap connects pixel> 1/2 (adusat-sky) + sky
	adusat=0.9*saturate(1)
	if (maskstr.ne.' ') then
c make mask on existing mask image
 	  call bleedmap(*813,1,3,ix,nu,nv,kf,0,0)
	else	
 	  call bleedmap(*813,1,3,ix,nu,nv,kf,0,1)
c fix the image
	end if
 813	call bpfix(*8131,1,3,0,ix,nu,nv,kf)
c  save average image to 2, for later cosmic ray rejection
 8131	write(6,*) '   ^^^ copying wtav image to frame 2 for save'
	call move(1,2,ix,nu,nv,kf)
c  do a tapered smoothing (note the ismtyepipe parameter is opposite
c   of the smooth ipar parameter, to keep consistency with iobjsm
	if (ismtypepipe.eq.1) then
	  write(6,*) '  ^^^^ taper smooth averaged image '
	  call smooth(*814,1,ismthlenp,0,ix,nu,nv,kf)
	else
	  write(6,*) '  ^^^^ box smooth averaged image '
	  call smooth(*814,1,ismthlenp,1,ix,nu,nv,kf)
	end if
c find object
 814	call mkoutname(primname,1,cext,'of')
	call mkoutcomm(primname,1,cext,'of')
	dumstr=poutname(1:lnblnk(poutname))
	write(6,*) '  ^^^^ final object finding '
	write(6,*) ' .. Out put name and comments: ',
     *               poutname(1:lnblnk(poutname))
	write(6,*) '        ',poutcomm(1:lnblnk(poutcomm))
	adusat=saturate(1)-500.
c still need to get the mask properly
 	call finobd(*817,1,dumstr,poutcomm,0.00,-1,IX,NU,NV,KF)
 817	write(6,*) '  ^^^^ copying original image and cr reject'
	call move(2,1,ix,nu,nv,kf)
	call rchkpos(*818,1,ix,nu,nv,kf)
c save final pos file
	call mkoutcomm(primname,1,cext,'cr')
 818	call ptype(*999,1,poutname,poutcomm,0,0,2,ix,nu,nv,kf)
c do cosmic ray reject
 999	return
	end
c/////////////////////////////////////////////////////////
c  subroutine to pipeline photometry
	subroutine pipephot(filestr,maskstr,nfilm,ib,ie,ibm,iem,
     *                    ix,nu,nv,kf,cext)
	integer nfra(5),ib(6),ie(6),ibm(6),iem(6)
	character filestr*60,maskstr*60,maskname*60,dumstr*60,
     *        filen*24,filent*60,cext*2,temname*60,primname*60,
     *        posfname*60,satposname*60
	integer ix(nu,nv,kf)
	include 'pppcore.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'picdir.inc'
	include 'objcat.inc'
	include 'ppparm.inc'
	nfra(1)=1
	dumstr='dummy'
	idum=0
	imaskpipe=2
	iastromtemp=iastrom
	read(cext,*,err=901) nchip
	do 110 i=2,5
	  nfra(i)=0
 110	continue
c do for each filter
	write(6,*)  ' IN PHOTPIPE # of filters: ',nfilt
	do 100, i=1,nfilt
	  write(6,*) ' @@@@ Photometry for chip ',cext,' filter ',i
	  filen=filestr(ib(i):ie(i))
c  primname is the root name of the pointing (without _ and chip number)
	  primname=filen(1:index(filen,'_')-1)
	  filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
c read image
	  write(6,*) '  ^^^^ reading image: ',filent
	  call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,0)
c check for .head files, which, if found, replaces the wcs solution
c   read from the .fits file
 800	  call chkwcshead(1,filent,cext)
	  call chzeros(*8001,*999,1,ix,nu,nv,kf,ichzero)
c read mask
 8001	  call getadusat(adusat,saturate(1),adusatpipe)
	  if (nfilm.ge.1) then
	    call getmasknam(i,ibm,iem,maskstr,maskname)
	    write(6,*) '  ^^^^ reading mask: ',maskname
	    call rdkfits(*801,2,maskname,irep,ix,nu,nv,kf,ir,0)
	  else if (nfilm.eq.0) then
	    dumstr='dummymask'
	    call rdkfits(*801,2,dumstr,irep,ix,nu,nv,kf,ir,0)
	  else if (nfilm.eq.-1) then
c create mask from image
	    call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	  write(6,*) ' ^^^^ created mask from image'
	  end if
c  fill zeros
 801	  irplim=-10
	  irprep=nint(saturate(1))
	  icomp=0
	  call searchpix(*802,1,icomp,' ',' ',1,ix,nu,nv,kf)
c  also create saturation and regular posfile name
 802	  call mkoutname(primname,1,cext,'ss')
	  satposname=poutname(1:lnblnk(poutname))
	  call mkoutname(primname,1,cext,'of')
	  posfname=poutname(1:lnblnk(poutname))
c  create shift 
          if (ichipsh.eq.1) then
c  use existing shift, no need for finding ref stars for nfilref
	    if (i.ne.nfilref) then
	      dxmean=pmxshift(i,jchip)
	      dymean=pmyshift(i,jchip)
	    end if
c	  else if (ichipsh.eq.2) then
c   for ichipsh=2, use wcs transformation; to be done after reading posfile
	  else if (ichipsh.eq.3) then
c shift is done for each filter/chip as photometry is done
            if (i.eq.nfilref) then
c create the ref position file (assume the ref image is always the 1st one
c   this uses the reference image and save reference objs positions
c    in common/refobj/. (No shift is done here)
 	      write(6,*) ' ^^^ qfnd for primary filter'
	      call FINOBD(*803,1,dumstr,dumstr,0.00,7,IX,NU,NV,KF)
 803	      write(6,*) ' ^^^ arf for position reference '
	      call autoref(1,ix,nu,nv,kf,0,*804)
 804	      continue
	    else
 	      write(6,*) ' ^^^ pos matching for filter',i
c apply wcs solution as an initial correction 
 	      if (iwcssh.eq.1.and.iwcs.eq.1) then
c the operating frame is always frame 1
	        call wcsshift(1,nfilref,iwcshift,jwcshift,2)
	        iinitpipe=iwcshift
	        jinitpipe=jwcshift
	        write(6,14) iinitpipe,jinitpipe
 14	format('  ^^^ estimated shifts from wcs: ',2i5)
	      else
	        iinitpipe=0
	        jinitpipe=0
	      end if
c  store the wcs shifts
	      xchshift(i,nchip)=float(iinitpipe)
	      ychshift(i,nchip)=float(jinitpipe)
c  determine shift between filter i and primary
	      call simposmat(1,iir,iret,1,1,ix,nu,nv,kf,*8041) 
c  the matching result parameter is stored
 8041	      imatch(i,nchip)=iret
c  Nov, 2005: determine if photometry should go ahead
	      if (imatch(i,nchip).lt.0) then
	        write(6,8042) nchip,i
 8042	format('   ***** WCS matching problem for Chip ',i2,
     *         ' filter ',i2,' skipped, to be re done')
	        goto 100
	      end if
	    end if
	  end if
c  make bleeding column mask, first read in saturation pos file
c     first make name, and save for other filters
c         use saturated pos file for reference filter, but total
c             pos file for others
	  if (i.eq.nfilref) then
 	    write(6,*) ' ^^^ reading saturated pos file ',satposname
 	    call rpof(*806,1,satposname,idum,0)
	  else
c  if not the primary file, shift the pos file
 	    write(6,*) ' ^^^ reading pos file for bleeding colm and phot' 
     *                    ,posfname
 	    call rpof(*805,1,posfname,idum,0)
 805	    continue
	    if (ichipsh.ne.3) then    
	      call pfshift(*806,1,0,0,dumstr,1)
	    else
c	      call wcsshift()
	    end if
	  end if
 806	  write(6,*) '  ^^^^ making bleeding mask, added to bp mask'
c first remove diffraction spike parameters
	  dsct1=dsc(1)
	  dsct2=dsc(2)
	  dsct3=dsc(3)
	  dsc(1)=-999
	  dsc(2)=0.
	  dsc(3)=0.
 	  call bleedmap(*807,1,2,ix,nu,nv,kf,0,0)
c fix bad column, both bad columns and bleed columns
 807	  call bpfix(*808,1,2,0,ix,nu,nv,kf)
 808	  dsc(1)=dsct1
	  dsc(2)=dsct2
	  dsc(3)=dsct3
c  read position file
	  if (i.eq.nfilref) then
	    write(6,*) ' ^^^ reading actual pos file ',posfname
 	    call rpof(*810,1,posfname,idum,0)
	  end if
c     do shift if needed  (dumstr here is just a dummy, can be anything)
c809      if (i.ne.nfilref) call pfshift(*810,1,0,0,dumstr,1)
c create cosmic ray map and fix
 810	write(6,*) '  ^^^  creating cosmic ray map and fix '
	  call cosraymap(*8101,1,2,' ',' ',nscrit,0,0,rwd,ix,nu,nv,kf)
c pad the CR map
 8101	  call padmap(*8102,2,ix,nu,nv,kf,0)
 8102	  call bpfix(*8103,1,2,0,ix,nu,nv,kf)
c create classification reference star file
c   first make name
 8103	  call mkoutname(primname,1,cext,'rf')
c  save the name for reference
	  arefname=poutname
	  write(6,*) '  ^^^^ aref: finding psfs'
	  adulowpipe=adulowfil(i)
 	  call autoref(1,ix,nu,nv,kf,1,*811)
c do photometry
c  create .phm file output name and comment
 811	  call mkoutname(primname,1,cext,'ph')
	  call mkoutcomm(primname,1,cext,'ph')
 	  temname=poutname(1:lnblnk(poutname))
	  write(6,*) '   ^^^^ running ppphot ',temname
	  if (i.eq.nfilref) then
c get astrometry
	    iastrom=iastromtemp
	  else
	    iastrom=-1
	  end if
	  call pppdri(*812,nfra,temname,poutcomm,0,ix,nu,nv,kf)
c save mspos hdr parameters
c812	  continue
 812	  call smspars(i,temname)
 100	continue
	iastrom=iastromtemp
 999	return
 901	write(6,*) ' ***chip number text read error ',cext,
     *             '  *** skipping to next chip'
	return
	end
c////////////////////////////////////////////////
c  driving routine to make a mask using subroutine createmask
c   ipar=0: create new mask frame
c        1: add to existing mask frame
c
	subroutine mkmask(*,nfr1,nfr2,ix,nu,nv,kf,ipar)
	include 'picdir.inc'
	integer ix(nu,nv,kf)
	call querynfr(nfr1,kf,0,*999,*2001)
 2001	call querynfr(nfr2,kf,4,*999,*2002)
 2002	call createmask(ix(1,1,nfr1),ix(1,1,nfr2),nu,nv,
     *         nii(nfr1),njj(nfr1),nii(nfr2),njj(nfr2),0)
	nfl(6,nfr2)=1
 999	return1
	end 
c//////////////////////////////////////////////////////////////
c  create mask from data by assuming zero pixels are bad pixels  
c
c   ipar=0: create new mask frame
c        1: add to existing mask frame
	subroutine createmask(ixpic,ixmask,nu,nv,niip,njjp,niim,
     *	                     njjm,ipar)
	integer ixpic(nu,nv),ixmask(nu,nv)
	if (ipar.eq.0) then
	  do 100 j=1,njjp
	    do 101 i=1,niip
	      if (ixpic(i,j).eq.0) then
	        ixmask(i,j)=1
	      else
	        ixmask(i,j)=0
	      end if
 101	    continue
 100	  continue
	  niim=niip
	  njjm=njjp
	else if (ipar.eq.1) then
	  if ((niim.ne.njjm).or.(niip.ne.njjp)) then
	    write(6,*) '   +++ warning: frame size not equal ',
     *               'image: ',niip,njjp,' mask: ',niim,njjm
	  end if
	  do 110 j=1,njjp
	    do 111 i=1,niip
	      if (ixpic(i,j).eq.0) then
	        ixmask(i,j)=1
	      end if
 111	    continue
 110	  continue
	end if
	return
	end
c//////////////////////////////////////////////////////////
c
c   subroutine to return output comment, based on root name
c
	subroutine mkoutcomm(root,nfr,cext,type)
	character root*60,cext*2,type*2 
	include 'pipeparm.inc'
	include 'imagepar.inc'
	poutcomm=root(1:lnblnk(root))//' chip '//cext(1:lnblnk(cext))
	if (type.eq.'ph') then
c  add filter name
	  poutcomm=poutcomm(1:lnblnk(poutcomm))//' '//filter(nfr)
	else if (type.eq.'ss') then
	  poutcomm=poutcomm(1:lnblnk(poutcomm))//' '//filter(nfr)//
     *        ' Saturated stars'
	else if (type.eq.'cr') then
	  poutcomm=poutcomm(1:lnblnk(poutcomm))//' '//filter(nfr)//
     *        ' CR reject'
	end if
c	write(6,*) '  mkoutcomm:  ',poutcomm(1:lnblnk(poutcomm))
c	write(6,*) '  filter   :  ',filter(nfr)
	return
	end
c//////////////////////////////////////////////////////////
c
c   subroutine to return output name, based on root name
	subroutine mkoutname(root,nfr,cext,type)
	character root*60,cext*2,type*2
	include 'pipeparm.inc'
	include 'imagepar.inc'
	poutname=root(1:lnblnk(root))//'^'//cext(1:lnblnk(cext))
	if (type.eq.'ss') then
c  add sat and filter name
	  poutname=poutname(1:lnblnk(poutname))//'sat'//filter(nfr)
	else if (type.eq.'rf'.or.type.eq.'ph') then
c  add filter name
	  poutname=poutname(1:lnblnk(poutname))//filter(nfr)
	end if
c	write(6,*) '  mkoutname:  ',poutname
c	write(6,*) '  filter   :  ',filter(nfr)
	return
	end
c////////////////////////////////////////////////////////////
c
	subroutine getmasknam(nob,ibm,iem,maskstr,maskname)
	character maskstr*60,maskname*60 
	integer ibm(6),iem(6)
	maskname=maskstr(ibm(nob):iem(nob))
	return
	end
c////////////////////////////////////////////////////////////
c  subroutine to set rcs pipeline parameters
c
c   ipar=0:  set default values
c   ipar=1:  set an individual parameter
c   ipar=2:  type out parameters
	subroutine setrcspipe(*,str,str1,nfr,rn,ipar)
	character str*60,str1*60,var*12
	include 'pipeparm.inc'
	if (ipar.eq.0.or.str.eq.'reset'.or.str.eq.'default') then
	  ircspipe=0
 	  irplim=0
 	  irprep=0
	  iinitpipe=0
	  jinitpipe=0
	  iwcssh=1
	  iaprefpipe=39
	  iapmaxpipe=39
	  cdiampipe=5.	
	  ishmaxpipe=2
	  iapmaxpipe=41
	  imaskpipe=2
	  npadpipe=1
	  ismtypepipe=0
	  ismthlenp=3
	  nsmthlenp=3
	  isbpipe=15
	  jsbpipe=15
	  crsigpipe=6.
	  ncrexcpipe=0
	  isatpixpipe=64800
	  adusatpipe=0.
	  colapert=3.0
	  ichipsh=1
	  do 121 i=1,5
	    adulowfil(i)=0.1
 121	  continue
	  return1
	else if (ipar.eq.2.or.str.eq.'list') then
	  write(6,1)
 1	format('  RCS Pipeline SET parameters (command setp): ')
	  write(6,401) ircspipe
 	  write(6,402) irplim
 	  write(6,403) irprep
	  write(6,404) iinitpipe
	  write(6,405) jinitpipe
	  write(6,4051) iwcssh
	  write(6,406) imaskpipe
	  write(6,407) npadpipe
	  write(6,408) ismtypepipe
	  write(6,4081) ismthlenp
	  write(6,409) iaprefpipe
	  write(6,410) iapmaxpipe
	  write(6,411) cdiampipe
	  write(6,412) ishmaxpipe
	  write(6,413) nsmthlenp
	  write(6,414) isbpipe
	  write(6,415) jsbpipe
	  write(6,416) crsigpipe
	  write(6,417) ncrexcpipe
	  write(6,418) isatpixpipe
	  write(6,420) (adulowfil(k),k=1,5)
	  write(6,421) colapert
	  write(6,422) ichipsh 
	  write(6,423) adusatpipe
	else if (ipar.eq.1) then
	  var=str(1:lnblnk(str))
	  if (var.eq.'irplim') then
	    if (str1.eq.' ') then
	      write(6,402) irplim
 302	      write(6,400)
	      call read1i(irplim,*999,*302)
	    else
	      irplim=nfr
	    end if
	    write(6,402) irplim
	  else if (var.eq.'irprep') then
	    if (str1.eq.' ') then
	      write(6,403) irprep
 303	      write(6,400)
	      call read1i(irprep,*999,*303)
	    else
	      irprep=nfr
	    end if
	    write(6,403) irprep
	  else if (var.eq.'iinitpipe') then
	    if (str1.eq.' ') then
	      write(6,404) iinitpipe
 304	      write(6,400)
	      call read1i(iinitpipe,*999,*304)
	    else
	      iinitpipe=nfr
	    end if
	    write(6,404) iinitpipe
	  else if (var.eq.'jinitpipe') then
	    if (str1.eq.' ') then
	      write(6,405) jinitpipe
 305	      write(6,400)
	      call read1i(jinitpipe,*999,*305)
	    else
	      jinitpipe=nfr
	    end if
	    write(6,405) jinitpipe
	  else if (var.eq.'iwcssh') then
	    if (str1.eq.' ') then
	      write(6,4051) iwcssh
 3051	      write(6,400)
	      call read1i(iwcssh,*999,*3051)
	    else
	      iwcssh=nfr
	    end if
	    write(6,4051) iwcssh
	  else if (var.eq.'imaskpipe') then
	    if (str1.eq.' ') then
	      write(6,406) imaskpipe
 306	      write(6,400)
	      call read1i(imaskpipe,*999,*306)
	    else
	      imaskpipe=nfr
	    end if
	    write(6,406) imaskpipe
	  else if (var.eq.'npadpipe') then
	    if (str1.eq.' ') then
	      write(6,407) npadpipe
 307	      write(6,400)
	      call read1i(npadpipe,*999,*307)
	    else
	      npadpipe=nfr
	    end if
	    write(6,407) npadpipe
	  else if (var.eq.'ismtypepipe') then
	    if (str1.eq.' ') then
	      write(6,408) ismtypepipe
 308	      write(6,400)
	      call read1i(ismtypepipe,*999,*308)
	    else
	      ismtypepipe=nfr
	    end if
	    write(6,408) ismtypepipe
	  else if (var.eq.'ismthlenp') then
	    if (str1.eq.' ') then
	      write(6,4081) ismthlenp
 3081	      write(6,400)
	      call read1i(ismthlenp,*999,*3081)
	    else
	      ismthlenp=nfr
	    end if
	    write(6,4081) ismthlenp
	  else if (var.eq.'iaprefpipe') then
	    if (str1.eq.' ') then
	      write(6,409) iaprefpipe
 309	      write(6,400)
	      call read1i(iaprefpipe,*999,*309)
	    else
	      iaprefpipe=nfr
	    end if
	    write(6,410) iaprefpipe
	  else if (var.eq.'iapmaxpipe') then
	    if (str1.eq.' ') then
	      write(6,410) iapmaxpipe
 310	      write(6,400)
	      call read1i(iapmaxpipe,*999,*310)
	    else
	      iapmaxpipe=nfr
	    end if
	    write(6,409) iapmaxpipe
	  else if (var.eq.'cdiampipe') then
	    if (str1.eq.' ') then
	      write(6,411) cdiampipe
 311	      write(6,400)
	      call read1r(cdiampipe,*999,*311)
	    else
	      cdiampipe=nfr
	    end if
	    write(6,411) cdiampipe
	  else if (var.eq.'ishmaxpipe') then
	    if (str1.eq.' ') then
	      write(6,412) ishmaxpipe
 312	      write(6,400)
	      call read1i(ishmaxpipe,*999,*312)
	    else
	      ishmaxpipe=nfr
	    end if
	    write(6,412) ishmaxpipe
	  else if (var.eq.'nsmthlenp') then
	    if (str1.eq.' ') then
	      write(6,413) nsmthlenp
 313	      write(6,400)
	      call read1i(nsmthlenp,*999,*313)
	    else
	      nsmthlenp=nfr
	    end if
	    write(6,413) nsmthlenp
	  else if (var.eq.'isbpipe') then
	    if (str1.eq.' ') then
	      write(6,414) isbpipe
 314	      write(6,400)
	      call read1i(isbpipe,*999,*314)
	    else
	      isbpipe=nfr
	    end if
	    write(6,414) isbpipe
	  else if (var.eq.'jsbpipe') then
	    if (str1.eq.' ') then
	      write(6,415) jsbpipe
 315	      write(6,400)
	      call read1i(jsbpipe,*999,*315)
	    else
	      jsbpipe=nfr
	    end if
	    write(6,415) jsbpipe
	  else if (var.eq.'crsigpipe') then
	    if (str1.eq.' ') then
	      write(6,416) crsigpipe
 316	      write(6,400)
	      call read1r(crsigpipe,*999,*316)
	    else
	      crsigpipe=rn
	    end if
	    write(6,416) crsigpipe
	  else if (var.eq.'ncrexcpipe') then
	    if (str1.eq.' ') then
	      write(6,417) ncrexcpipe
 317	      write(6,400)
	      call read1i(ncrexcpipe,*999,*317)
	    else
	      ncrexcpipe=nfr
	    end if
	    write(6,418) ncrexcpipe
	  else if (var.eq.'isatpixpipe') then
	    if (str1.eq.' ') then
	      write(6,418) isatpixpipe
 318	      write(6,400)
	      call read1i(isatpixpipe,*999,*318)
	    else
	      isatpixpipe=nfr
	    end if
	    write(6,418) isatpixpipe
	  else if (var.eq.'adulow1'.or.var.eq.'adulowfil1') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(1)
 319	      write(6,400)
	      call read1r(adulowfil(1),*999,*319)
	    else
	      adulowfil(1)=rn
	    end if
	    write(6,419) adulowfil(1)
	  else if (var.eq.'adulow2'.or.var.eq.'adulowfil2') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(2)
 320	      write(6,400)
	      call read1r(adulowfil(2),*999,*320)
	    else
	      adulowfil(2)=rn
	    end if
	    write(6,419) adulowfil(2)
	  else if (var.eq.'adulow3'.or.var.eq.'adulowfil3') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(3)
 321	      write(6,400)
	      call read1r(adulowfil(3),*999,*321)
	    else
	      adulowfil(3)=rn
	    end if
	    write(6,419) adulowfil(3)
	  else if (var.eq.'adulow4'.or.var.eq.'adulowfil4') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(4)
 322	      write(6,400)
	      call read1r(adulowfil(4),*999,*322)
	    else
	      adulowfil(4)=rn
	    end if
	    write(6,419) adulowfil(4)
	  else if (var.eq.'adulow5'.or.var.eq.'adulowfil5') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(5)
 323	      write(6,400)
	      call read1r(adulowfil(5),*999,*323)
	    else
	      adulowfil(5)=rn
	    end if
	    write(6,419) adulowfil(5)
	  else if (var.eq.'colapert') then
	    if (str1.eq.' ') then
	      write(6,421) colapert
 331	      write(6,400)
	      call read1r(colapert,*999,*331)
	    else
	      colapert=rn 
	    end if
	    write(6,421) colapert
	  else if (var.eq.'ichipsh') then
	    if (str1.eq.' ') then
	      write(6,422) ichipsh
 332	      write(6,400)
	      call read1i(ichipsh,*999,*332)
	    else
	      ichipsh=nfr
	    end if
	    write(6,422) ichipsh
	  else if (var.eq.'adusatpipe') then
	    if (str1.eq.' ') then
	      write(6,423) adusatpipe
 333	      write(6,400)
	      call read1r(adusatpipe,*999,*333)
	    else
	      adusatpipe=rn
	    end if
	    write(6,423) adusatpipe
	  else
	    write(6,*) '  *** no such rcspipe set parameter '
	  end if
	end if
 400	format(' Enter new value: ',$)
 401	format(' ircspipe:',i2,' - photometry pipeline switch ('
     *         ,'turned on when called')
 402	format(' irplim:',i5,' - lower limit for replacement for rple')
 403	format(' irprep:',i6,' - replacement value for rple; set to 0 ',
     *      'for auto (adusat)')
 404	format(' iinitpipe:',i3,' - initial i shift for auto matching')
 405	format(' jinitpipe:',i3,' - initial j shift for auto matching')
 4051	format(' iwcssh:',i2,' - switch for auto wcs rough shift (1=on)')
 406	format(' imaskpipe:',i2,' - mask frame number (likely hardwired)')
 407	format(' npadpipe: ',i2,' - padding layer for mask for padd')
 408	format(' ismtypepipe: ',i2,' - tsmo smoothing algorithm: 1=tapered, 
     *               0=box')
 4081	format(' ismthlenp: ',i2,' - tsmo smoothing length, 3 or 5')
 409	format(' iaprefpipe:',i3,' - max aperture for autoref phot')
 410	format(' iapmaxpipe:',i3,' - max aperture diameter for photometry')
 411	format(' ciadmpipe:',f5.1,' - diameter for centroiding routine')
 412	format(' ishmaxpipe:',i2,' - max pixel shift allowed for centroiding')
 413	format(' nsmthlenp:',i2,' - smoothing box size for obj find check')
 414	format(' isbpipe:',i2,' - box x size for median filtering for crm')
 415	format(' jsbpipe:',i2,' - box y size for median filtering for crm')
 416	format(' crsogpipe:',f6.2,' - sigma crit for crm')
 417	format(' ncrexcpipe:',i2,' - mask frame number for exclusion in crm')
 418	format(' isatpixpipe:',i6,' - saturated pix value to be replaced')
 419	format(' adulowfil(n):',f7.4,' - low adu criterion for filter')
 420	format(' adulowfil(1-5):',5f7.4,/
     *               '        low adu criterion for filters 1 to 5')
 421	format(' colapert:',f6.2,' - color aperture in " for mspos ')
 422	format(' ichipsh:',i2,' - algorithm for finding chip shifts'/
     *   ' 1: shifts computed/fixed first; 2: use WCS on pos file;/',
     *   '    3: shifts computed as needed')
 423	format(' adusatpipe:',f7.0,' - overide header saturation value',
     *         ' if set to negative')
 999	return1
	end
c/////////////////////////////////////////////////////////////////////
c  subroutine to save fits header info onto filinfo common block for mkhdr
c   the image is always image #1
c    jfil is the filter number
c
	subroutine smspars(jfil,phmnamein)
	character phmnamein*60
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'picdir.inc'
c 	write(6,*) ' filter ',jfil,nfilref
	phmname(jfil)=phmnamein(1:lnblnk(phmnamein)-4)
	fzpt(jfil)=zeropt(1)
	fcolterm(jfil)=clrterm(1)
	fgain(jfil)=ccdgn(1)
	fexptime(jfil)=exptime(1)
	fairmterm(jfil)=airmterm(1)
	fscale(jfil)=pixscale(1)
	frlat(jfil)=rlat(1)
	frlong(jfil)=rlong(1)
	ffiltn(jfil)=filter(1)
c  get obs date to the right format
        fobsdate(jfil)=obsdate(1)(9:10)//'/'//obsdate(1)(6:7)//
     *          '/'//obsdate(1)(3:4)
c  get obs time to the right format
c    determine if the hr is one or two digit
	do 120 i=1,20
	  if (obstime(1)(i:i).eq.':') goto 121
 120	continue
 121	continue
	if (i.eq.2) then 
	  fobstime(jfil)=' '//obstime(1)(1:4)
	else
	  fobstime(jfil)=obstime(1)(1:5)
	end if
c get observatory name based on longitude, reverse engineer the name
c	write(6,*) '  observatory longitude ',frlong(jfil)
	if (frlong(jfil).lt.-154..and.frlong(jfil).gt.-157.) then
	  fobser(jfil)='cfht'
	else if (frlong(jfil).gt.110..and.frlong(jfil).lt.113.) then
	  fobser(jfil)='kpno'
	else if (frlong(jfil).gt.70..and.frlong(jfil).lt.70.75) then
	  fobser(jfil)='lco'
	else if (frlong(jfil).gt.70.75.and.frlong(jfil).lt.71.00) then
	  fobser(jfil)='ctio'
	end if
c	if (fobser(jfil).eq.' ') fobser(jfil)='cfht'
c	write(6,*) ' observatory assigned: ',fobser(jfil)
	if (jfil.eq.nfilref) then
c  for filter 1, save the general pointing info
c   get RA and Dec string properly
	  frastr=rastr(1)
c replace : with blanks
	  do 110 i=1,lnblnk(frastr)
	    if (frastr(i:i).eq.':') frastr(i:i)=' '
 110	  continue
c check if hour is single digit, add 0 if it is
	  ntemp=index(frastr,' ')
	  if (ntemp.eq.2) frastr='0'//frastr
	  fdecstr=decstr(1)
	  do 111 i=1,lnblnk(fdecstr)
	    if (fdecstr(i:i).eq.':') fdecstr(i:i)=' '
 111	  continue
	end if
c copy wcs solution
	fcrpix1(jfil)=crpix1(1)
	fcrpix2(jfil)=crpix2(1)
	fcrval1(jfil)=crval1(1)
	fcrval2(jfil)=crval2(1)
	fcd1_1(jfil)=cd1_1(1)
	fcd1_2(jfil)=cd1_1(1)
	fcd2_1(jfil)=cd1_1(1)
	fcd2_2(jfil)=cd1_1(1)
	ifwcs=iwcs
	fra11(jfil)=ra11(1)
	fdec11(jfil)=dec11(1)
	return
	end
c////////////////////////////////////////////////////////////
c driving routine to make .hdr file, mspos2 batch jobs, 
c  ipar=0: make .hdr file 
c  ipar=1: also make msposin file 
c  ipar=2: run the msposin file
c    ix is dummy,  only for calling rdkfits
c
	subroutine msposdr(*,nch1,nch2,ipar,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,dumstr*8,imdir*55
c char variable declar''s are temp,  need to get rootnmae,cext
	character errmsg*30,rootname*60,cext*2,filent*60,primname*60
	integer status,ib(6),ie(6),rmode
	integer ix(nu,nv,kf)
	common/imagedir/imdir
	include 'pipeparm.inc'
	nfilref=1
 600	write(6,1) 
 1	format('  ++ warning: frame 1 will be overwritten'/
     *    '  Enter fits file names (.fits.*/fz implicit), max 6:'
     *          /': ',$)
	read(5,'(a60)',end=999) filestr
c parse file string
	call fstr(filestr,60,ib,ie,nfilt)
c determine the number of chips	using the first image (all others
c    must have the same number of chips!)
	filen=filestr(ib(1):ie(1))
	call imageex(filen,filenc,iimex)
	if (iimex.lt.0) then
	  ircspipe=0
	  return1
	end if
c  attempt to open file  (cfitsio rwmode: 0=read, 1=write)
	rmode=0
c	wmode=1
	status=0
	call ftgiou(lun,status)    
	call ftnopn(lun,imdir(1:lnblnk(imdir))//
     *      filenc(1:lnblnk(filenc))//'[0]',rmode, status)
	call ftgkyj(lun,'NEXTEND',nextendt,dumstr,status)
c  if not found, set nextend to 1
	if (status.ne.0) nextendt=1
	write(6,*) ' # of extension: ',nextendt,status
	call ftgerr(status,errmsg)
	call ftclos(lun, status)
	call ftfiou(lun, status)
	if (nch1.eq.0) then
	  jc1=1
	  jc2=nextendt
	else if (nch2.eq.0) then
	  jc1=nch1
	  jc2=nch1
	else
	  jc1=nch1
	  jc2=nch2
	end if
	rootname=filen(1:index(filen,'_')-1)
	do 100 jj=jc1,jc2
c extension string
	istatus=encode(cext,float(jj),1,0,idum)
c read header for each filter
	  do 101 i=1,nfilt
c make fits file name for chip jj, filter i
	    filen=filestr(ib(i):ie(i))
	    primname=filen(1:index(filen,'_')-1)
	    filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	    write(6,*) '  ^^^^ reading image: ',filent
	    call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,-1)
c save header block
 800	    call headtst(1)
c get values of key words from header
	    call getkeyval(*309,1,kf)
 309	    call mkoutname(primname,1,cext,'ph')
c *** need to figure out phmname
 	    call smspars(i,poutname)
 101	  continue
c make header 
	  call mkhdr(rootname,cext)
 100	continue
c 
 999	return1
	end 
c///////////////////////////////////////////////////////////////
c 
c  subroutine to make .hdr files
c    almost all information comes from .inc blocks
c
c    assignment of parameters  are made in smspar to the
c              common block filinfo for use in making header.
	subroutine mkhdr(rootname,cext)
	character filestr*60,rootname*60,hdrname*24,decsign*1,
     *   filterstr*20,cext*2,ncostr*1,vstring*16,fstr*60,
     *    nfitstr*1,nskipstr*2,nfilstr*1,fieldname*60,ds*1
 	character phmstr*60
	dimension jforder(6),jrefcol(6)
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'objcat.inc'
	include 'picdir.inc'
c temporarily fix the filter order (z,r,g), assuming input as r,z,g
	jforder(1)=2
	jforder(2)=1
	jforder(3)=3
	jrefcol(1)=2
	jrefcol(2)=1
	jrefcol(3)=2
	fieldname=rootname(1:min0(18,lnblnk(rootname)))//'_'//
     *        cext(1:lnblnk(cext))
	hdrname=fieldname(1:lnblnk(fieldname))//'.hdr'
	write(6,*) '  ++ making .hdr file: ',hdrname
	call opennew(29,1,28,hdrname,*999)
c write name
	write(29,1) fieldname
 1	format(1X,A16,6X,'CONTROL',16X,'// object name, z')
c   parse  RA string
c	write(6,*) frastr, fdecstr 
	itemp=0
	if (itemp.eq.0) goto 888
	n1=index(frastr,':')
	read(frastr(1:n1-1),*) irh
	n2=index(frastr(n1+1:lnblnk(frastr)),':')
	read(frastr(n1+1:n2-1),*) irm
	read(frastr(n2+1:lnblnk(frastr)),*) rs
c parse dec string
	ds=fdecstr(1:1)
	if (ds.ne.'-') ds='+'
	n1=index(fdecstr,':')
	read(fdecstr(1:n1-1),*) idd
	idd=iabs(idd)
	n2=index(fdecstr(n1+1:lnblnk(fdecstr)),':')
	read(fdecstr(n1+1:n2-1),*) idm
	read(fdecstr(n2+1:lnblnk(fdecstr)),*) ds
c  compute BII (temp)
	bii=90.
c write RA and dec
	write(29,2) irh,irm,rs,sd,idd,idm,ds,bii
 2	format(1X,I2,1X,I2,1X,F5.2,3X,A1,I2,1X,I2,1X,F5.2,10X,F5.1,4X,
     *          '// RA and DEC (2000), BII')
 888	continue
	aphat=fcrval1(1)
	call galactic1(fcrval1(1),fcrval2(1),reall,bii,0)
 	write(29,22) frastr(1:12),fdecstr(1:12),bii
 22	format(1X,a12,2X,a12,9x,f5.1,5x,
     *          '// RA and DEC (2000), BII')
c write filter number and names
        istat=encode(ncostr,float(nfilt),1,0,idum)
        istat=encode(vstring,float(42-nfilt*2),1,0,idum)
c	write(6,*) ncostr,vstring
        fstr='(1X,I1,2X,'//ncostr//'(A1,1X),'//
     *       vstring(1:lnblnk(vstring))//
     *       'X,''// # of colors and name(s)'')'
        WRITE(29,fstr) nfilt, (ffiltn(jforder(k)),K=1,nfilt)
c write .phm names for nfilt filters
	do 100 kk=1,nfilt
c take care of filter order
	  k=jforder(kk)
c	write(6,*) ' writing phm name ',k,phmname(k)
	  write(29,3) phmname(k),ffiltn(k)
 3	format(1X,A24,21x,'// data file name for ',A1)
 100	continue
c for each filter, write obs, date and integration time (2 lines)
	do 101 kk=1,nfilt
c	write(6,*) ' writing obs name ',k,fobser(k)
	  k=jforder(kk)
	  write(29,4) fobser(k),frlong(k),frlat(k), ffiltn(k)
 4	format(1X,A4,3X,F8.3,2X,F7.3,21X,'// Obs,longitude,'
     *          ,'latitude: ',a1)
c need to interpret obsdate, obstime ****
	  write(29,5) fobsdate(k)(1:8),fobstime(k)(1:5),
     *           fexptime(k),ffiltn(k)
 5	format(1X,A8,1x,a5,1x,f8.2,22x,
     *    '// UT date, time, & int time: ',a1)
 101	continue  
c default color aperture in arcsec
	write(29,6) colapert
 6	format(1X,f5.2,40X,'// default colour aperture in "')
c reference colors for nfilt (temp for 2 filters)
	istatus=encode(nfilstr,float(nfilt),1,0,idum)
c	nfilstr='2'
	ntem=46-nfilt*2
	istatus=encode(nskipstr,float(ntem),1,0,idum)
c	nskipstr='42'
	fstr='('//nfilstr//'i2,'//nskipstr//
     *    'x,''//ref color for'','//nfilstr//'(1x,a1))'
	write(29,fstr) (jrefcol(j),j=1,nfilt),
     *           (ffiltn(jforder(k)),k=1,nfilt)
c  zero pt, airmass and color term
	do 102 kk=1,nfilt
	  k=jforder(kk)
c currently set all the color terms to zero
	  fcolterm(k)=0.0
	  write(29,7) fzpt(k),fairmterm(k),fcolterm(k),ffiltn(k)
 7	format(3f7.3,25x,'// 0 pt, airmass & color term: ',a1)
 102	continue
c  if nfilt>2, write complementary terms
	if (nfilt.gt.2) then
	  do 103 kk=1,nfilt
	    k=jforder(kk)
c temporary fix
	    comzeropt=fzpt(jforder(jrefcol(kk)))
c 	    compcolor=-0.5*fcolterm(k)
	    compcolor=0.
	    write(29,9) comzeropt,compcolor,ffiltn(k)
 9	format(2f7.3,32x,'// compl 0 pt & color term: ',a1)
 103	  continue
	else if (nfilt.eq.1) then
c single filter, enter average color
	end if
c  scale,  gain and size
	do 104 kk=1,nfilt
	  k=jforder(kk)
	  write(29,8) fscale(k),fgain(k),ffiltn(k)
  8     format(1x,f7.5,1x,f5.2,32x,'//  "/pix and e-/ADU for ',a1)
 104    continue
c orientation now always fixed at 1, rot angle=0.
c  final version should try to read a .pos file to see the limits
c   otherwise use nii and njj
	itempp=0
        write(29,14) itempp,itemp,itempp,itemp
 14     FORMAT('     0.00',1x,' 1',1x,4i5,13X,'// rot:deg, (1,1):'
     *        ,'0=NW,1=ES, pix limits')
c center object is taken to be 1/2 of total object
c   this requires reading the .pos file if exists
c	ncenter=nint(0.5*nob(1))
	ncenter=-10
c  setting ncenter to <-1 will make mspos find the nearest object
c   to the center of the chip of filter 1 to be the center
        write(29,10) ncenter
 10     format(i6,40x,'// reference object number')
	close(unit=29)
 999	return
	end
c////////////////////////////////////////////////////////////////
c  subroutine to perturb the single value saturation pixels by +/- 10
c    to make allow object finding to deal with too many pixeles
c    having the same value
	subroutine fixsatpix(*,nfr,isatpix,ix,nu,nv,kf)
	integer ix(nu,nv,kf)
	include 'picdir.inc'
	call querynfr(nfr,kf,0,*999,*2000)
 2000	if (isatpix.le.0) then
 600	  write(6,1) isatpix
 1	format('  Enter top pixel value that needs to be replaced [',
     *         i6,']: ',$)
	  call read1i(isatpix,*999,*600)
	end if
	iseed=863 
	ifixed=0
	do 100 j=1,njj(nfr)
	  do 101 i=1,nii(nfr)
	    if (ix(i,j,nfr).ge.isatpix) then
c compute a random number with amplitude 10 adu
	      randn=gasdev(iseed)
c this is to prevent a really big change!
	      randn=amin1(1.,randn)
	      randn=amax1(-1.,randn)
	      ix(i,j,nfr)=ix(i,j,nfr)+NINT(10.*randn)
	      ifixed=ifixed+1
	    end if
 101	  continue
 100	continue
	write(6,*) '  ... number of pixels perturbed by +/-10: ',ifixed
	nfl(6,nfr)=1
 999	return1
	end 
c/////////////////////////////////////////////////////////////
c  subroutine to check if an image actually contain non-zero pixles
c    return by *, *, or ipar=0 for okay, ipar=1 bad image
c
	subroutine chzeros(*,*,nfr,ix,nu,nv,kf,ipar)
	integer ix(nu,nv,kf)
	include 'picdir.inc'
c   spot check mask to make sure it isn''t all zeros
 	nzero=0
	ipar=0
	DO 100 j=200,njj(nfr)-200,50
	  DO 101 i=200,nii(nfr)-200,50
	    if (ix(i,j,nfr).ne.0) goto 110
	    nzero=nzero+1
 101	  continue
 100	continue
c all checked pixels are zeros, not an image
	ipar=1
	write(6,1) nzero
 1	format(' **** zero check: all pixels checked (',i6,
     *         ') are zero.'/
     *  '   Assume this is not a proper image, process aborted.')
	return2
 110	return1
	return
	end
c////////////////////////////////////////////
c   subroutine to return the shift in pixel of 2 images , using wcs
c
c  ipar=1 use ppp buffer image headers  (not implemented yet)
c  ipar=2 use pipeline filter image info headers
c
c  nb is the current image (in ppp buffer), 
c  nf is the reference image:center RA and Dec already computed,
c     the data for this is common block fwcpar
c
c  - compute the pixel co-ordinate (in nf1) of (1,1) RA and Dec of nf2
c  - compute shifts needed in pixels to put center of nf1 onto the pixels
c         and store the result
c
c  NOTE this assumes the same pixel scale for nb and nf
	subroutine wcsshift(nb,nf,ishift,jshift,ipar)
	real*8 ratemp,dectemp
	include 'imagepar.inc'
	include 'pipeparm.inc'
	if (ipar.eq.1) then
	  write(6,*) '  ** using buffer frames not implemented yet *'
	  return
	else
c comptute the pixel co-ord in nb of the center RA and Dec of nf
c   the wcs solution is that of nf in the ppp buffer
	  ratemp=fra11(nf)*15.
	  dectemp=fdec11(nf)
	write(6,*) ' (1,1) RA, dec of ref',fra11(nf),fdec11(nf)
c  (1,1) in nf is equal to (xc,yc) in nb
	  call xypixpos(nb,xc,yc,ratemp,dectemp)
	write(6,*) ' xy of RA Dec in new frame ',xc,yc
	  ishift=nint(xc)
	  jshift=nint(yc)
	end if
	return
	end 
c//////////////////////////////////////////////////////////////
c  subroutine to redo chips with bad WCS shifts by using average
c    shift of the filter
c
c   *** need to deal with making headers
c
	subroutine photbadchips(jc1,jc2,
     *         filestr,maskstr,nfilm,ie,ib,iem,ibm,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,hdrname*60,
     *       maskstr*60,cext*2 
c	integer ib(6),ie(6),ibm(6),iem(6)
	integer ix(nu,nv,kf)
	dimension avxshif(10),avyshif(10),naver(10)
	include 'pppcore.inc'
	include 'ppparm.inc'
	include 'pipeparm.inc'
c  first check which are the bad chips and compute averages
	write(6,*) ' RRRR: rephotometry; checking chips',jc1,jc2
 	call avshifts(avxshif,avyshif,jc1,jc2,naver,irerun)
c  print out average for debugging
	do 100 i=nfilt
	  write(6,700) i,avxshif(i),avyshif(i),naver(i)
 700	format(' filter ',i2,' av x/y shifts ',2f7.2,' N =',i3)
	  if (naver(i).eq.0) then
	     write(6,*) ' **** no data for computing average shifts'
     *            ,', aborting re-photometry'
	     return
	  end if
 100	continue
c  
	if (irerun.eq.0) then
	  write(6,*)  '   --  no bad WCS shift; photometry done!'
	  return
	end if
c now redo bad chips
	do 110 i=1,nfilt
	  if (i.eq.nfilref) goto 110
	  do 111 j=jc1,jc2
	    if (imatch(i,j).le.-1) then
	      jchip=j
c redo the bad chip
	      istatus=encode(cext,float(j),1,0,idum)
	      call fixphotpipe(avxshift,avyshift,i,filestr,maskstr,
     *               nfilm,ix,nu,nv,kf,ie,ib,iem,ibm,cext)
	    end if
 111	  continue
 110	continue
	return
	end
c////////////////////////////////////////////////////////////////
c subroutine to average x y shifts 
c   the assumption is that the chips are read differently 
c     such that the first half of the chips are different from
c     the 2nd half (i.e., where [1,1] is).  Note that this seems 
c    to work for MegaCam, but CFH12k is not so simply dividied`
c 
 	subroutine avshifts2(avxshif1,avyshif1,
     *      avxshif2,avyshif2,jc1,jc2,naver1,naver2,nextendt)
c the running index in the array is for filter number
	dimension avxshif1(*),avyshif1(*),naver1(*),
     *            avxshif2(*),avyshif2(*),naver2(*)
	include 'pipeparm.inc'
	include 'imagepar.inc'
c  nsplit is where the chips are divided on their orientation
	nsplit=nint(nextendt*0.5)
	do 100 i=1,nfilt
c  don''t bother with the reference filter
	  if (i.eq.nfilref) goto 100
	  xshiftaver1=0.
	  yshiftaver1=0.
	  xshiftaver2=0.
	  yshiftaver2=0.
	  nav1=0
	  nav2=0
	  do 101 j=jc1,jc2
	    if (imatch(i,j).ge.1) then
	      if (j.le.nsplit) then
	        xshiftaver1=xshiftaver1+pmxshift(i,j)
	        yshiftaver1=yshiftaver1+pmyshift(i,j)
	        nav1=nav1+1
	      else
	        xshiftaver2=xshiftaver2+pmxshift(i,j)
	        yshiftaver2=yshiftaver2+pmyshift(i,j)
	        nav2=nav2+1
	      end if
c	    else if (imatch(i,j).le.-1) then
c  re running (for any one of the chips) required
c	      irerun=1
	    end if
 101	  continue
	  if (nav1.eq.0) then
	    avxshif1(i)=999.9
	    avyshif1(i)=999.9
	    write(6,*)' *** no data for computing aver shifts',i
	  else
	    avxshif1(i)=xshiftaver1/float(nav1)
	    avyshif1(i)=yshiftaver1/float(nav1)
	  end if
	  naver1(i)=nav1
	  if (nav2.eq.0) then
	    avxshif2(i)=999.9
	    avyshif2(i)=999.9
	    write(6,*)' *** no data for computing aver shifts',i
	  else
	    avxshif2(i)=xshiftaver2/float(nav2)
	    avyshif2(i)=yshiftaver2/float(nav2)
	  end if
	  naver2(i)=nav2
 100	continue
	return
	end
c////////////////////////////////////////////////////////////////
c subroutine to average x y shifts
c
        subroutine avshifts(avxshif,avyshif,jc1,jc2,naver,irerun)
        dimension avxshif(*),avyshif(*),naver(*)
        include 'pipeparm.inc'
        irerun=0
        do 100 i=1,nfilt
c  don''t bother with the reference filter
          if (i.eq.nfilref) goto 100
          xshiftaver=0.
          yshiftaver=0.
          naver(i)=0
          do 101 j=jc1,jc2
            if (imatch(i,j).ge.1) then
              xshiftaver=xshiftaver+xchshift(i,j)
              yshiftaver=yshiftaver+ychshift(i,j)
              naver(i)=naver(i)+1
            else if (imatch(i,j).le.-1) then
c  re running required
              irerun=1
            end if
 101      continue
          if (naver(i).eq.0) then
             avxshif(i)=999.9
             avyshif(i)=999.9
          if (naver(i).eq.0)
     *    write(6,*) ' **** no data for computing aver shifts',i
          else
            avxshif(i)=xshiftaver/float(naver(i))
            avyshif(i)=yshiftaver/float(naver(i))
          end if
 100    continue
        return
        end
c////////////////////////////////////////////////
c  subroutine to do bad chip j, filter i
c
	subroutine fixphotpipe(avxshift,avyshift,ifilter,
     *       filestr,maskstr,nfilm,ix,nu,nv,kf,ie,ib,iem,ibm,cext)
	integer ix(nu,nv,kf)
	integer nfra(5),ib(6),ie(6),ibm(6),iem(6)
	dimension avxshift(*),avyshift(*)
	character filestr*60,maskstr*60,maskname*60,dumstr*60,
     *        filen*24,filent*60,cext*2,temname*60,primname*60,
     *        posfname*60,satposname*60
	include 'pppcore.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'picdir.inc'
	include 'ppparm.inc'
	dumstr='dummy'
	idum=0
	imaskpipe=2
	iastromtemp=iastrom
	nfra(1)=1
	do 777 jj=2,5
	  nfra(jj)=0
 777	continue
	write(6,*) ' #### repeating photometry for chip ',
     *    cext,' filter ',ifilter
	read(cext,*,err=901) nchip
c  first refind the reference position stars from reference filter
c  (copy filter number to "i" to make copying from pipephot easier...)
	i=nfilref
	filen=filestr(ib(i):ie(i))
	primname=filen(1:index(filen,'_')-1)
	filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	call mkoutname(primname,1,cext,'of')
	posfname=poutname(1:lnblnk(poutname))
c read image
	write(6,*) '    +++ first: find pos ref from filter ',i
	write(6,*) '  #### reading image: ',filent
	call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,0)
 800	call chzeros(*8001,*999,1,ix,nu,nv,kf,ichzero)
c read mask
 8001	call getadusat(adusat,saturate(1),adusatpipe)
	if (nfilm.ge.1) then
	  call getmasknam(i,ibm,iem,maskstr,maskname)
	  write(6,*) '  ^^^^ reading mask: ',maskname
	  call rdkfits(*801,2,maskname,irep,ix,nu,nv,kf,ir,0)
	else if (nfilm.eq.0) then
	  dumstr='dummymask'
	  call rdkfits(*801,2,dumstr,irep,ix,nu,nv,kf,ir,0)
	else if (nfilm.eq.-1) then
c create mask from image
	  call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	  write(6,*) ' #### created mask from image'
	end if
c  fill zeros
 801	irplim=-10
	irprep=nint(saturate(1))
	icomp=0
	call searchpix(*802,1,icomp,' ',' ',1,ix,nu,nv,kf)
c create the ref position file (assume the ref image is always the 1st one
c   this uses the reference image and save reference objs positions
c    in common/refobj/. (No shift is done here)
 802	    write(6,*) ' ^^^ qfnd for primary filter'
	    call FINOBD(*803,1,dumstr,dumstr,0.00,7,IX,NU,NV,KF)
 803	    write(6,*) ' #### arf for position reference '
	    call autoref(1,ix,nu,nv,kf,0,*804)
c  now do the bad filter
c  (copy filter number to "i" to make copying from pipephot easier...)
 804	i=ifilter
c do for each filter
	write(6,*) '  ### now photometry for the bad filter +++ ',
     *            ' chip ',cext,' filter ',i
	filen=filestr(ib(i):ie(i))
	primname=filen(1:index(filen,'_')-1)
	filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
c read image
	write(6,*) '  ### reading image: ',filent
	call rdkfits(*890,1,filent,irep,ix,nu,nv,kf,ir,0)
 890	call chzeros(*8901,*999,1,ix,nu,nv,kf,ichzero)
c read mask
 8901	call getadusat(adusat,saturate(1),adusatpipe)
	if (nfilm.ge.1) then
	  call getmasknam(i,ibm,iem,maskstr,maskname)
	  write(6,*) '  ###  reading mask: ',maskname
	  call rdkfits(*891,2,maskname,irep,ix,nu,nv,kf,ir,0)
	else if (nfilm.eq.0) then
	  dumstr='dummymask'
	  call rdkfits(*891,2,dumstr,irep,ix,nu,nv,kf,ir,0)
	else if (nfilm.eq.-1) then
c create mask from image
	  call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	  write(6,*) ' ###  created mask from image'
	end if
c  fill zeros
 891	irplim=-10
	irprep=nint(saturate(1))
	icomp=0
	call searchpix(*892,1,icomp,' ',' ',1,ix,nu,nv,kf)
c  create shift :
 892  write(6,*) ' ### pos matching for filter',i
c  use initial shifts from average
	    iinitpipe=nint(avxshift(i))
	    jinitpipe=nint(avyshift(i))
	    write(6,14) iinitpipe,jinitpipe
 14	format('  ### shifts estimated from average: ',2i5)
c  determine shift between filter i and primary
	    call simposmat(1,iir,iret,1,1,ix,nu,nv,kf,*8041) 
 8041	    imatch(i,nchip)=iret
	    if (imatch(i,nchip).lt.0) then
	      write(6,8042) nchip,i
 8042	format('   ***** STILL matching problem for Chip ',i2,
     *         ' filter ',i2,' give up!!')
	      goto 100
	    end if
c  make bleeding column mask, first read in saturation pos file
c     first make name, and save for other filters
 	  write(6,*) ' ### reading saturated pos file ',satposname
 	  call rpof(*805,1,satposname,idum,0)
c  if not the primary file, shift the pos file
 805	  if (i.ne.nfilref) call pfshift(*806,1,0,0,dumstr,1)
 806	  write(6,*) '  ###  making bleeding mask, added to bp mask'
c first remove diffraction spike parameters
	  dsct1=dsc(1)
	  dsct2=dsc(2)
	  dsct3=dsc(3)
	  dsc(1)=-999
	  dsc(2)=0.
	  dsc(3)=0.
 	  call bleedmap(*807,1,2,ix,nu,nv,kf,0,0)
c fix bad column, both bad columns and bleed columns
 807	  call bpfix(*808,1,2,0,ix,nu,nv,kf)
 808	  dsc(1)=dsct1
	  dsc(2)=dsct2
	  dsc(3)=dsct3
c  read position file
	  write(6,*) ' ### reading actual pos file ',posfname
 	  call rpof(*809,1,posfname,idum,0)
c     do shift if needed  (dumstr here is just a dummy, can be anything)
 809      if (i.ne.nfilref) call pfshift(*810,1,0,0,dumstr,1)
c create cosmic ray map and fix
 810	write(6,*) '  ###  creating cosmic ray map and fix '
	  call cosraymap(*8101,1,2,' ',' ',nscrit,0,0,rwd,ix,nu,nv,kf)
c pad the CR map
 8101	  call padmap(*8102,2,ix,nu,nv,kf,0)
 8102	  call bpfix(*8103,1,2,0,ix,nu,nv,kf)
c create classification reference star file
c   first make name
 8103	  call mkoutname(primname,1,cext,'rf')
c  save the name for reference
	  arefname=poutname
	  write(6,*) '  ###  aref: finding psfs'
	  adulowpipe=adulowfil(i)
 	  call autoref(1,ix,nu,nv,kf,1,*811)
c do photometry
c  create .phm file output name and comment
 811	  call mkoutname(primname,1,cext,'ph')
	  call mkoutcomm(primname,1,cext,'ph')
 	  temname=poutname(1:lnblnk(poutname))
	  write(6,*) '   ###  running ppphot ',temname
	  if (i.eq.nfilref) then
c get astrometry
	    iastrom=iastromtemp
	  else
	    iastrom=-1
	  end if
	  call pppdri(*812,nfra,temname,poutcomm,0,ix,nu,nv,kf)
c save mspos hdr parameters
 812	  continue
c812	  call smspars(i,temname)
 100	continue
	iastrom=iastromtemp
 999	return
 901	write(6,*) ' ***chip number text read error ',cext,
     *             '  *** skipping to next chip'
	return
	end
c///////////////////////////////////////////////////////////////
c  quick and dirty bleeding map for very bright stars picked
c    up from USNO catalog
c
c  this simply makes a bleeding column through the whole vertical
c    direction of the chip of 21 pixels in width
c
	subroutine qkbsmask(ni,nj,nfr,cext,ix,nu,nv,kf)
	integer ix(nu,nv,kf)
	character cext*2
	include 'picdir.inc'
	common/bstarxy/nbstar,bstx(10),bsty(10),bsrad(10),bsmag(10)
	nii(nfr)=ni
	njj(nfr)=nj
	nfl(6,nfr)=1
c  chip number
	read(cext,*) nchip
c  fill array with 0''s
	do 100 j=1,nj
	  do 101 i=1,ni
	    ix(i,j,nfr)=0
 101	  continue
 100	continue
c for each bright star, make a column of 1''s 21 pixel wide, but
c  do not fill the +/- 15 pix around the bright star center.
c
c Mar 9, 2007
c  for special chip(s?) on megacam, the bleeding is in the
c   other direction and broader
	do 110 k=1,nbstar
	  if (bsmag(k).le.9.0) then
	    ibs=nint(bstx(k))
	    jbs=nint(bsty(k))
	    write(6,*) '   Bright star mag: ',bsmag(k),' at ',
     *                bstx(k),bsty(k)
	    if (nchip.eq.3.or.nchip.eq.34) then
	      if (bsmag(k).gt.5.5) then
	        write(6,*) '  +++ making bleeding columns i',
     *             ' 50pix thick, between',  ibs-500,ibs+500
	        write(6,*) '  ( leaving ',ibs-10,' to ',ibs+10,' free'
	        do 211 j=max(1,jbs-25),min(nj,jbs+25)
	           do 212 i=max(1,ibs-500),min(ni,ibs+500)
	            if (i.lt.ibs-10.or.i.gt.ibs+10) ix(i,j,nfr)=1
 212	          continue
 211	        continue
	      else
c  very bright star, do the mask across the whole chip
	        write(6,*) '  +++ making bleeding columns i ',
     *             ' 400 pix thick across whole chip'
	        write(6,*) '  ( leaving ',ibs-10,' to ',ibs+10,' free'
	        do 221 j=max(1,jbs-200),min(nj,jbs+200)
	           do 222 i=1,ni
	            if (i.lt.ibs-10.or.i.gt.ibs+10) ix(i,j,nfr)=1
 222	          continue
 221	        continue
	      end if
	    else
c regular chips, bleeding is up and down j direction
	    write(6,*) '  +++ making bleeding columns i between',
     *               ibs-10,ibs+10
	    write(6,*) '  ( leaving ',jbs-15,' to ',jbs+15,' free'
	      do 111 j=1,nj
	        if (j.le.jbs-15.or.j.ge.jbs+15) then
	          do 112 i=max(1,ibs-10),min(ni,ibs+10)
	            ix(i,j,nfr)=1
 112	          continue
	        end if
 111	      continue
	    end if
	  end if
 110	continue
	return
	end
c////////////////////////////////////////////////
c  subroutine to fix adusat to be slightly below the saturation
c    value given in a  fits header, 
c
	subroutine getadusat(adusat,saturate,adusatpipe)
c  over ride fits header saturate parameter if adusatpipe is negative 
	if (adusatpipe.lt.0.) then
	  adusat=abs(adusatpipe)
	  return
	end if
	if (saturate.gt.0) then
	  if (saturate.lt.33000.) then
	    adusat=saturate-500
	  else if (saturate.lt.50000.) then
	    adusat=saturate-1000.
c	  else if (saturate.lt.60000.) then
c	    adusat=saturate-2000.
	  else 
	    adusat=saturate-2000.
	  end if
	else
c  if no value saturate is given, assume short integer saturation
	  adusat=32000.
	end if
	return
	end
c///////////////////////////////////////////////////////
c subroutine to update wcs solution from the .fits file
c   using .head file which has a more accurrate wcs solution
	subroutine chkwcshead(nfr,filent,cext)
	logical ex
	character filent*60,cext*2,headname*60,filenthead*60
	filenthead=filent(1:index(filent,'.')-1)
 	headname=filenthead(1:lnblnk(filenthead))//'_'//
     *      cext(1:lnblnk(cext))//'.head'
c    first check for the existence of .head file, if not found,
c          check for _quick_head file
c   
c	write(6,*) ' ... searching for ',headname
	inquire(file=headname,exist=ex)
	if (ex.eqv..true.) then
c read wcs from the .head file
	  write(6,*) ' --- reading replacement wcs header ',
     *         headname
	  call readheadwcs(nfr,headname)
	else
c check for quick head file
	  headname=filenthead(1:lnblnk(filenthead))//'_'//
     *             cext(1:lnblnk(cext))//'qk.head'
c	write(6,*) ' ... searching for ',headname
	  inquire(file=headname,exist=ex)
	  if (ex.eqv..true.) then
	    write(6,*) ' --- reading quick wcs header ',headname
	    call readheadwcs(nfr,headname)
	  end if
	end if
	return
	end
c///////////////////////////////////////////
c subroutine to read wcs from .head file
c
	subroutine readheadwcs(nfr,headname)
	character headname*60,hdr*80
	include 'imagepar.inc'
	open(unit=41,file=headname,status='old',err=999)
 100	read(41,'(a80)',end=998) hdr
        if (hdr(1:6).eq.'CRPIX1') then
          read(hdr(11:31),*,err=990) crpix1(nfr)
          iwcs=1
          write(6,*) ' .head wcs found;  CRPIX1 ',crpix1(nfr),iwcs
        else if (hdr(1:6).eq.'CRPIX2') then
          read(hdr(11:31),*,err=990) crpix2(nfr)
        else if (hdr(1:6).eq.'CRVAL1') then
          read(hdr(11:31),*,err=990) crval1(nfr)
        else if (hdr(1:6).eq.'CRVAL2') then
          read(hdr(11:31),*,err=990) crval2(nfr)
        else if (hdr(1:5).eq.'CD1_1') then
          read(hdr(11:31),*,err=990) cd1_1(nfr)
        else if (hdr(1:5).eq.'CD1_2') then
          read(hdr(11:31),*,err=990) cd1_2(nfr)
        else if (hdr(1:5).eq.'CD2_1') then
          read(hdr(11:31),*,err=990) cd2_1(nfr)
        else if (hdr(1:5).eq.'CD2_2') then
          read(hdr(11:31),*,err=990) cd2_2(nfr)
	end if
	goto 100
 999	write(6,*) '  *** .head file open error ***'
	return
 990	write(6,*) ' *** header reading error, at ',hdr(11:31)
 998	return
	end
