C//////////////////////////////////////////////////////////////
c  additional subroutines developed for RCS pipeline
c
c...............................................................
c  RCS2 pipeline command:
c     n input fits file names, n=number of filter, 1st name=primary filter
c
c     (1) perform object finding using weigthed average of specified 
c           fits images  (currently max of 2)
c     (2) photometry for each filter (.phm and .ast files)
c     (3) generate mspos2 .hdr, spawn mspos2 to generate .cat file
c     (4) repeat for next chip
c
c   ipar=1: object finding only
c   ipar=2: photometry only
c   ipar=3: object finding and photometry
c
c  nc1, nc2: chip number range; if 0, all chips
	subroutine rcspipe(*,nch1,nch2,ipar,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,hdrname*60,
     *            maskstr*60,cext*2,imdir*55,dumstr*8,errmsg*30
	integer ib(6),ie(6),ibm(6),iem(6),status,rmode
	integer ix(nu,nv,kf)
	include 'pppcore.inc'
	include 'ppparm.inc'
	include 'pipeparm.inc'
	common/imagedir/imdir
	common/bstarxy/nbstar,bstx(10),bsty(10),bsrad(10),bsmag(10)
c temporary set for pipeparm
c	iapmaxpipe=49
c	npadpipe=1
c	iinitpipe=0
c	jinitpipe=0
c	ismtypepipe=1
c  set ircspipe=1 when starting
	ircspipe=1
c for now, the primary filter is always the first one on the input string
	nfilref=1
c filestr is a string containing the root names of images for each filter
	write(6,1)
 1	format('  Enter fits file names (.fits.*/fz implicit), max 6:'
     *          /': ',$)
	read(5,'(a60)',end=999) filestr
c save the string in the common block for later use
	pfiltstr=filestr
c  parse the file string for getting names of the files
	call fstr(filestr,60,ib,ie,nfilt)
	if (nfilt.le.0) then
	  write(6,*) ' **error in image name string ***'
	  ircspipe=0
	  return1
	end if
c determine the number of chips	using the first image (all others
c    must have the same number of chips!)
	filen=filestr(ib(1):ie(1))
c   .hdr file name root: the patch name with ^cext added
	do 161 k=1,lnblnk(filen)
	  if (filen(k:k).eq.'_') goto 162
 161	continue
 162	hdrname=filestr(ib(1):k-1)
	call imageex(filen,filenc,iimex)
	if (iimex.lt.0) then
	  ircspipe=0
	  return1
	end if
c  attempt to open file  (cfitsio rwmode: 0=read, 1=write)
	rmode=0
c	wmode=1
	status=0
	call ftgiou(lun,status)    
	call ftnopn(lun,imdir(1:lnblnk(imdir))//
     *      filenc(1:lnblnk(filenc))//'[0]',rmode, status)
	call ftgkyj(lun,'NEXTEND',nextendt,dumstr,status)
c  if not found, set nextend to 1
	if (status.ne.0) nextendt=1
	write(6,*) ' # of extension: ',nextendt,status
	call ftgerr(status,errmsg)
	call ftclos(lun, status)
	call ftfiou(lun, status)
	if (nch1.eq.0) then
	  jc1=1
	  jc2=nextendt
	else if (nch2.eq.0) then
	  jc1=nch1
	  jc2=nch1
	else
	  jc1=nch1
	  jc2=nch2
	end if

	write(6,*) ' +++ doing chips ',jc1,' to ',jc2
	maskstr=' '
	write(6,2)
 2	format('  Enter corresponding mask fits file names:'
     *          /': ',$)
	read(5,'(a60)',end=999) maskstr
c parse the file names
	if (maskstr.ne.' ') then
	  call fstr(maskstr,80,ibm,iem,nfilm)
	  if (maskstr(ibm(1):iem(1)).eq.'auto') then
c for automatic mask creation
	    nfilm=-1
	  else 
	    if (nfilm.ne.nfilt) then
	      write(6,*) ' ** # of mask files not eq image file, aborted'
	      return1
	    end if
	    write(6,*) ' Images to be processed (image/mask file names):'
	    do 151 i=1,nfilt
	      write(6,3) i,filestr(ib(i):ie(i)),maskstr(ibm(i):iem(i))
 3	format(' image ',i2,' :  ',a24,2x,a24)
 151	    continue
	  end if
	else 
	  write(6,*) ' ++ no mask file names, assuming no bad pix masks'
	  nfilm=-1
	  write(6,*) ' Images to be processed (no masks images):'
	  do 152 i=1,nfilt
	    write(6,3) i,filestr(ib(i):ie(i))
 152	  continue
	end if
c  parameters:  All normal input parameters are passed to subroutines
c    using a common block: pipeparm.  The parameters must be set
c    before calling the pipeline command using: setp
c
c  object finding frames (need to make the read statement here
c     more intelligent so that 1 number can be entered
	if (ipar.eq.1.or.ipar.eq.3) then
	  nobf1=1
	  nobf2=2
 601	  write(6,10) nobf1,nobf2
 10	format(' Enter 2 image numbers from list for ',
     *     'object finding  (0 for not used) [',2i2,']: ',$)
	  call read2i(nobf1,nobf2,*999,*601)
	end if
c for each chip, do object finding
	do 100 jj=jc1,jc2
c extension string
	  istatus=encode(cext,float(jj),1,0,idum)
c  this is the parameter that is passed to the pipeparm parameter
	  jchip=jj
	  write(6,*) '   >>>>> Doing chip ',cext,' <<<<<<<'
c object finding
	  if (ipar.eq.1.or.ipar.eq.3) then
	    write(6,*) ' object finding for chip ',cext
            call pipeobjf(nobf1,nobf2,filestr,maskstr,
     *          nfilm,ib,ie,ibm,iem,ix,nu,nv,kf,cext)
	  end if
c photometry
	  if (ipar.eq.2.or.ipar.eq.3) then
	    write(6,*) ' photometry for chip ',cext
	    call pipephot(filestr,maskstr,nfilm,ib,ie,ibm,iem,
     *                    ix,nu,nv,kf,cext)
c make .hdr file 
	    call mkhdr(hdrname,cext)
c make msposin file
c	    call mkmspbatch(hdrname,cext)
c spawn mspos2
c	    call system('nice '//pppath(1:lnblnk(pppath))//
c    *         '/mspos2 < '//mspos2in.fil//' > '//mspos2out.fil
	  end if
 100    continue
	if ((jc2-jc1).gt.2) then
	  write(6,*) '  +++ checking/redoing bad WCS chips ++++'
	  call photbadchips(jc1,jc2,
     *         filestr,maskstr,nfilm,ie,ib,iem,ibm,ix,nu,nv,kf)
	else
	  write(6,*) '  +++ fewer than 3 chips, no checking',
     *          ' for redoing bad WCS chips '
	end if
	ircspipe=0
 999	return1
	end
c/////////////////////////////////////////////////////////
c  subroutine to pipeline photometry
	subroutine pipephot(filestr,maskstr,nfilm,ib,ie,ibm,iem,
     *                    ix,nu,nv,kf,cext)
	integer nfra(5),ib(6),ie(6),ibm(6),iem(6)
	character filestr*60,maskstr*60,maskname*60,dumstr*60,
     *        filen*24,filent*60,cext*2,temname*60,primname*60,
     *        posfname*60,satposname*60
	integer ix(nu,nv,kf)
	include 'pppcore.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'picdir.inc'
	include 'ppparm.inc'
	nfra(1)=1
	dumstr='dummy'
	idum=0
	imaskpipe=2
	iastromtemp=iastrom
	read(cext,*,err=901) nchip
	do 110 i=2,5
	  nfra(i)=0
 110	continue
c do for each filter
	write(6,*)  ' IN PHOTPIPE # of filters: ',nfilt
	do 100, i=1,nfilt
	  write(6,*) ' @@@@ Photometry for chip ',cext,' filter ',i
	  filen=filestr(ib(i):ie(i))
	  primname=filen(1:index(filen,'_')-1)
	  filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
c read image
	  write(6,*) '  ^^^^ reading image: ',filent
	  call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,0)
 800	  call chzeros(*8001,*999,1,ix,nu,nv,kf,ichzero)
c read mask
 8001	  adusat=saturate(1)-1000.
	  if (nfilm.ge.1) then
	    call getmasknam(i,ibm,iem,maskstr,maskname)
	    write(6,*) '  ^^^^ reading mask: ',maskname
	    call rdkfits(*801,2,maskname,irep,ix,nu,nv,kf,ir,0)
	  else if (nfilm.eq.0) then
	    dumstr='dummymask'
	    call rdkfits(*801,2,dumstr,irep,ix,nu,nv,kf,ir,0)
	  else if (nfilm.eq.-1) then
c create mask from image
	    call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	  write(6,*) ' ^^^^ created mask from image'
	  end if
c  fill zeros
 801	  irplim=-10
	  irprep=nint(saturate(1))
	  icomp=0
	  call searchpix(*802,1,icomp,' ',' ',1,ix,nu,nv,kf)
c  create shift 
 802	  if (i.eq.nfilref) then
c create the ref position file (assume the ref image is always the 1st one
c   this uses the reference image and save reference objs positions
c    in common/refobj/. (No shift is done here)
 	    write(6,*) ' ^^^ qfnd for primary filter'
	    call FINOBD(*803,1,dumstr,dumstr,0.00,7,IX,NU,NV,KF)
 803	    write(6,*) ' ^^^ arf for position reference '
	    call autoref(1,ix,nu,nv,kf,0,*804)
c  also create saturation and regular posfile name
 804	    call mkoutname(primname,1,cext,'ss')
	    satposname=poutname(1:lnblnk(poutname))
	    call mkoutname(primname,1,cext,'of')
	    posfname=poutname(1:lnblnk(poutname))
	  else
 	    write(6,*) ' ^^^ pos matching for filter',i
c apply wcs solution as an initial correction 
 	    if (iwcssh.eq.1.and.iwcs.eq.1) then
c the operating frame is always frame 1
	      call wcsshift(1,nfilref,iwcshift,jwcshift,2)
	      iinitpipe=iwcshift
	      jinitpipe=jwcshift
	      write(6,14) iinitpipe,jinitpipe
 14	format('  ^^^ estimated shifts from wcs: ',2i5)
	    else
	      iinitpipe=0
	      jinitpipe=0
	    end if
c  store the wcs shifts
	    xchshift(i,nchip)=float(iinitpipe)
	    ychshift(i,nchip)=float(jinitpipe)
c  determine shift between filter i and primary
	    call simposmat(1,iir,iret,1,1,ix,nu,nv,kf,*8041) 
c  the matching result parameter is stored
 8041	    imatch(i,nchip)=iret
c  Nov, 2005: determine if photometry should go ahead
	    if (imatch(i,nchip).lt.0) then
	      write(6,8042) nchip,i
 8042	format('   ***** WCS matching problem for Chip ',i2,
     *         ' filter ',i2,' skipped, to be re done')
	      goto 100
	    end if
	  end if
c  make bleeding column mask, first read in saturation pos file
c     first make name, and save for other filters
c         use saturated pos file for reference filter, but total
c             pos file for others
	  if (i.eq.nfilref) then
 	    write(6,*) ' ^^^ reading saturated pos file ',satposname
 	    call rpof(*806,1,satposname,idum,0)
	  else
c  if not the primary file, shift the pos file
 	    write(6,*) ' ^^^ reading pos file for bledding colum ',
     *                    posfname
 	    call rpof(*805,1,posfname,idum,0)
 805	    call pfshift(*806,1,0,0,dumstr,1)
	  end if
 806	  write(6,*) '  ^^^^ making bleeding mask, added to bp mask'
c first remove diffraction spike parameters
	  dsct1=dsc(1)
	  dsct2=dsc(2)
	  dsct3=dsc(3)
	  dsc(1)=-999
	  dsc(2)=0.
	  dsc(3)=0.
 	  call bleedmap(*807,1,2,ix,nu,nv,kf,0,0)
c fix bad column, both bad columns and bleed columns
 807	  call bpfix(*808,1,2,0,ix,nu,nv,kf)
 808	  dsc(1)=dsct1
	  dsc(2)=dsct2
	  dsc(3)=dsct3
c  read position file
	  write(6,*) ' ^^^ reading actual pos file ',posfname
 	  call rpof(*809,1,posfname,idum,0)
c     do shift if needed  (dumstr here is just a dummy, can be anything)
 809      if (i.ne.nfilref) call pfshift(*810,1,0,0,dumstr,1)
c create cosmic ray map and fix
 810	write(6,*) '  ^^^  creating cosmic ray map and fix '
	  call cosraymap(*8101,1,2,' ',' ',nscrit,0,0,rwd,ix,nu,nv,kf)
c pad the CR map
 8101	  call padmap(*8102,2,ix,nu,nv,kf,0)
 8102	  call bpfix(*8103,1,2,0,ix,nu,nv,kf)
c create classification reference star file
c   first make name
 8103	  call mkoutname(primname,1,cext,'rf')
c  save the name for reference
	  arefname=poutname
	  write(6,*) '  ^^^^ aref: finding psfs'
	  adulowpipe=adulowfil(i)
 	  call autoref(1,ix,nu,nv,kf,1,*811)
c do photometry
c  create .phm file output name and comment
 811	  call mkoutname(primname,1,cext,'ph')
	  call mkoutcomm(primname,1,cext,'ph')
 	  temname=poutname(1:lnblnk(poutname))
	  write(6,*) '   ^^^^ running ppphot ',temname
	  if (i.eq.nfilref) then
c get astrometry
	    iastrom=iastromtemp
	  else
	    iastrom=-1
	  end if
	  call pppdri(*812,nfra,temname,poutcomm,0,ix,nu,nv,kf)
c save mspos hdr parameters
 812	  continue
c812	  call smspars(i,temname)
 100	continue
	iastrom=iastromtemp
 999	return
 901	write(6,*) ' ***chip number text read error ',cext,
     *             '  *** skipping to next chip'
	return
	end
