C//////////////////////////////////////////////////////////////
c  additional subroutines developed for RCS pipeline
c
c...............................................................
c  RCS2 pipeline command:
c     n input fits file names, n=number of filter, 1st name=primary filter
c
c     (1) perform object finding using weigthed average of specified 
c           fits images  (currently max of 2)
c     (2) photometry for each filter (.phm and .ast files)
c     (3) generate mspos2 .hdr, spawn mspos2 to generate .cat file
c     (4) repeat for next chip
c
c   ipar=1: object finding only
c   ipar=2: photometry only
c   ipar=3: object finding and photometry
c
c  nch1, nch2: chip number range; if 0, all chips
	subroutine rcspipe(*,nch1,nch2,ipar,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,hdrname*60,
     *            maskstr*60,cext*2,imdir*55,dumstr*8,errmsg*30
	integer ib(6),ie(6),ibm(6),iem(6),status,rmode
	integer ix(nu,nv,kf)
	include 'pppcore.inc'
	include 'ppparm.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	common/satstars/nsatstars,ixsats(1000),jysats(1000)
	common/hdrfoder/nfilh,jforder(6),jrefcol(6)
	common/imagedir/imdir
	common/bstarxy/nbstar,bstx(20),bsty(20),bsrad(20),bsmag(20)
        common/imagestats/imstatspar(50),ich_id,ch_rms(50),ch_sky(50),
     *     nbstar_ch(50),ch_bsmag(15,50),ch_bsx(15,50),ch_bsy(15,50)
	common/bsmaskdata/ghostplx(2,2,36,3),ghostply(2,2,36,3),
     *         bsmaskrad(0:3),bspmag(3,2),ighostread
c temporary set for pipeparm
c	iapmaxpipe=49
c	npadpipe=1
c	ismtypepipe=1
c  set ircspipe=1 when starting
	ircspipe=1
c for now, the primary filter is always the first one on the input string
	nfilref=1
c  filter number for hdr to 0 if it is out of bound
	if (jforder(1).lt.0.or.jforder(1).gt.6) jforder(1)=0
c filestr is a string containing the root names of images for each filter
	write(6,1)
 1	format('  Enter fits file names (.fits.*/fz implicit), max 6:'
     *          /': ',$)
	read(5,'(a60)',end=999) filestr
c save the string in the common block for later use
	pfiltstr=filestr
c  parse the file string for getting names of the files
	call fstr(filestr,60,ib,ie,nfilt)
	if (nfilt.le.0) then
	  write(6,*) ' **error in image name string ***'
	  ircspipe=0
	  return1
	end if
c determine the number of chips	using the first image (all others
c    must have the same number of chips!)
	filen=filestr(ib(1):ie(1))
c   .hdr file name root: the patch name with ^cext added
c
c   the separation between filter and pointing name must be 
c     denoted by "_"
	do 161 k=1,lnblnk(filen)
	  if (filen(k:k).eq.'_') goto 162
 161	continue
 162	hdrname=filestr(ib(1):k-1)
	call imageex(filen,filenc,iimex)
	if (iimex.lt.0) then
	  ircspipe=0
	  return1
	end if
c  attempt to open file  (cfitsio rwmode: 0=read, 1=write)
	rmode=0
c	wmode=1
	status=0
	call ftgiou(lun,status)    
	call ftnopn(lun,imdir(1:lnblnk(imdir))//
     *      filenc(1:lnblnk(filenc))//'[0]',rmode, status)
	call ftgkyj(lun,'NEXTEND',nextendt,dumstr,status)
c  if not found, set nextend to 1
	if (status.ne.0) nextendt=1
	write(6,*) ' # of extension: ',nextendt,status
	if (status.ne.0.and.nextendt.eq.1) then
	  write(6,*) ' *** possible fits header error, assume extension=36'
	  nextendt=36
	end if
c  temp hard-wired statement to fix header extension number problem
	call ftgerr(status,errmsg)
	call ftclos(lun, status)
	call ftfiou(lun, status)
	if (nch1.eq.0) then
	  jc1=1
	  jc2=nextendt
	else if (nch2.eq.0) then
	  jc1=nch1
	  jc2=nch1
	else
	  jc1=nch1
	  jc2=min0(nextendt,nch2)
	end if
	write(6,*) ' +++ doing chips ',jc1,' to ',jc2
	maskstr=' '
	write(6,2)
 2	format('  Enter corresponding mask fits file names:'
     *          /': ',$)
	read(5,'(a60)',end=999) maskstr
c parse the file names
	if (maskstr.ne.' ') then
	  call fstr(maskstr,80,ibm,iem,nfilm)
	  if (maskstr(ibm(1):iem(1)).eq.'auto') then
c for automatic mask creation
	    nfilm=-1
	  else 
	    if (nfilm.ne.nfilt) then
	      write(6,*) ' ** # of mask files not eq image file, aborted'
	      return1
	    end if
	    write(6,*) ' Images to be processed (image/mask file names):'
	    do 151 i=1,nfilt
	      write(6,3) i,filestr(ib(i):ie(i)),maskstr(ibm(i):iem(i))
 3	format(' image ',i2,' :  ',a24,2x,a24)
 151	    continue
	  end if
	else 
	  write(6,*) ' ++ no mask file names, assuming no bad pix masks'
	  nfilm=-1
	  write(6,*) ' Images to be processed (no masks images):'
	  do 152 i=1,nfilt
	    write(6,3) i,filestr(ib(i):ie(i))
 152	  continue
	end if
c read in ghost image polynomial
	if (ighostread.ne.1) then
c	  call rdghostpoly(*870)
c870	  ighostread=1
	  call initghost
	else
c  assume no ghost polynomial
	  ighostread=99
	  write(6,*) ' +++ WARNING: no ghost-image offset file'
	end if
c  parameters:  All normal input parameters are passed to subroutines
c    using a common block: pipeparm.  The parameters must be set
c    before calling the pipeline command using: setp
c
c  object finding frames (need to make the read statement here
c     more intelligent so that 1 number can be entered
c   nobf1 and nobf2 are *filter* numbers, as ordered in filestr
	if (ipar.eq.1.or.ipar.eq.3) then
	  nobf1=1
	  nobf2=2
 601	  write(6,10) nobf1,nobf2
 10	format(' Enter 2 image (filter) numbers from list for ',
     *     'object finding'/ 
     *   '  (enter 0 for the 2nd one if not used) [',2i2,']: ',$)
	  call read2i(nobf1,nobf2,*999,*601)
	  write(6,*) '  ++ object finding using filters ',nobf1,nobf2
	end if
c
c  added July 27 2008
c      get bright star list and compute sky and rms values for
c        each chip and stored in common block common/imagestats for later use
c
c  this is done only for the reference filter
	filen=filestr(ib(1):ie(1))
	call frstats(jc1,jc2,filen,ix,nu,nv,kf)
c.........................
c   compute shifts between filters for different chips
c   if there is only one filter, then it is always nfilref (no need to shift)
	if (nfilt.gt.1) then
c  for object finding, and only uses one chip, then don''t do shift
	  if (ipar.eq.2.and.nobf2.eq.0) goto 701
c  normal mode: chip shifts computed before doing anything else
c for ichipsh 2 and 3, the shifts are done while doing photometry
	  if (ichipsh.eq.1) then
	    call mulshift(jc1,jc2,nextendt,filestr,1,
     *         nfilm,ie,ib,iem,ibm,ix,nu,nv,kf)    
	  end if
	end if
 701	continue
c................................
c for each chip, do object finding and/or photometry
	do 100 jj=jc1,jc2
c extension string
	  istatus=encode(cext,float(jj),1,0,idum)
c  this is the parameter that is passed to the pipeparm parameter
	  jchip=jj
	  write(6,*) '   >>>>> Processing chip ',cext,' <<<<<<<'
c object finding
	  if (ipar.eq.1.or.ipar.eq.3) then
	    write(6,*) ' ++object finding for chip ',cext,'; filters: '
     *                   ,nobf1,nobf2
            call pipeobjf(nobf1,nobf2,filestr,maskstr,
     *          nfilm,ib,ie,ibm,iem,ix,nu,nv,kf,cext)
	    if (imaskout.ge.1)  then
	write(6,*) ' == calling mask file writing routine =='
	      filen=filestr(ib(1):ie(1))
	      call getadusat(adusat,saturate(1),adusatpipe)
	      call wrtmask(1,3,filen,cext,adusat,ix,nu,nv,kf)
	    end if
	  end if
c photometry
	  if (ipar.eq.2.or.ipar.eq.3) then
	    write(6,*) ' photometry for chip ',cext
	    call pipephot(filestr,maskstr,nfilm,ib,ie,ibm,iem,
     *                   1,ix,nu,nv,kf,cext)
c make .hdr file 
	    call mkhdr(hdrname,cext)
c make msposin file (to be written)
c	    call mkmspbatch(hdrname,cext)
c spawn mspos2
c	    call system('nice '//pppath(1:lnblnk(pppath))//
c    *         '/mspos2 < '//mspos2in.fil//' > '//mspos2out.fil
	  end if
 100    continue
c old bad chipfix
        if (ichipsh.eq.3) then
	  if ((jc2-jc1).gt.2) then
	    write(6,*) '  +++ checking/redoing bad WCS chips ++++'
	    call photbadchips(jc1,jc2,
     *         filestr,maskstr,nfilm,ie,ib,iem,ibm,ix,nu,nv,kf)
	  else
	    write(6,*) '  +++ fewer than 3 chips, no checking',
     *          ' for redoing bad WCS chips '
	  end if
          return
	end if
	ircspipe=0
 999	return1
	end
c
c////////////////////////////////////////////////////////////////
c
c  RCS2 pipeline to add n filters to existing data already
c     n input fits file names, n=number of filter, 1st name=primary filter
c 
c     required input: primary filter name, new filter names, 
c                  filter names of images already have photometry done.
c            all these images are needed so that headers can be
c             generated, with option of checking for existing .hdr files
c              to update.
c
c     (1) determine shifts between primary and target filters
c     (2) photometry for each filter (.phm and .ast files)
c     (3) generate new mspos2 .hdr, spawn mspos2 to generate .cat file
c     (4) repeat for next chip
c
c
c  nch1, nch2: chip number range; if 0, all chips
	subroutine rcspipeadd(*,nch1,nch2,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,hdrname*60,
     *            maskstr*60,cext*2,imdir*55,dumstr*8,errmsg*30
     *           ,newfstr*60,donefstr*60,forderstr*60,crefstr*60
	integer ib(6),ie(6),ibm(6),iem(6),ibh(6),ieh(6),status,rmode
     *           ,ibn(5),ien(5),ibd(6),ied(6),ibc(6),iec(6)
	integer ix(nu,nv,kf)
	include 'pppcore.inc'
	include 'ppparm.inc'
	include 'pipeparm.inc'
	common/hdrfoder/nfilh,jforder(6),jrefcol(6)
	common/imagedir/imdir
	common/bstarxy/nbstar,bstx(20),bsty(20),bsrad(20),bsmag(20)
c temporary set for pipeparm
c	iapmaxpipe=49
c	npadpipe=1
c	ismtypepipe=1
c  set ircspipe=1 when starting
	ircspipe=1
c for now, the primary filter is always the first one on the input string
	nfilref=1
	if (jforder(1).lt.0.or.jforder(1).gt.6) jforder(1)=0
c filestr is a string containing the root names of images for each filter
c  request names of filters already have photometry
	write(6,1)
 1	format('  Enter fits names (.fits/fz implicit) for filters',
     *    '  already has phot',
     *   / '    pos reference filter goes first: ',$)
	read(5,'(a60)',end=999) donefstr
	call fstr(donefstr,60,ibd,ied,nfild)
c  parse the file strings for getting names of the files
	if (nfild.le.0) then
	  write(6,*) ' **error in old image name string ***'
	  ircspipe=0
	  return1
	end if
	write(6,*) ' old filters ',donefstr,nfild
	write(6,2)
 2	format('  Enter fits names (.fits/fz implicit) for new filters,',
     *     ' max total images 6: ',$)
	read(5,'(a60)',end=999) newfstr
	call fstr(newfstr,60,ibn,ien,nfiln)
	if (nfiln.le.0) then
	  write(6,*) ' **error in new image name string ***'
	  ircspipe=0
	  return1
	end if
	write(6,*) ' new filters ',newfstr,nfiln
c  request the order of the filters for .hdr file
	write(6,4)
 4	format('  Enter order of filter numbers for .hdr file: ',$)
	read(5,'(a60)',end=999) forderstr
c  the .hdr file, filter order
	write(6,*) '  filter order string ', forderstr
	call fstr(forderstr,60,ibh,ieh,nfilh)
	if (nfilh.le.0) then
	  write(6,*) ' **error in .hrd filter order string ***'
	  ircspipe=0
	  return1
	end if
c  the .hdr file, reference color
	write(6,14)
 14	format('  Enter ref color numbers for .hdr file: ',$)
	read(5,'(a60)',end=999) crefstr
	call fstr(crefstr,60,ibc,iec,nfilc)
	if (nfilc.le.0) then
	  write(6,*) ' **error in .hrd filter order string ***'
	  ircspipe=0
	  return1
	end if
c --- now using the done and new filter strings, create a single
c   string which contains the pos ref filter first, and the rest
c   of the new filters, and assign this to filestr (the usual name)
c	filestr=donefstr(ibd(1):ied(1))//' '//newfstr(1:lnblnk(newfstr))
	filestr=donefstr(ibd(1):ied(nfild))//' '
     *          //newfstr(1:lnblnk(newfstr))
c save the string in the common block for later use
	pfiltstr=filestr
c parse the created string
	call fstr(filestr,60,ib,ie,nfilt)
	write(6,*) ' total filters ',filestr,nfilt
c  filter order string sets the order by which the filters appears on
	write(6,*) '  old filter, new filter, total filter: ',nfild
     *                  ,nfiln, nfilt
c   parse the filter order
	if (nfilh.eq.0) then
	  write(6,*) ' +++ filter order string = null, default assumed'
	  jforder(1)=0
	  nfilh=nfilt
	else
	  do 261 mm=1, nfilh
	    read(forderstr(ibh(mm):ieh(mm)),*,err=921) jforder(mm)
 261	  continue
	  goto 262
 921	    write(6,*) '  *** error in header filter order ** aborted ',
     *              mm,forderstr
	    return1
 262	  continue
	end if
c   parse the reference color string
	if (nfilc.eq.0) then
	  write(6,*) ' +++reference color string=null, default assumed'
	  jrefcol(1)=0
	else
	  do 271 mm=1, nfilc
	    read(crefstr(ibc(mm):iec(mm)),*,err=922) jrefcol(mm)
 271	  continue
	  goto 272
 922	    write(6,*) '  *** error in header filter order ** aborted ',
     *              mm,crefstr
	    return1
 272	  continue
	end if
c
c determine the number of chips	using the first image (all others
c    must have the same number of chips!)
	filen=filestr(ib(1):ie(1))
c   .hdr file name root: the patch name with ^cext added
c
c   the separation between filter and pointing name must be 
c     denoted by "_"
	do 161 k=1,lnblnk(filen)
	  if (filen(k:k).eq.'_') goto 162
 161	continue
 162	hdrname=filestr(ib(1):k-1)
	call imageex(filen,filenc,iimex)
	if (iimex.lt.0) then
	  ircspipe=0
	  return1
	end if
c  attempt to open file  (cfitsio rwmode: 0=read, 1=write)
	rmode=0
c	wmode=1
	status=0
	call ftgiou(lun,status)    
	call ftnopn(lun,imdir(1:lnblnk(imdir))//
     *      filenc(1:lnblnk(filenc))//'[0]',rmode, status)
	call ftgkyj(lun,'NEXTEND',nextendt,dumstr,status)
c  if not found, set nextend to 1
	if (status.ne.0) nextendt=1
	write(6,*) ' # of extension: ',nextendt,status
	call ftgerr(status,errmsg)
	call ftclos(lun, status)
	call ftfiou(lun, status)
	if (nch1.eq.0) then
	  jc1=1
	  jc2=nextendt
	else if (nch2.eq.0) then
	  jc1=nch1
	  jc2=nch1
	else
	  jc1=nch1
	  jc2=min0(nextendt,nch2)
	end if
	write(6,*) ' +++ doing chips ',jc1,' to ',jc2
	maskstr=' '
	write(6,5)
 5	format('  Enter corresponding mask fits file names:'
     *  /' note: if not auto, must have ref image, then new images: ',$)
	read(5,'(a60)',end=999) maskstr
c parse the file names
	if (maskstr.ne.' ') then
	  call fstr(maskstr,80,ibm,iem,nfilm)
	  if (maskstr(ibm(1):iem(1)).eq.'auto') then
c for automatic mask creation
	    nfilm=-1
	  else 
	    if (nfilm.ne.nfilt) then
	      write(6,*) ' ** # of mask files not eq image file, aborted'
	      return1
	    end if
	    write(6,*) ' Images to be processed (image/mask file names):'
	    do 151 i=1,nfilt
	      write(6,3) i,filestr(ib(i):ie(i)),maskstr(ibm(i):iem(i))
 3	format(' image ',i2,' :  ',a24,2x,a24)
 151	    continue
	  end if
	else 
	  write(6,*) ' ++ no mask file names, assuming no bad pix masks'
	  nfilm=-1
	  write(6,*) ' Images to be processed (no masks images):'
	  do 152 i=1,nfilt
	    write(6,3) i,filestr(ib(i):ie(i))
 152	  continue
	end if
c  parameters:  All normal input parameters are passed to subroutines
c    using a common block: pipeparm.  The parameters must be set
c    before calling the pipeline command using: setp
c
c.........................
c   compute shifts between new filters and ref fil for different chips
c only allowable mode: chip shifts computed before doing anything else
	ichipsh=1
	write(6,*) ' ++ichipsh (parameter for shifting) set to 1 '
	if (ichipsh.eq.1) then
c note: mulshift does not require the .pos file, since it uses qfind for
c  objects to compare.  It requires the reference filter, and is modified
c   from the previous one to o
c
c  do shifting for the new filters only
	write(6,*) '  +++ matching, starting filter ',nfilt-nfiln+1
	  call mulshift(jc1,jc2,nextendt,filestr,nfilt-nfiln+1,
     *              nfilm,ie,ib,iem,ibm,ix,nu,nv,kf)    
c determine if the shift is too large for photometry to continue
	  call checkshift(nfilt-nfiln+1,jc1,jc2,iret,hdrname)
c  if iret=-1, abort the photometry
	  if (iret.lt.0) then
	     return1
	   end if
	end if
c................................
c for each chip, do photometry
c    but do not do the reference filter, 
c    note that pos file is read while doing photometry
c	call fstr(filestr,60,ib,ie,nfilt)
	do 100 jj=jc1,jc2
c extension string
	  istatus=encode(cext,float(jj),1,0,idum)
c  this is the parameter that is passed to the pipeparm parameter
	  jchip=jj
	  write(6,*) '   >>>>> Doing chip ',cext,' <<<<<<<'
c read .pos file
c photometry
	  write(6,*) ' photometry for chip ',cext
	  call pipephot(filestr,maskstr,nfilm,ib,ie,ibm,iem,
     *                 nfilt-nfiln+1,ix,nu,nv,kf,cext)
c read in fits header parameters for other filters for making hdr file
	  
c make .hdr file 
	  call mkhdr(hdrname,cext)
c make msposin file (to be written)
c	  call mkmspbatch(hdrname,cext)
c spawn mspos2
c	  call system('nice '//pppath(1:lnblnk(pppath))//
c    *         '/mspos2 < '//mspos2in.fil//' > '//mspos2out.fil
 100    continue
	ircspipe=0
 999	return1
	end
c/////////////////////////////////////////////////////////////////////
c  subroutine to decide whether the lost in area is too large due
c   to shifting between 2 filters.
c        iccdgap, jccdgap: gaps in pixels; 
c         gappercent: the area lost limit
c   the data from pmxshift and pmyshift are used (these are already
c         corrected for the bad chips) for computing average shfits;
c       note that it appears that the gaps are not constant (since
c         the shifts are fluctuating by as much as 20 pix)
c   nfil: filter number
c   jc1, jc2: chip range
c   nii, njj: chip size
c
	subroutine checkshift(nfil,jc1,jc2,iret,headname)
	character outname*28,headname*60
	include 'pipeparm.inc'
	include 'picdir.inc'
c  compute average lost of area over the chips: use absolute value
c    first compute the average lost in pixels in I an J direction
	aavx=0.
	aavy=0.
	iret=1
	write(6,*) '  *ccdgap ',iccdgap,jccdgap,' nfil ',nfil,nfilt
	do 101 j=nfil,nfilt
	  do 100 i=jc1,jc2
	write(6,*) ' chip ',i,' shifts ;',pmxshift(j,i),pmyshift(j,i)
	    if (abs(pmxshift(j,i)).gt.float(iccdgap)) then
	      aavx=aavx+abs(abs(pmxshift(j,i))-iccdgap)
	    end if
	    if (abs(pmyshift(j,i)).gt.float(jccdgap)) then
	      aavy=aavy+abs(abs(pmyshift(j,i))-jccdgap)
	    end if
 100	  continue
	  aavx=aavx/float(jc2-jc1+1)
	  aavy=aavy/float(jc2-jc1+1)
c  compute % area lost
	  arealost=njj(1)*aavx+nii(1)*aavy
	  percentlost=100.*arealost/(nii(1)*njj(1))
	  write(6,1) j,aavx,aavy,arealost,percentlost
 1	  format(' -- Area shift check -- filter',i3,/
     *         '   Average lost in pixel in I beyond gap: ',f7.4/
     *         '   Average lost in pixel in J beyond gap: ',f7.4/
     *         '   area lost (pix^2): ',f8.0,' % lost:',f6.2)
	  if (percentlost.gt.gappercent) then
	    write(6,2) gappercent
 2	format('  *** area lost greater than gappercent= ',f6.2, /
     *        '  Resampling required, photometry aborted...')
	    iret=-1
c  generate a warning file, and abort  (this may not work properly
c         for more than 1 new filter, need to make modification
c         to allow the checking done for all filters before aborting
c
	    outname=headname(1:lnblnk(headname))//'.RESAMPLE'
	    call opennew(29,1,28,outname,*999)
	    write (29,10) headname(1:lnblnk(headname))
 10	format('  Pointing ',a20, ' shift too large; resampling',
     *             ' chips recommended')
	    do 110 m=jc1,jc2
	      write(29,*) ' chip ',m,' shifts ;',pmxshift(j,m),
     *         pmyshift(j,m)
 110	    continue
	    write(29,1) j,aavx,aavy,arealost,percentlost
	    write(29,2) gappercent
	    write(29,11)
 11	format('  (To over-ride the abort, pset gappercent to a '
     *            ,'larger value)')
	    close(unit=29)
	  end if
 101	continue
 999	return
	end 
c///////////////////////////////////////////////////////////////////////
c  subroutine to get key word values from the header
c
	subroutine getkeyval(*,nfr,kf)
	real*8 ratemp,dectemp
	character headbst*2880,headtem*2880,hdr(36)*80,temfil*20,
     *            headblk*2880,ncomc*36,fitsname*24
	common/header/bzero,bscale,headblk(20),nhb
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	COMMON/PICC/ ncomc(15),fitsname(15)
c	include 'picdir.inc'
	include 'imagepar.inc'
	equivalence (headtem(1:1),hdr(1)(1:1))
	call QUERYNFR(NFR,KF,0,*999,*2001)
c go through header block to look for key words
 2001	continue
	if (nhbst(nfr).le.0) then
	  write(6,*) '  *** 0 header block for image, no parameters '
	  return1
	end if
c  reasonable default values, in case the don''t exit in header
	ncombine(nfr)=1
c	nextend(nfr)=1
c going through each card
	do 100 j=1,nhbst(nfr)
	  headtem=headbst(j,nfr)
c	if (j.eq.1) write(6,*) (hdr(kk),kk=1,36)
	  do 101 i=1,36 
	    if (hdr(i)(1:3).eq.'END') then
c header done
	      go to 998
	    else if (hdr(i)(1:5).eq.'BZERO') then
	      read(hdr(i)(11:30),*,err=999) bzest(nfr)
	      bzero=bzest(nfr)
	    else if (hdr(i)(1:5).eq.'BSCAL') then
	      read(hdr(i)(11:30),*,err=999) bscst(nfr)
	      bscale=bscst(nfr)
	    else if (hdr(i)(1:7).eq.'NEXTEND') then
	      read(hdr(i)(11:30),*,err=999) nextendh(nfr)
c	write(6,*) hdr(i)(1:30)
c	write(6,*) nextend(nfr)
	    else if (hdr(i)(1:7).eq.'FILTER ') then
	      read(hdr(i)(11:30),*) temfil
	      filter(nfr)=temfil(1:1)
	    else if (hdr(i)(1:6).eq.'OBJECT') then
	      read(hdr(i)(11:30),*) ncomc(nfr)
	    else if (hdr(i)(1:7).eq.'OBSTIME'.or.hdr(i)(1:2).eq.'UT'.
     *              or.hdr(i)(1:5).eq.'UTIME') then
	      read(hdr(i)(11:30),*) obstime(nfr)
	    else if (hdr(i)(1:8).eq.'DATE-OBS') then
	      read(hdr(i)(11:30),*) obsdate(nfr)
	    else if (hdr(i)(1:7).eq.'EXPTIME') then
	      read(hdr(i)(11:30),*,err=990) exptime(nfr)
            else if (hdr(i)(1:7).eq.'AIRMASS')  then
                read(hdr(i)(23:34),*,err=990) airmassobs(nfr)
            else if (hdr(i)(1:3).eq.'MJD') then
              read(hdr(i)(11:30),*,err=990) mjuldate(nfr)
            else if (hdr(i)(1:5).eq.'GAIN ') then
              read(hdr(i)(11:30),*,err=990) ccdgn(nfr)
            else if (hdr(i)(1:8).eq.'RDNOISE ') then
              read(hdr(i)(11:30),*,err=990) readnoise(nfr)
            else if (hdr(i)(1:8).eq.'SATURATE') then
              read(hdr(i)(11:30),*,err=990) saturate(nfr)
            else if (hdr(i)(1:8).eq.'NCOMBINE') then
              read(hdr(i)(11:30),*,err=990) ncombine(nfr)
            else if (hdr(i)(1:6).eq.'PIXSCA') then
              read(hdr(i)(11:30),*,err=990) pixscale(nfr)
            else if (hdr(i)(1:3).eq.'RA ') then
              read(hdr(i)(11:30),*) rastr(nfr)
            else if (hdr(i)(1:4).eq.'DEC ') then
              read(hdr(i)(11:30),*) decstr(nfr)
            else if (hdr(i)(1:5).eq.'EPOCH') then
              read(hdr(i)(20:30),*,err=990) epoch(nfr)
            else if (hdr(i)(1:8).eq.'LATITUDE') then
              read(hdr(i)(20:30),*,err=990) rlat(nfr)
            else if (hdr(i)(1:8).eq.'LONGITUD') then
              read(hdr(i)(20:30),*,err=990) rlong(nfr)
            else if (hdr(i)(1:6).eq.'CRPIX1') then
              read(hdr(i)(11:31),*,err=990) crpix1(nfr)
              iwcs=1
              write(6,*) '  wcs found;  CRPIX1 ',crpix1(nfr),iwcs
            else if (hdr(i)(1:6).eq.'CRPIX2') then
              read(hdr(i)(11:31),*,err=990) crpix2(nfr)
            else if (hdr(i)(1:6).eq.'CRVAL1') then
              read(hdr(i)(11:31),*,err=990) crval1(nfr)
            else if (hdr(i)(1:6).eq.'CRVAL2') then
              read(hdr(i)(11:31),*,err=990) crval2(nfr)
            else if (hdr(i)(1:5).eq.'CD1_1') then
              read(hdr(i)(11:31),*,err=990) cd1_1(nfr)
            else if (hdr(i)(1:5).eq.'CD1_2') then
              read(hdr(i)(11:31),*,err=990) cd1_2(nfr)
            else if (hdr(i)(1:5).eq.'CD2_1') then
              read(hdr(i)(11:31),*,err=990) cd2_1(nfr)
            else if (hdr(i)(1:5).eq.'CD2_2') then
              read(hdr(i)(11:31),*,err=990) cd2_2(nfr)
	    else if (hdr(i)(1:7).eq.'PHOT_C ') then
              read(hdr(i)(11:30),*,err=990) zeropt(nfr)
	    else if (hdr(i)(1:7).eq.'PHOT_X ') then
              read(hdr(i)(11:30),*,err=990) clrterm(nfr)
	    else if (hdr(i)(1:7).eq.'PHOT_K ') then
              read(hdr(i)(11:30),*,err=990) airmterm(nfr)
	    end if
	    goto 101
 990	    write(6,3) j,i,hdr(i)(1:10)
 3	format('  +++  header card read error, blk ',i2,' card ',
     *         i2,5x,a10)
 101	  continue
 100	continue
c if wcs exists, compute co-ordinate of chip center
 998	if (iwcs.eq.1) then
	  call astrom(nfr,1.0,1.0,ratemp,dectemp)
	  ra11(nfr)=ratemp
	  dec11(nfr)=dectemp
	write(6,*) ' RA Dec of (1,1) ',ra11(nfr),dec11(nfr)
	end if
 999	return1
	end 
c//////////////////////////////////////////////////////////////
c subroutine to type out fits header values (for checking)
c
	subroutine fitsvals(*,nfr,kf)
	character headbst*2880
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	include 'imagepar.inc'
	include 'picdir.inc'
	call QUERYNFR(NFR,KF,0,*999,*2001)
 2001	write(6,1) nii(nfr),njj(nfr),bzest(nfr),bscst(nfr)
     *     ,nextendh(nfr),obsdate(nfr)(1:12),
     *       obstime(nfr)(1:12),mjuldate(nfr),rastr(nfr)(1:12),
     *    decstr(nfr)(1:12),epoch(nfr),rlat(nfr),
     *    rlong(nfr),exptime(nfr),filter(nfr),airmassobs(nfr),
     *    pixscale(nfr),ccdgn(nfr),readnoise(nfr),saturate(nfr),iwcs
1	format(' Image size ',2i5,'  BZERO=',f10.2,'  BSCALE=',g13.6,
     *  '  N-chip: ',i2,/
     * ' obsdate: ',a12,'  obstime: ',a12,'  mJDate:',f15.7,/
     *   ' RA ',a12,' Dec ',a12,'  Epoch ',f6.1,/
     *   ' obs lat',f10.5,'   obs long ',f10.5,/' exp time(s)',
     *   f10.2,' filter: ',a1,'  Airmass',f5.2,'  pix scale',f7.4,
     *   /' gain',f6.3,'  rdnoise ',f5.2,'  saturation ',f6.0,
     *         '  wcs: ',i1)
	if (iwcs.gt.0) then
c output wcs paramters
	  write(6,2) crpix1(nfr),crpix2(nfr),crval1(nfr),crval2(nfr),
     *       cd1_1(nfr),cd1_2(nfr),cd2_1(nfr),cd2_2(nfr)
 2	format(' WCS parameters:  crpix 1 2:',2f12.5,/
     *         '                  crval 1 2:',2f12.7,/
     *            '        cd1_1,1_2,2_1,2_2:',4g13.5)
	  write(6,3) zeropt(nfr),airmterm(nfr),clrterm(nfr)
 3	format(' Photometric Calibration:',/
     *         '       A_0=',f7.3,'   A_airmass=',f7.3,
     *         '   A_colr=',f7.3)
	end if
 999	return1
	end
c////////////////////////////////////////////////////////////////////
c     driving subroutine to find bright stars on chip and produce
c     approx masks for haloes
c
c   to get a large list of stars, use nfr=negative in the command get*
c
	subroutine usnodri(*,nfrt,kf,bsmag1,bsmag2,igetstar)
	real rad,smag(1000)
	double precision sra(1000),sdec(1000),ratem,dectem
	integer nstars,ii
	common/bstarxy/nbstar,bstx(20),bsty(20),bsrad(20),bsmag(20)
	include 'imagepar.inc'
	include 'picdir.inc'
	nfr=iabs(nfrt)
	write(6,*) nfr,nfrt
	call QUERYNFR(NFR,KF,0,*999,*2001)
c search size detemine by size of image
 2001	continue
	if (nfrt.lt.0) then
 600	  write(6,22) rad
 22	format('  Enter search radius in min [',f6.2,']: ',$)
	  call read1r(rad,*999,*600)
	else
	  rad=0.5*sqrt(1.*(nii(nfr)*nii(nfr)+njj(nfr)*njj(nfr)))
     *           *pixscale(nfr)/60.
	end if
	write(6,*) ' pix scale and search radius',pixscale(nfr),rad
c     ---- Find bright stars in and around frame
c first determine the ra and dec of the center of chip using wcs
	x0=0.5*float(nii(nfr))
	y0=0.5*float(njj(nfr))
	call astrom(nfr,x0,y0,ratem,dectem)
	write(6,*) ' field center ',x0,y0,'  RA,Dec ',ratem,dectem
c	rat2=ratem*15.
	rat2=ratem
	dect2=dectem
	if (igetstar.eq.2) then
 	  call queryusnob1(rat2,dect2,rad,bsmag1,bsmag2,
     *         nstars,sra,sdec,smag)
	else if (igetstar.eq.1) then
	  call querytycho(rat2,dect2,rad,bsmag1,bsmag2,
     *         nstars,sra,sdec,smag)
	end if
	write(6,20) nstars,bsmag1,bsmag2
 20	format(i4,' bright stars between mags ',f6.2,' and ',f6.2)
	nbstar=0
	ns=0
	if (nfrt.gt.0) then
	  xlim1=-30.
	  xlim2=30.
	  ylim1=-30.
	  ylim2=30.
c  these limits are for the actual ghost 
	  xlim1=-300.
	  xlim2=300.
	  ylim1=-300.
	  ylim2=300.
	else
	  xlim1=-100000.
	  xlim2=100000.
	  ylim1=-100000.
	  ylim2=100000.
	end if
	write(6,17) xlim2
 17	format('     List of stars within',f7.0,' pix of CCD chip'/ 
     *    ' #     RA         Dec      mag      Ipix    Jpix'
     *     ,' qkmask rad')
      do ii=1,nstars
c       write(6,15) ii,sra(ii),sdec(ii),smag(ii)
 15	format(i4,2f11.5,2x,f6.2,'  checking RA Dec')
	call xypixpos(nfr,xi,yj,sra(ii),sdec(ii))
	if (xi.gt.xlim1.and.xi.lt.float(nii(nfr))+xlim2) then
	  if (yj.gt.ylim1.and.yj.lt.float(njj(nfr))+ylim2) then
	    ns=ns+1
c  max number saved: 20
	    if (nbstar.le.19) then
	      nbstar=nbstar+1
	      bstx(nbstar)=xi
	      bsty(nbstar)=yj
	      bsmag(nbstar)=smag(ii)
c size of mask: this is just a fake calibration
	      if (smag(ii).le.6.) then
	        bsrad(nbstar)=1500.
	      else if (smag(ii).le.7.) then
	        bsrad(nbstar)=1000.
	      else if (smag(ii).le.8.) then
	        bsrad(nbstar)=650.
	      else if (smag(ii).le.9.) then
	        bsrad(nbstar)=350.
	      else if (smag(ii).le.10.) then
	        bsrad(nbstar)=200.
	      else if (smag(ii).le.11.) then
	        bsrad(nbstar)=110
	      else
	        bsrad(nbstar)=60
	      end if
              write(6,11) ns,sra(ii),sdec(ii),smag(ii),xi,yj,
     *                   bsrad(nbstar)
	    else
	      dumrad=0.0
              write(6,11) ns,sra(ii),sdec(ii),smag(ii),xi,yj,dumrad
 11	format(i3,2f11.6,f6.2,3x,2f7.1,2x,f6.0)
c save the result to the bright star array:
	    end if
	  else
	write(6,*) ' out of y bound'
            write(6,12) sra(ii),sdec(ii),smag(ii),xi,yj
	  end if
	else
          write(6,12) sra(ii),sdec(ii),smag(ii),xi,yj
 12	format(' 0 ',2f11.6,f6.2,3x,2f7.1,' not on chip')
	end if
      enddo
	if (nbstar.lt.ns) write(6,*) 
     *       ' too many bright stars, only first 10 saved'	  
 999	return1
	end
c////////////////////////////////////////////////////////////////////
c  subroutine for finding bright stars from Tycho catalog
c   need to be connected to the network and running wget
c     ----------------------------------------------------------------
c  adopted from the USNO query, Aug 12, 2008
c
      subroutine querytycho(ra,dec,radius,m1,m2,nstars,sra,sdec,smag)
      character url*64,cra*8,cdec*8,cradius*5,query*119,junk*8,id*12
      character*133 ar
      real ra,dec,radius,smag(1000),m1,m2,junkreal,smagb(1000),
     *  smagv(1000),bmag1,bmag2
      double precision sra(1000),sdec(1000),sratemp,sdectemp
      integer nstars
      integer stat,ii,ios,junkint
	nstars=0
c     Search within radius [arcmins] of ra,dec and mag limits: m1-->m2
c     return nstars, with coords sra,sdec and mags smag
c     return r_mag by default. if r is not present use b_mag
c     [no colour correction from b-->r, as yet]
c -- Take RA, Dec (in degrees), search radius and retrieve catalogue of USNO-B1 
c      objects in field
c     This will work on any PC running wget (fairly standard linux)
c     -- check for wget
      call system('wget --version >& /dev/null',stat)
      if(stat.ne.0) then 
        write(6,*) ' ++ you do not have wget installed, search aborted'
        nstars=0
        return
      endif
c     --
	write(cra,fmt='(f8.4)') ra
	write(cdec,fmt='(f8.4)') dec
	write(cradius,fmt='(f5.2)') radius
	write(6,*) ' RA: ',cra,'  Dec ',cdec,'  radius(arcmin): ',radius
	ibadweb=0
 300	continue
	if (ibadweb.eq.0) then
c     -- Build query (for CADC)
	  url=
     *   'http://vizier.hia.nrc.ca/cgi-bin/asu-tsv/?-source=I/259/TYC2'
	else if (ibadweb.eq.1) then
	  url=
     *    'webviz.u-strasbg.fr/viz-bin/asu-tsv/?-source=I/259/TYC2'
	end if
        query=url(1:lnblnk(url))//'&-c.ra='//cra//'&-c.dec='//
     *        cdec(1:lnblnk(cdec))
     *            //'&-c.rm='//cradius//'&-out.max=10000'
       print *,query
c     -- execute call to wget and place output table in temp.TAB
       call system("wget '"//query//"' -O temp.TAB")
c     --
c     -- Read in catalogue [header is junk. last line of junk starts '--------'
c        then format is ID, RA, Dec,  4*junk, R1mag, B2mag, R2mag, Imag
c
c     this is horrendous due to the formatting of USSO cat!!!!
c
c     need to apply magnitude cuts here
      junk='zzzzzzzz'
      open(unit=11,file='temp.TAB',status='old',form='formatted')
c read to find the first line of data
      do while(junk.ne.'----')
          read(unit=11,fmt=*,end=998) junk
c check if the website is working, error message is ****
	  if (junk(2:4).eq.'***') then
	      ibadweb=ibadweb+1
	      if (ibadweb.eq.1) then
c use UVIC mirrored site
	       write(6,*) ' +++ CADC network down, using Strasbg mirror'
	         goto 300
	      else
	        write(6,*) ' *** Both Tycho servers down, should abort!!'
c no grace abort to get out from here yet...
	         return 
	      end if
	   end if
        enddo
c     now read real data
	nstar=0
	do 100 ii=1,10000
	  read(11,fmt='(A133)',end=998) ar
	  if (ar(1:5).eq.'') goto 101
	  iline=iline+1
          print *,ar
          print *,ar(6:10),ar(29:35),ar(37:42),ar(51:62),ar(64:75)
c  read in both B and V mag; note the program ignore proper motion 
c    which are included in the wget data, should one want to use it
c         read(ar,err=999,end=998) id,bmag,vmag,sratemp,sdectemp
c40	format(5x,A5,19X,F7.3,1X,f7.3,7x,F13.8,2x,f12.8) 
	  id=ar(6:10)
	  read(ar(29:35),*,err=999) bmag
	  read(ar(37:42),*,err=999) vmag
	  read(ar(51:62),*,err=999) sratemp
	  read(ar(64:75),*,err=999) sdectemp
c	  write(6,*) id,bmag,vmag,sratemp,sdectemp
c  assign v mag if exists, otherwise, b mag
	  if (vmag.lt.0.1) vmag=bmag
c compute R magnitude using color-color relation from tycho+usno
c	  smagtemp=vmag-0.5337*(bmag-vmag)-0.03
c compute r magnitude using transformation from Bilir et al 2005)
 	  smagtemp=vmag-0.49*(bmag-vmag)+0.14
	  if( (smagtemp.lt.m1).or.(smagtemp.gt.m2) ) then
            print *,'rejecting ',ii,id,bmag,vmag,smagtemp
	  else
            nstars=nstars+1
	    sra(nstars)=sratemp
	    sdec(nstars)=sdectemp
	    smag(nstars)=smagtemp
	    smagv(nstars)=vmag
	    smagb(nstars)=bmag
c           print *,ii,' keeping ',id,smag(nstars)
	  endif 
 100	continue
 101 	continue
       do ii=1,nstars
          print *,ii,sra(ii),sdec(ii),smagv(ii),smagb(ii),smag(ii)
       enddo
c      print *,m1,m2,nstars
       close(unit=11)
	return
 998	close(unit=11)
	write(6,*) '  --- end of file reached --'
	return
 999	close(unit=11)
	write(6,*) '  --- read ERROR ',ii,ar
	return
      end
c////////////////////////////////////////////////////////////////////
c  modification of Gilbank''s subroutine for finding bright stars from USNO 
c   need to be connected to the network and running wget
c     ----------------------------------------------------------------
c
      subroutine queryusnob1(ra,dec,radius,m1,m2,nstars,sra,sdec,smag)
      character url*58,cra*8,cdec*8,cradius*5,query*119,junk*8,id*12
      character*133 ar
      real ra,dec,radius,smag(1000),m1,m2,junkreal,smagb(1000),
     *  rmag1,rmag2,bmag1,bmag2,fakemag(4)
      double precision sra(1000),sdec(1000)
      integer nstars
      integer stat,ii,ios,junkint
	nstars=0
c     Search within radius [arcmins] of ra,dec and mag limits: m1-->m2
c     return nstars, with coords sra,sdec and mags smag
c     return r_mag by default. if r is not present use b_mag
c     [no colour correction from b-->r, as yet]
c -- Take RA, Dec (in degrees), search radius and retrieve catalogue of USNO-B1 
c      objects in field
c     This will work on any PC running wget (fairly standard linux)
c     -- check for wget
      call system('wget --version >& /dev/null',stat)
      if(stat.ne.0) then 
        write(6,*) ' ++ you do not have wget installed, search aborted'
        nstars=0
        return
      endif
c     --
      write(cra,fmt='(f8.4)') ra
      write(cdec,fmt='(f8.4)') dec
      write(cradius,fmt='(f5.2)') radius
	write(6,*) ' RA: ',cra,'  Dec ',cdec,'  radius(arcmin): ',radius
	ibadweb=0
 300	continue
	if (ibadweb.eq.0) then
c     -- Build query (for CADC)
      url='http://vizier.hia.nrc.ca/viz-bin/asu-acl?-source=USNO-B1.0'
	else if (ibadweb.eq.1) then
      url='http://vizier.u-strasbg.fr/viz-bin/asu-acl?-source=USNO-B1.0'
	end if
        query=url//'&-c.ra='//cra//'&-c.dec='//cdec
     *            //'&-c.rm='//cradius//'&-out.max=10000'
c      print *,query
c     -- execute call to wget and place output table in /tmp/temp.TAB
       call system("wget '"//query//"' -O /tmp/temp.TAB")
c     --
c     -- Read in catalogue [header is junk. last line of junk starts '--------'
c        then format is ID, RA, Dec,  4*junk, R1mag, B2mag, R2mag, Imag
c
c     this is horrendous due to the formatting of USSO cat!!!!
c
c     need to apply magnitude cuts here
      junk='zzzzzzzz'
      open(unit=11,file='/tmp/temp.TAB',status='old',form='formatted')
c read to find the first line of data
      do while(junk.ne.'--------')
          read(unit=11,fmt=*) junk
c check if the website is working, error message is ****
	  if (junk(2:4).eq.'***') then
	      ibadweb=ibadweb+1
	      if (ibadweb.eq.1) then
c use UVIC mirrored site
	       write(6,*) ' +++ CADC network down, using Strasbg mirror'
	         goto 300
	      else
	         write(6,*) '  **** Both USNO servers down, should abort!!'
c no grace abort to get out from here yet...
	         return 
	      end if
	   end if
        enddo
c     now read real data
      ios=0
      ii=1
	iline=1
      id='XXXXXX'
      do while(ar(1:5).ne.'[EOD]')
         read(11,fmt='(A133)')ar
	 iline=iline+1
c         print *,ar
c         print *,ar(73:77),ar(79:83),ar(85:89),ar(91:95)
         read(ar,fmt='(A12,1X,F10.6,1X,F10.6)') id,sra(ii),sdec(ii)
c  read in all 4 mags. The problem with the format is that if the
c   last fields are blank, they are tabs rather space.  The following
c   reading algorithm should take care of it
	 ifm=0
	 do 120 kk=73,91,6
	   ifm=ifm+1
	   fakemag(ifm)=0.000
	   read(ar(kk:kk+4),33,err=120) fakemag(ifm)
 120	 continue
 33	format(f5.2)
c  assign r mag if exists, otherwise, b mag
	rmag1=fakemag(1)
	bmag1=fakemag(2)
	rmag2=fakemag(3)
	bmag2=fakemag(4)
c	write(6,*) ' line ',iline,rmag1,bmag1,rmag2,bmag1
	if (rmag1.gt.0.01) then
	  smag(ii)=rmag1
	else if (rmag2.gt.0.01) then
	  smag(ii)=rmag2
	else if (bmag1.gt.0.01) then
	  smag(ii)=bmag1
	else if (bmag2.gt.0.01) then
	  smag(ii)=bmag1
	end if
      if( (smag(ii).lt.m1).or.(smag(ii).gt.m2) ) then
         ii=ii-1
c         print *,'rejecting ',smag(ii),sra(ii),sdec(ii)
      else
c         print *,ii,iline,' keeping ',smag(ii)
      endif 
      ii=ii+1
 890 	continue
      enddo
      nstars=ii-1
       do ii=1,nstars
          print *,ii,sra(ii),sdec(ii),smag(ii)
       enddo
c      print *,m1,m2,nstars
       close(unit=11)
	return
      end

c////////////////////////////////////////////////////////////////////
c  Gilbank''s subroutine for finding bright stars from USNO 
c   need to be connected to the network and running wget
c     ----------------------------------------------------------------
c
      subroutine queryusnob2(ra,dec,radius,m1,m2,nstars,sra,sdec,smag)
      character url*58,cra*8,cdec*8,cradius*5,query*119,junk*8,id*12
      character*133 ar
      real ra,dec,radius,smag(1000),m1,m2,junkreal,smagb(1000)
      double precision sra(1000),sdec(1000)
      integer nstars
      integer stat,ii,ios,junkint
	nstars=0
c     Search within radius [arcmins] of ra,dec and mag limits: m1-->m2
c     return nstars, with coords sra,sdec and mags smag
c     return r_mag by default. if r is not present use b_mag
c     [no colour correction from b-->r, as yet]
c -- Take RA, Dec (in degrees), search radius and retrieve catalogue of USNO-B1 
c      objects in field
c     This will work on any PC running wget (fairly standard linux)
c     -- check for wget
      call system('wget --version >& /dev/null',stat)
      if(stat.ne.0) then 
        write(6,*) ' ++ you do not have wget installed, search aborted'
        nstars=0
        return
      endif
c     --
c     -- Build query (for CADC)
      url='http://vizier.hia.nrc.ca/viz-bin/asu-acl?-source=USNO-B1.0'
      write(cra,fmt='(f8.4)') ra
      write(cdec,fmt='(f8.4)') dec
      write(cradius,fmt='(f5.2)') radius
	write(6,*) ' RA: ',cra,'  Dec ',cdec,'  radius(arcmin): ',radius
      query=url//'&-c.ra='//cra//'&-c.dec='//cdec//'&-c.rm='//cradius//
     *     '&-out.max=10000'
c      print *,query
c     -- execute call to wget and place output table in /tmp/temp.TAB
      call system("wget '"//query//"' -O /tmp/temp.TAB")
c     --
c     -- Read in catalogue [header is junk. last line of junk starts '--------'
c        then format is ID, RA, Dec,  4*junk, R1mag, B2mag, R2mag, Imag
c
c     this is horrendous due to the formatting of USSO cat!!!!
c
c     need to apply magnitude cuts here
      junk='zzzzzzzz'
      open(unit=11,file='/tmp/temp.TAB',status='old',form='formatted')
      do while(junk.ne.'--------')
         read(unit=11,fmt=*) junk
c         print *,junk
      enddo
c     now read real data
      ios=0
      ii=1
      id='XXXXXX'
      do while(ar(1:5).ne.'[EOD]')
         read(11,fmt='(A133)')ar
c         print *,ar
c         print *,ar(85:90), '---',ar(90:95)
         read(ar,fmt='(A12,1X,F10.6,1X,F10.6)') id,sra(ii),sdec(ii)
c         if(ar(85:85).eq.'1') print *,'not okay'
c     check for valid number in magnitude column
c       start from 85 NOT 86, as mags < 10. will have ' ' at 85!!!!
         if( (ar(86:86).eq.'1').or.
     *        (ar(86:86).eq.'2').or.(ar(86:86).eq.'3').or.
     *        (ar(86:86).eq.'4').or.(ar(86:86).eq.'5').or.
     *        (ar(86:86).eq.'6').or.(ar(86:86).eq.'7').or.
     *        (ar(86:86).eq.'8').or.(ar(86:86).eq.'9') )
     *    read(ar(85:90),fmt='(F5.2)') smagb(ii)
c         if(ar(85:86).ne.'') read(ar(85:90),fmt='(F5.2)') smagb(ii)
c         if (ar(90:95).ne.'') read(ar(90:95),fmt='(F5.2)') smag(ii)
         if( (ar(90:90).eq.'1').or.
     *        (ar(90:90).eq.'2').or.(ar(90:90).eq.'3').or.
     *        (ar(90:90).eq.'4').or.(ar(90:90).eq.'5').or.
     *        (ar(90:90).eq.'6').or.(ar(90:90).eq.'7').or.
     *        (ar(90:90).eq.'8').or.(ar(90:90).eq.'9') ) then
            read(ar(90:95),fmt='(F5.2)') smag(ii)
         else
            smag(ii)=smagb(ii)  ! **** r_mag not present, switch to b
         endif
c         if (ar(90:95).eq.'') then 
c            smag(ii)=smagb(ii)  ! **** r_mag not present, switch to b
c                                ! might want to do some average colour correction
c         endif
c            read(ar,fmt=22) id,sra(ii),sdec(ii),
c     *           junkreal,junkint,junkint,junkreal,junkint,junkint,
c     *       junkint,
c     *           smagb(ii)      !,smag(ii)
c            smag(ii)=smagb(ii) ! **** r_mag not present, switch to b
c                               ! might want to do some average colour correction
c         else
c            read(ar,fmt=23) id,sra(ii),sdec(ii),
c     *           junkreal,junkint,junkint,junkreal,junkint,junkint,
c     *           junkint,
c     *           smagb(ii),smag(ii)
c         endif
cc         print *,id,sra(ii),sdec(ii),smag(ii)
c
cc 22   format(A12,4x,F10.6,5x,F10.6,I3,I3,F6.1,I6,I6,I1,F5.2,F5.2)
c 22   format(
c     +        A12,1X,F10.6,1X,F10.6,1X,F5.2,1X,I3,1X,I3,1X,F6.1,1X,I6,
c     +        1X,I6,1X,I1,1X,F5.2)!,1X,F5.2)
c
c 23   format(
c     +        A12,1X,F10.6,1X,F10.6,1X,F5.2,1X,I3,1X,I3,1X,F6.1,1X,I6,
c     +        1X,I6,1X,I1,1X,F5.2,1X,F5.2)
c      print *,id,sra(ii),sdec(ii),
c     *        junkreal,junkint,junkint,junkreal,junkint,junkint
c     +  I1,1X,I3,1X,I3)
c     1X,I1,1X,I1,1X,I1,1X,A3,2X,F5.2,1X,I1,1X,I1,A1,
c     +  I3,1X,I2,1X,I4,I4,2X,F5.2,1X,I1,1X,I1,A1,I3,1X,I2,1X,I4,I4,2X,
c     +  F5.2,1X,I1,1X,I1,A1,I3,1X,I2,1X,I4,I4,2X,F5.2,1X,I1,1X,I1,A1,
c     +  I3,1X,I2,1X,I4,I4,2X,F5.2,1X,I1,1X,I1,A1,I3,1X,I2,1X,I4,I4)
      if( (smag(ii).lt.m1).or.(smag(ii).gt.m2) ) then
         ii=ii-1
c         print *,'rejecting ',smag(ii)
      else
          print *,ii,'keeping ',smag(ii)
      endif 
      ii=ii+1
      enddo
      nstars=ii-1
       do ii=1,nstars
          print *,sra(ii),sdec(ii),smag(ii)
       enddo
       print *,m1,m2,nstars
       close(unit=11)
	return
      end

c/////////////////////////////////////////////////////////
c  subroutine to pipeline object finding
c     done on extention cext
c     option of: summing 2 frames (nobf1 nobf2), 
c                 in the order of image names in filestr
c                mask frame names in maskstr
c
c  Aug 17 2008; changed the way masking is done
c    and save the exact mask used for objfind in frame 4 for save later
	subroutine pipeobjf(nobf1,nobf2,filestr,maskstr,
     *            nfilm,ib,ie,ibm,iem,ix,nu,nv,kf,cext)
	character filestr*60,maskstr*60,dumstr*60,maskname*60,cext*2
     *        ,filent*60,filen*24,primname*60,tempname*24
	dimension nfrar(5),ib(6),ie(6),iem(6),ibm(6)
	integer ix(nu,nv,kf)
	common/bstarxy/nbstar,bstx(20),bsty(20),bsrad(20),bsmag(20)
	common/satstars/nsatstars,ixsats(1000),jysats(1000)
	include 'pppcore.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'ppparm.inc'
	include 'objcat.inc'
	include 'picdir.inc'
        common/imagestats/imstatspar(50),ich_id,ch_rms(50),ch_sky(50),
     *   nbstar_ch(50),ch_bsmag(15,50),ch_bsx(15,50),ch_bsy(15,50)
c prepare image
	filen=filestr(ib(nobf1):ie(nobf1))
	filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	read(cext,*,err=998) ich_id 
c save the primary file name for making output names
	primname=filen(1:index(filen,'_')-1)
c  NOTE: nobf1 and nobf2 are the ordinal number of the files in the
c   filestr string; the images to be operated on in ppp is always
c    1 for obj finding, 2 for averaging, and 2 or 3 for masks
	call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,0)
c check if there really is data on the chip, by assume a chip
c  with all zeros is an empty chip
 800	call chzeros(*8001,*999,1,ix,nu,nv,kf,ichzero)
 8001	call chkwcshead(1,filen,cext)
c  fill negatives
	call smspars(nfilref,filent)
	irplim=-2
	irprep=nint(saturate(1))+500
	icomp=0
	nsatstars=0
	call getadusat(adusat,saturate(1),adusatpipe)
	write(6,*) '  ^^^^ rple lim/repl, adusat ',irplim,irprep,adusat
	call searchpix(*8002,1,icomp,' ',' ',1,ix,nu,nv,kf)
	write(6,*) ' ^^^^ randomize saturated pixels'
c8002	call fixsatpix(*801,1,isatpixpipe,ix,nu,nv,kf)
 8002	call fixsatpix(*801,1,int(saturate(1)),ix,nu,nv,kf)
c make a copy to 4
 801	continue
	call move(1,4,ix,nu,nv,kf)
c
c  get mask for primary frame: read or make mask
    	if (nfilm.gt.0) then
c  case 1, read mask from disk
	  maskname=maskstr(ibm(nobf1):iem(nobf1))
 	  call getmasknam(nobf1,ibm,iem,maskstr,maskname)
 	  if (maskname.ne.' ') then
	    call rdkfits(*802,3,maskname,irep,ix,nu,nv,kf,ir,0)
c  no need to fix image!
 802	    write(6,*) '  ^^^ read in mask for primary image'
	  end if
	else if (nfilm.eq.-1) then
c case 2: mask is created (the usual mode); "createmask" use zeros in the
c        ccd image to make the mask (CFHT Elixir standard).
	  call createmask(ix(1,1,1),ix(1,1,3),nu,nv,
     *                    nii(1),njj(1),nii(3),njj(3),0)
c jul28 2008, the image is initially fixed for cosmetic bad pixels;
c   with this, it is not longer necessary to include these bad
c    pixels in subsequence actions.
  	  call bpfix(*803 ,1,3,0,ix,nu,nv,kf)
 803 	  write(6,*) ' ^^^^ bad pixel mask created from primary image '
	else if (nfilm.eq.0) then
c  read in dummy mask, called dummymask.fits
	  dumstr='dummymask'
	  call rdkfits(*8031,3,dumstr,irep,ix,nu,nv,kf,ir,0)
 8031	  write(6,*) '  ^^^ read in dummy mask for primary image'
	end if
c set mask frame to 3  (this is hardwired at this point)
	imaskpipe=3
c ........... adding bright star mask, for saturated object finding (not final)
c
c  find bright stars using star catalog (wcs solution from frame 1)
c      bsmag1 and bsmag2 are global ppparm set parameters.
c      The positions are stored and masks created when doing sky
 	if (imstatspar(ich_id).eq.1) then
c   get the bright star from the common block
          nbstar=nbstar_ch(ich_id)
	  do 160 k=1,nbstar
            bstx(k)=ch_bsx(k,ich_id)
            bsty(k)=ch_bsy(k,ich_id)
            bsmag(k)=ch_bsmag(k,ich_id)
 160	  continue
	else 
	  call usnodri(*808,1,kf,bsmag1,bsmag2,igetstar)
	end if
c
c quick object finding for positional reference (qfnd) (frame 1)
c  used of averaging for obj finding, and also for shifting for photometry
c  aug 2008: not sure why bright stars are found before this. Are they
c   needed for computing sky?  Bright stars are not masked here.
 808	if( ichipsh.eq.2) then
	  write(6,*) ' ^^^ qfnd ','  jlpos ',jlpos,ilpos
	  call FINOBD(*809,1,dumstr,dumstr,0.00,7,IX,NU,NV,KF)
c  search for reference stars from the quickfind list, and store
 809	  write(6,*) ' ^^^ arf '
	  call autoref(1,ix,nu,nv,kf,0,*8091)
 8091	  continue
	end if
c
c  find saturated stars (using frame 1 only), 
c   ++ this should be improved by having saturation masks from both frames
c    and then shift and sum
 	call mkoutname(primname,1,cext,'ss')
	call mkoutcomm(primname,1,cext,'ss')
	dumstr=poutname(1:lnblnk(poutname))
c set thersky=-100 for saturated stars, save original value in tthresky
	tthresky=thresky
	thresky=-100.
c temporarily increase iir and ior
	iir=iir+4
	ior=ior+10
 	write(6,*) 
     *   ' ^^ making qk bright star mask for saturated stars finding --'
c  create a bleeding column mask if there are very bright stars (m<9.0)
c    added Dec 2005
	mkbsmask=0
	if (nbstar.gt.0) then
	  do 167 kj=1,nbstar
	    if (bsmag(kj).le.9.0) mkbsmask=1
 167	  continue
c  if there are bright stars, create a mask to block out serious big
c     areas of saturated pixles.  Note that for the purpose of finding
c     saturated stars, this is purely to prevent too many saturated
c     pixels from being identified as "stars", saturated pixles are
c    still needed to identify saturated stars.
c
	  if (mkbsmask.eq.1) then
	    write(6,*) '  +++ very bright stars found --making mask'
c make a quick and dirty bright star saturation mask on frame 3
c modified Apr 17 2008: the bright star mask is made on top of the
c   bad column mask, rather than wiping it out (as it did before)
c    - note that for saturated star, it continues to use "qkbsmask"
c      for masking, and interpolate over.  The new star ghost cannot
c      be used (and not needed), since it would wipe out the saturated
c      star in the first instance.
	    call qkbsmask(nii(1),njj(1),3,cext,ix,nu,nv,kf,1)
c  fix the image before finding saturated stars
 	    call bpfix(*8092,1,3,0,ix,nu,nv,kf)
 8092	    continue
	  end if
	end if
 	call finobd(*810,1,dumstr,poutcomm,0.00,-2,IX,NU,NV,KF)
c reset values
 810	thresky=tthresky
	iir=iir-4
	ior=ior-10
c save the saturated stars in an array to be merged later with
c   the object finding file
	nsatstars=nob(1)
	do 180 i=1,nob(1)
	  ixsats(i)=ijoi(i,1)
	  jysats(i)=ijoj(i,1)
 180	continue
c  preparing for final object finding for the case of using 2 images
c create average if nobf2=2, read image into 2
c   NOTE this needs to be worked on still - masking is not done properly
	if (nobf2.gt.0) then
	  write(6,*) ' *** summing filter images ',nobf1,nobf2,
     *                   ' for object finding'
	  icomp=0
	  filen=filestr(ib(nobf2):ie(nobf2))
	  filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	  call rdkfits(*804,2,filent,irep,ix,nu,nv,kf,ir,0)
 804	  irplim=-10
	  irprep=nint(saturate(2))
c  smspars here should take the filter number of the 2nd filter, not frame 2
	  call smspars(2,filent)
 	  call searchpix(*805,2,icomp,' ',' ',1,ix,nu,nv,kf)
 805	  continue
c  shift 2nd image
	  if (ichipsh.eq.1) then
c  copy shift from computed shift array
	    dxmean=pmxshift(noblref,jchip)
	    dymean=pmyshift(noblref,jchip)
c computing matching transform between 1 and 2, using ref stars created
	  else if (ichipsh.eq.2) then
c  from the last call of autoref (stored in array).
c
c  first compute roughshift using wcs
 	    if (iwcssh.eq.1.and.iwcs.eq.1) then
	      call wcsshift(1,1,iwcshift,jwcshift,2)
	      iinitpipe=iwcshift
	      jinitpipe=jwcshift
	      write(6,14) iinitpipe,jinitpipe
 14	format('  ^^^ estimated shifts from wcs: ',2i5)
	    else
	      iinitpipe=0
	      jinitpipe=0
	    end if
	    write(6,*) '  ^^^^ computing shifts between filters 1 & 2'
            call simposmat(2,iir,iret,1,1,ix,nu,nv,kf,*811) 
	  end if
c  averaging two frames before object finding
 811	  nfrar(1)=1
	  nfrar(2)=2
c  result is overwritten in 1 (which will be used for object fin
	  nfrar(3)=1
	  nfrar(4)=0
	  nfrar(5)=0
	  write(6,*) '  ^^^^ wtav, weighted (unfixed) average '
	  call combine2(*812,nfrar,ix,nu,nv,kf)
c
c  now make a combined frame 1 and frame 2 mask, frame 1 mask is still
c    at 3
c
c  case 1: read in mask
 812	  if (nfilm.gt.0) then
	    call getmasknam(nobf2,ibm,iem,maskstr,maskname)
	    write(6,*) ' ^^^ read in mask for frame 2: ',maskname
 	    call rdkfits(*806,2,maskname,irep,ix,nu,nv,kf,ir,0)
c shift mask and add to frame 3
 806	    call shiftp(*807,2,0.,0.,ix,nu,nv,kf,2)
 807	    call sumpic(*8071,2,3,3,0,ix,nu,nv,kf)
 8071	    continue
	  else if (nfilm.eq.-1) then
c  case 2: 
c shift image 2, and create mask by adding to frame 3 mask
	    call shiftp(*807,2,0.,0.,ix,nu,nv,kf,2)
 8072	    call createmask(ix(1,1,1),ix(1,1,3),nu,nv,
     *               nii(2),njj(2),nii(3),njj(3),1)
	  end if
c note that if dummy= mask, it is already read
        else
c  for using a single frame for object finding
c   Aug 17 2008, read in image again, and redo all the masking
c     from start
          call move(4,1,ix,nu,nv,kf) 
	  write(6,*) '  ++ remaking mask for final object finding '
	  call createmask(ix(1,1,1),ix(1,1,3),nu,nv,
     *                    nii(1),njj(1),nii(3),njj(3),0)
	end if
c
c  +++ final object finding ++++
c  make bleeding column mask, add to frame 3
c  set adusat to .9 saturation to account partially for averaging 
c   note: bleedmap connects pixel> 1/2 (adusat-sky) + sky
c     other masking to be done in object finding subroutine
	adusat=0.9*saturate(1)
	if (maskstr.ne.' ') then
c make saturation column and defraction spike mask on existing mask image
c  create new mask (since the old one has quick bright star mask)
c      note that the image already has the bad columns fixed
 	  call bleedmap(*813,1,3,ix,nu,nv,kf,0,0,0)
	else	
	  write(6,*) ' ^^^ adding bleeding column map to bp mask '
c new image
 	  call bleedmap(*813,1,3,ix,nu,nv,kf,0,0,0)
	end if
c fix the image (for bad column and bleeding column) (Is this needed?)
c    (it does help in background computing during obj finding>)
c   Aug 17, 2008; however, it makes fake objects at the edge of bad columns
c      because of the interpolation
c813	call bpfix(*8131,1,3,0,ix,nu,nv,kf)
c  add bright stars to the mask, with 2nd masking criteria set
 813 	call bstarmask(3,1,ix,nu,nv,kf,cext,1,2,0)
c  save average image to 2, for later cosmic ray rejection
c       write(6,*) '   ^^^ copying image to frame 2 for save'
c  commands to write out interim images for checking
c	tempname='tempfix'//cext(1:lnblnk(cext))//'.fits'
c	tempname='tempmask'//cext(1:lnblnk(cext))//'.fits'
c	call qkimageout(1,nii(1),njj(1),ix,nu,nv,kf,tempname)
c	call qkimageout(3,nii(3),njj(3),ix,nu,nv,kf,tempname)
        if (nobf2.eq.2) call move(1,2,ix,nu,nv,kf)
c  do a tapered smoothing (note the ismtyepipe parameter is opposite
c   of the smooth ipar parameter, to keep consistency with iobjsm
	if (ismtypepipe.eq.1) then
	  write(6,*) '  ^^^^ taper smooth image '
	  call smooth(*814,1,ismthlenp,0,ix,nu,nv,kf)
	else
	  write(6,*) '  ^^^^ box smooth image '
	  call smooth(*814,1,ismthlenp,1,ix,nu,nv,kf)
	end if
c find object
 814	call mkoutname(primname,1,cext,'of')
	call mkoutcomm(primname,1,cext,'of')
	dumstr=poutname(1:lnblnk(poutname))
	write(6,*) '  ^^^^ final object finding '
	write(6,*) ' .. Out put name and comments: ',
     *               poutname(1:lnblnk(poutname))
	write(6,*) '        ',poutcomm(1:lnblnk(poutcomm))
	adusat=saturate(1)-500.
c still need to get the mask properly
 	call finobd(*817,1,dumstr,poutcomm,0.00,-1,IX,NU,NV,KF)
 817	write(6,*) '  ^^^^ copying original image and cr reject'
        if (nobf2.eq.0) then
	  call move(4,1,ix,nu,nv,kf)
        else
	  call move(2,1,ix,nu,nv,kf)
        end if
	call rchkpos(*818,1,ix,nu,nv,kf)
c save final pos file
 818	call mkoutcomm(primname,1,cext,'cr')
c  add saturated stars if they are not found by objfinding
c   Note this is done after rejecting sharp objects, due to
c     some saturated objects have sharp peaks
	call addsatstars(1,ix,nu,nv,kf)
c  out put cosmic rejeceted position file
    	call ptype(*999,1,poutname,poutcomm,0,0,2,ix,nu,nv,kf)
c do cosmic ray reject
 999	return
 998	write(6,*) ' *** ILLEGAL chip number string ',cext,' ABORTED'
	return
	end
c/////////////////////////////////////////////////////////
c  subroutine to pipeline photometry
c  jan11, 2008, added parameter ifilbeg in case the reference
c             filter photometry is not needed (for adding new filter)
	subroutine pipephot(filestr,maskstr,nfilm,ib,ie,ibm,iem,
     *                   ifilbeg,ix,nu,nv,kf,cext)
	integer nfra(5),ib(6),ie(6),ibm(6),iem(6)
	character filestr*60,maskstr*60,maskname*60,dumstr*60,
     *        filen*24,filent*60,cext*2,temname*60,primname*60,
     *        posfname*60,satposname*60,refilname*60
	integer ix(nu,nv,kf)
	include 'pppcore.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'picdir.inc'
	include 'objcat.inc'
	include 'ppparm.inc'
        common/imagestats/imstatspar(50),ich_id,ch_rms(50),ch_sky(50),
     *   nbstar_ch(50),ch_bsmag(15,50),ch_bsx(15,50),ch_bsy(15,50)
	nfra(1)=1
	dumstr='dummy'
	idum=0
	imaskpipe=2
	iastromtemp=iastrom
	read(cext,*,err=901) nchip
	ich_id=nchip
	do 110 i=2,5
	  nfra(i)=0
 110	continue
c do for each filter
c	write(6,*)  ' IN PHOTPIPE # of filters: ',nfilt
c	do 100, i=ifilbeg,nfilt
	do 100, i=1,nfilt
	  if (i.lt.iflbeg) then
	    write(6,*) ' +++reading header for chip ',cext,' filter ',i
	  else
	    write(6,*) ' @@@@ Photometry for chip ',cext,' filter ',i
	  end if
	  filen=filestr(ib(i):ie(i))
c  primname is the root name of the pointing (without _fitler and chip number)
	  primname=filen(1:index(filen,'_')-1)
c save the reference filter name for future use
	  filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	  if (i.eq.nfilref) refilname=filent
c read image
	  write(6,*) '  ^^^^ reading image: ',filent
	  call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,0)
c check for .head files, which, if found, replaces the wcs solution
c   read from the .fits file
 800	  call chkwcshead(1,filent,cext)
c copy filter header info into into wcs common block 
	  call smspars(i,filent)
c if this is a filter already done then just get the header info
c          and go on to the next filter
	  if (i.lt.ifilbeg) then    
c  create .phm file output name and comment
 	    call mkoutname(primname,1,cext,'ph')
 	    temname=poutname(1:lnblnk(poutname))
	    write(6,*) '   ^^^^ phm name for hdr file',temname
	    go to 812
	  end if
	  call chzeros(*8001,*999,1,ix,nu,nv,kf,ichzero)
c read mask
 8001	  call getadusat(adusat,saturate(1),adusatpipe)
	  if (nfilm.ge.1) then
	    call getmasknam(i,ibm,iem,maskstr,maskname)
	    write(6,*) '  ^^^^ reading mask: ',maskname
	    call rdkfits(*801,2,maskname,irep,ix,nu,nv,kf,ir,0)
	  else if (nfilm.eq.0) then
	    dumstr='dummymask'
	    call rdkfits(*801,2,dumstr,irep,ix,nu,nv,kf,ir,0)
	  else if (nfilm.eq.-1) then
c create mask from image
	    call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	  write(6,*) ' ^^^^ created mask from image'
	  end if
c  fill zeros
 801	  irplim=-10
	  irprep=nint(saturate(1))
	  icomp=0
	  call searchpix(*802,1,icomp,' ',' ',1,ix,nu,nv,kf)
c  also create saturation and regular posfile name
 802	  call mkoutname(primname,1,cext,'ss')
	  satposname=poutname(1:lnblnk(poutname))
	  call mkoutname(primname,1,cext,'of')
	  posfname=poutname(1:lnblnk(poutname))
c  create shift 
          if (ichipsh.eq.1) then
c  use existing shift, no need for finding ref stars for nfilref
c   (note shifting is not done for reference filter image)
	     if (i.ne.nfilref) then
	       dxmean=pmxshift(i,jchip)
	       dymean=pmyshift(i,jchip)
	       deltheta=pmdeltath(i,jchip)
	     else
	       dxmean=0.
	       dymean=0.
	     end if
c	  else if (ichipsh.eq.2) then
c   for ichipsh=2, use wcs transformation; to be done after reading posfile
c    implemented Jun06 2008, 
c         read wcs solution for the master list filter, put in frame 3
c  need to implement
c   	    call chkwcshead(3,filent,cextmaster)
c    transform position files from master to current filter

c    check goodness of match

	  else if (ichipsh.eq.3) then
c shift is done for each filter/chip as photometry is done
            if (i.eq.nfilref) then
c create the ref position file (assume the ref image is always the 1st one
c   this uses the reference image and save reference objs positions
c    in common/refobj/. (No shift is done here)
 	      write(6,*) ' ^^^ qfnd for primary filter'
	      call FINOBD(*803,1,dumstr,dumstr,0.00,7,IX,NU,NV,KF)
 803	      write(6,*) ' ^^^ arf for position reference '
	      call autoref(1,ix,nu,nv,kf,0,*804)
 804	      continue
	    else
 	      write(6,*) ' ^^^ pos matching for filter',i
c apply wcs solution as an initial correction 
 	      if (iwcssh.eq.1.and.iwcs.eq.1) then
c the operating frame is always frame 1
	        call wcsshift(1,nfilref,iwcshift,jwcshift,2)
	        iinitpipe=iwcshift
	        jinitpipe=jwcshift
	        write(6,14) iinitpipe,jinitpipe
 14	format('  ^^^ estimated shifts from wcs: ',2i5)
	      else
	        iinitpipe=0
	        jinitpipe=0
	      end if
c  store the wcs shifts
	      xchshift(i,nchip)=float(iinitpipe)
	      ychshift(i,nchip)=float(jinitpipe)
c  determine shift between filter i and primary
	      call simposmat(1,iir,iret,1,1,ix,nu,nv,kf,*8041) 
c  the matching result parameter is stored
 8041	      imatch(i,nchip)=iret
c  Nov, 2005: determine if photometry should go ahead
	      if (imatch(i,nchip).lt.0) then
	        write(6,8042) nchip,i
 8042	format('   ***** WCS matching problem for Chip ',i2,
     *         ' filter ',i2,' skipped, to be re done')
	        goto 100
	      end if
	    end if
	  end if
c  make bleeding column mask, first read in saturation pos file
c     first make name, and save for other filters
c         use saturated pos file for reference filter, but total
c             pos file for others
	  if (i.eq.nfilref) then
 	    write(6,*) ' ^^^ reading saturated pos file ',satposname
 	    call rpof(*806,1,satposname,idum,0)
	  else
c  if not the primary file, shift the pos file
 	    write(6,*) ' ^^^ reading pos file for bleeding colm and phot' 
     *                    ,posfname
 	    call rpof(*805,1,posfname,idum,0)
 805	    continue
	    if (ichipsh.ne.3) then    
c  added Aug 19 2008:  different options of shifting
c    (1) imatch>0 and delta-theta<0.01, use xy shift only
c    (2)              delta-theth>0.01, use xy+ small angle rotation
c    (3) imatch<0   : use wcs fshift, and if fails again, use
c                      explicit rotate+shift fit (Bauching''s program
	   write(6,*) ' -- shifting data: imatch ',imatch(i,nchip),
     *               '  rotation ',deltheta*57.296
c  ipostr=1 is the normal case (others are for testing)
	      ipostr=1
	      if (imatch(i,nchip).ge.0) then
	        if (abs(deltheta*57.296).le.0.01) then
c  this is the normal case for no rotation
	          call pfshift(*806,1,0,0,dumstr,1)
	        else
	          if (ipostr.eq.1) then
 	          write(6,*) ' -- shifting include small angle rotation'
     *                ,dxmean,dymean,deltheta
                     do 175 ik=1,nob(1)
	               iorig=ijoi(ik,1)
                       ijoi(ik,1)=ijoi(ik,1)+
     *                        nint(dxmean-ijoj(ik,1)*deltheta)
                       ijoj(ik,1)=ijoj(ik,1)+
     *                        nint(iorig*deltheta+dymean)
 175	            continue
	          else if (ipostr.eq.2) then
 	            write(6,*) ' -- Using WCS solutions for shifting'
 	            call pfshiftwcs(1,nfilref)
	          else if (ipostr.eq.3) then
c  Bauching''s fit ; 
 	            write(6,*) ' -- Using fitted shift+rotate transform'
	            call pfshiftfit(1,posfname,refilname,ix,nu,nv,kf)
	          end if
	        end if
	      else
c for the case of bad match  (.mat.FAIL)
	        if (ipostr.eq.2) then
	          write(6,*) ' -- Using WCS solutions for shifting'
 	          call pfshiftwcs(1,nfilref)
	        else 
 	          write(6,*) ' -- Using fitted shift+rotate transform'
	          call pfshiftfit(1,posfname,refilname,ix,nu,nv,kf)
	        end if
	      end if
c   save transformed pos file for checking
c   this is activated for debugging only
c	      call mkoutname(primname,1,cext,'tr')
c	      call mkoutcomm(primname,1,cext,'tr')
c   	      call ptype(*851,1,poutname,poutcomm,0,0,2,ix,nu,nv,kf)
c851	      continue
	    else
c	      call wcsshift()
	    end if
	  end if
 806	  write(6,*) '  ^^^^ making bleeding mask, added to bp mask'
c first remove diffraction spike parameters
	  dsct1=dsc(1)
	  dsct2=dsc(2)
	  dsct3=dsc(3)
	  dsc(1)=-999
	  dsc(2)=0.
	  dsc(3)=0.
 	  call bleedmap(*807,1,2,ix,nu,nv,kf,0,0,0)
c fix bad column, both bad columns and bleed columns
 807	  call bpfix(*808,1,2,0,ix,nu,nv,kf)
 808	  dsc(1)=dsct1
	  dsc(2)=dsct2
	  dsc(3)=dsct3
c  read position file (for refernece filter; for other filters, already read
	  if (i.eq.nfilref) then
	    write(6,*) ' ^^^ reading actual pos file ',posfname
 	    call rpof(*810,1,posfname,idum,0)
	  end if
c create cosmic ray map and fix
 810	write(6,*) '  ^^^  creating cosmic ray map and fix '
	  call cosraymap(*8101,1,2,' ',' ',nscrit,0,0,rwd,ix,nu,nv,kf)
c pad the CR map
 8101	  call padmap(*8102,2,ix,nu,nv,kf,0)
 8102	  call bpfix(*8103,1,2,0,ix,nu,nv,kf)
c create classification reference star file
c   first make name
 8103	  call mkoutname(primname,1,cext,'rf')
c  save the name for reference
	  arefname=poutname
	  write(6,*) '  ^^^^ aref: finding psfs'
	  adulowpipe=adulowfil(i)
c for reference filter, use imagestats background for autoref
	  if (i.eq.nfilref) then
	    ich_id=nchip
	  else
	    ich_id=0
	  end if
 	  call autoref(1,ix,nu,nv,kf,1,*811)
c do photometry
c  create .phm file output name and comment
 811	  call mkoutname(primname,1,cext,'ph')
	  call mkoutcomm(primname,1,cext,'ph')
 	  temname=poutname(1:lnblnk(poutname))
	  write(6,*) '   ^^^^ running ppphot ',temname
	  if (i.eq.nfilref) then
c get astrometry
	    iastrom=iastromtemp
	  else
	    iastrom=-1
	  end if
	  call pppdri(*812,nfra,temname,poutcomm,0,ix,nu,nv,kf)
c save mspos hdr parameters
 812	  call smspars(i,temname)
 100	continue
	iastrom=iastromtemp
 999	return
 901	write(6,*) ' ***chip number text read error ',cext,
     *             '  *** skipping to next chip'
	return
	end
c////////////////////////////////////////////////
c  driving routine to make a mask using subroutine createmask
c   ipar=0: create new mask frame
c        1: add to existing mask frame
c
	subroutine mkmask(*,nfr1,nfr2,ix,nu,nv,kf,ipar)
	include 'picdir.inc'
	integer ix(nu,nv,kf)
	call querynfr(nfr1,kf,0,*999,*2001)
 2001	call querynfr(nfr2,kf,4,*999,*2002)
 2002	call createmask(ix(1,1,nfr1),ix(1,1,nfr2),nu,nv,
     *         nii(nfr1),njj(nfr1),nii(nfr2),njj(nfr2),0)
	nfl(6,nfr2)=1
 999	return1
	end 
c//////////////////////////////////////////////////////////////
c  create mask from data by assuming zero pixels are bad pixels  
c
c   ipar=0: create new mask frame
c        1: add to existing mask frame
	subroutine createmask(ixpic,ixmask,nu,nv,niip,njjp,niim,
     *	                     njjm,ipar)
	integer ixpic(nu,nv),ixmask(nu,nv)
	if (ipar.eq.0) then
	  do 100 j=1,njjp
	    do 101 i=1,niip
c  Aug 2008: some Elixir frames have bad pix marked as -.5 etc, 
c       change the bad pix mask to include pix with -1 value.
c  sep11 2008: found some bad pixels have value 1.xx
c	      if (ixpic(i,j).eq.0) then
c	      if (ixpic(i,j).eq.0.or.ixpic(i,j).eq.-1) then
 	      if (ixpic(i,j).le.2.and.ixpic(i,j).ge.-2) then
	        ixmask(i,j)=1
	      else
	        ixmask(i,j)=0
	      end if
 101	    continue
 100	  continue
	  niim=niip
	  njjm=njjp
	else if (ipar.eq.1) then
	  if ((niim.ne.njjm).or.(niip.ne.njjp)) then
	    write(6,*) '   +++ warning: frame size not equal ',
     *               'image: ',niip,njjp,' mask: ',niim,njjm
	  end if
	  do 110 j=1,njjp
	    do 111 i=1,niip
c	      if (ixpic(i,j).eq.0) then
c	      if (ixpic(i,j).eq.0.or.ixpic(i,j).eq.-1) then
 	      if (ixpic(i,j).le.2.and.ixpic(i,j).ge.-2) then
	        ixmask(i,j)=1
	      end if
 111	    continue
 110	  continue
	end if
	return
	end
c//////////////////////////////////////////////////////////
c
c   subroutine to return output comment, based on root name
c
	subroutine mkoutcomm(root,nfr,cext,type)
	character root*60,cext*2,type*2 
	include 'pipeparm.inc'
	include 'imagepar.inc'
	poutcomm=root(1:lnblnk(root))//' chip '//cext(1:lnblnk(cext))
	if (type.eq.'ph') then
c  add filter name
	  poutcomm=poutcomm(1:lnblnk(poutcomm))//' '//filter(nfr)
	else if (type.eq.'ss') then
	  poutcomm=poutcomm(1:lnblnk(poutcomm))//' '//filter(nfr)//
     *        ' Saturated stars'
	else if (type.eq.'cr') then
	  poutcomm=poutcomm(1:lnblnk(poutcomm))//' '//filter(nfr)//
     *        ' CR reject'
	else if (type.eq.'tr') then
	  poutcomm=poutcomm(1:lnblnk(poutcomm))//' '//filter(nfr)//
     *        ' transformed'
	else if (type.eq.'mk') then
	  poutcomm=poutcomm(1:lnblnk(poutcomm))//' '//filter(nfr)//
     *        ' mask'
	end if
c	write(6,*) '  mkoutcomm:  ',poutcomm(1:lnblnk(poutcomm))
c	write(6,*) '  filter   :  ',filter(nfr)
	return
	end
c//////////////////////////////////////////////////////////
c
c   subroutine to return output name, based on root name
c      the filter name is added via filter(nfr)
c
	subroutine mkoutname(root,nfr,cext,type)
	character root*60,cext*2,type*2
	include 'pipeparm.inc'
	include 'imagepar.inc'
	poutname=root(1:lnblnk(root))//'^'//cext(1:lnblnk(cext))
	if (type.eq.'ss') then
c for saturated star posfile,  add sat and filter name
	  poutname=poutname(1:lnblnk(poutname))//'sat'//filter(nfr)
	else if (type.eq.'rf'.or.type.eq.'ph') then
c  for .ref, .phm, and .pos files, add filter name
	  poutname=poutname(1:lnblnk(poutname))//filter(nfr)
	else if (type.eq.'mk') then
c name for mask fits file 
	  poutname=poutname(1:lnblnk(root))//filter(nfr)//
     *         '_'//cext(1:lnblnk(cext))//'mk'
	else if (type.eq.'tr') then
c name for transformed  position file  
	  poutname=poutname(1:lnblnk(poutname))//'trf'//filter(nfr)
	end if
 	write(6,*) '  mkoutname:  ',poutname
c	write(6,*) '  filter   :  ',filter(nfr)
	return
	end
c////////////////////////////////////////////////////////////
c
	subroutine getmasknam(nob,ibm,iem,maskstr,maskname)
	character maskstr*60,maskname*60 
	integer ibm(6),iem(6)
	maskname=maskstr(ibm(nob):iem(nob))
	return
	end
c////////////////////////////////////////////////////////////
c  subroutine to set rcs pipeline parameters
c
c   ipar=0:  set default values
c   ipar=1:  set an individual parameter
c   ipar=2:  type out parameters
	subroutine setrcspipe(*,str,str1,nfr,rn,ipar)
	character str*60,str1*60,var*12
	include 'pipeparm.inc'
	if (ipar.eq.0.or.str.eq.'reset'.or.str.eq.'default') then
	  ircspipe=0
 	  irplim=0
 	  irprep=0
	  iinitpipe=0
	  jinitpipe=0
	  iwcssh=1
	  iaprefpipe=39
	  iapmaxpipe=39
	  cdiampipe=5.	
	  ishmaxpipe=2
	  iapmaxpipe=41
	  imaskpipe=2
	  npadpipe=1
	  ismtypepipe=0
	  ismthlenp=3
	  nsmthlenp=3
	  isbpipe=15
	  jsbpipe=15
	  crsigpipe=6.
	  ncrexcpipe=0
	  isatpixpipe=64800
	  adusatpipe=0.
	  colapert=3.0
	  ichipsh=1
	  iccdgap=72
	  jccdgap=72
	  gappercent=1.
	  imaskout=1
	  igetstar=1
	  do 121 i=1,5
	    adulowfil(i)=0.1
 121	  continue
	  return1
	else if (ipar.eq.2.or.str.eq.'list') then
	  write(6,1)
 1	format('  RCS Pipeline SET parameters (command setp): ')
	  write(6,401) ircspipe
 	  write(6,402) irplim
 	  write(6,403) irprep
	  write(6,404) iinitpipe
	  write(6,405) jinitpipe
	  write(6,4051) iwcssh
	  write(6,406) imaskpipe
	  write(6,407) npadpipe
	  write(6,408) ismtypepipe
	  write(6,4081) ismthlenp
	  write(6,409) iaprefpipe
	  write(6,410) iapmaxpipe
	  write(6,411) cdiampipe
	  write(6,412) ishmaxpipe
	  write(6,413) nsmthlenp
	  write(6,414) isbpipe
	  write(6,415) jsbpipe
	  write(6,416) crsigpipe
	  write(6,417) ncrexcpipe
	  write(6,418) isatpixpipe
	  write(6,420) (adulowfil(k),k=1,5)
	  write(6,421) colapert
	  write(6,422) ichipsh 
	  write(6,423) adusatpipe
	  write(6,424) iccdgap
	  write(6,4241) jccdgap
	  write(6,425) gappercent
	  write(6,426) imaskout
	  write(6,427) igetstar
	else if (ipar.eq.1) then
	  var=str(1:lnblnk(str))
	  if (var.eq.'irplim') then
	    if (str1.eq.' ') then
	      write(6,402) irplim
 302	      write(6,400)
	      call read1i(irplim,*999,*302)
	    else
	      irplim=nfr
	    end if
	    write(6,402) irplim
	  else if (var.eq.'irprep') then
	    if (str1.eq.' ') then
	      write(6,403) irprep
 303	      write(6,400)
	      call read1i(irprep,*999,*303)
	    else
	      irprep=nfr
	    end if
	    write(6,403) irprep
	  else if (var.eq.'iinitpipe') then
	    if (str1.eq.' ') then
	      write(6,404) iinitpipe
 304	      write(6,400)
	      call read1i(iinitpipe,*999,*304)
	    else
	      iinitpipe=nfr
	    end if
	    write(6,404) iinitpipe
	  else if (var.eq.'jinitpipe') then
	    if (str1.eq.' ') then
	      write(6,405) jinitpipe
 305	      write(6,400)
	      call read1i(jinitpipe,*999,*305)
	    else
	      jinitpipe=nfr
	    end if
	    write(6,405) jinitpipe
	  else if (var.eq.'iwcssh') then
	    if (str1.eq.' ') then
	      write(6,4051) iwcssh
 3051	      write(6,400)
	      call read1i(iwcssh,*999,*3051)
	    else
	      iwcssh=nfr
	    end if
	    write(6,4051) iwcssh
	  else if (var.eq.'imaskpipe') then
	    if (str1.eq.' ') then
	      write(6,406) imaskpipe
 306	      write(6,400)
	      call read1i(imaskpipe,*999,*306)
	    else
	      imaskpipe=nfr
	    end if
	    write(6,406) imaskpipe
	  else if (var.eq.'npadpipe') then
	    if (str1.eq.' ') then
	      write(6,407) npadpipe
 307	      write(6,400)
	      call read1i(npadpipe,*999,*307)
	    else
	      npadpipe=nfr
	    end if
	    write(6,407) npadpipe
	  else if (var.eq.'ismtypepipe') then
	    if (str1.eq.' ') then
	      write(6,408) ismtypepipe
 308	      write(6,400)
	      call read1i(ismtypepipe,*999,*308)
	    else
	      ismtypepipe=nfr
	    end if
	    write(6,408) ismtypepipe
	  else if (var.eq.'ismthlenp') then
	    if (str1.eq.' ') then
	      write(6,4081) ismthlenp
 3081	      write(6,400)
	      call read1i(ismthlenp,*999,*3081)
	    else
	      ismthlenp=nfr
	    end if
	    write(6,4081) ismthlenp
	  else if (var.eq.'iaprefpipe') then
	    if (str1.eq.' ') then
	      write(6,409) iaprefpipe
 309	      write(6,400)
	      call read1i(iaprefpipe,*999,*309)
	    else
	      iaprefpipe=nfr
	    end if
	    write(6,410) iaprefpipe
	  else if (var.eq.'iapmaxpipe') then
	    if (str1.eq.' ') then
	      write(6,410) iapmaxpipe
 310	      write(6,400)
	      call read1i(iapmaxpipe,*999,*310)
	    else
	      iapmaxpipe=nfr
	    end if
	    write(6,409) iapmaxpipe
	  else if (var.eq.'cdiampipe') then
	    if (str1.eq.' ') then
	      write(6,411) cdiampipe
 311	      write(6,400)
	      call read1r(cdiampipe,*999,*311)
	    else
	      cdiampipe=nfr
	    end if
	    write(6,411) cdiampipe
	  else if (var.eq.'ishmaxpipe') then
	    if (str1.eq.' ') then
	      write(6,412) ishmaxpipe
 312	      write(6,400)
	      call read1i(ishmaxpipe,*999,*312)
	    else
	      ishmaxpipe=nfr
	    end if
	    write(6,412) ishmaxpipe
	  else if (var.eq.'nsmthlenp') then
	    if (str1.eq.' ') then
	      write(6,413) nsmthlenp
 313	      write(6,400)
	      call read1i(nsmthlenp,*999,*313)
	    else
	      nsmthlenp=nfr
	    end if
	    write(6,413) nsmthlenp
	  else if (var.eq.'isbpipe') then
	    if (str1.eq.' ') then
	      write(6,414) isbpipe
 314	      write(6,400)
	      call read1i(isbpipe,*999,*314)
	    else
	      isbpipe=nfr
	    end if
	    write(6,414) isbpipe
	  else if (var.eq.'jsbpipe') then
	    if (str1.eq.' ') then
	      write(6,415) jsbpipe
 315	      write(6,400)
	      call read1i(jsbpipe,*999,*315)
	    else
	      jsbpipe=nfr
	    end if
	    write(6,415) jsbpipe
	  else if (var.eq.'crsigpipe') then
	    if (str1.eq.' ') then
	      write(6,416) crsigpipe
 316	      write(6,400)
	      call read1r(crsigpipe,*999,*316)
	    else
	      crsigpipe=rn
	    end if
	    write(6,416) crsigpipe
	  else if (var.eq.'ncrexcpipe') then
	    if (str1.eq.' ') then
	      write(6,417) ncrexcpipe
 317	      write(6,400)
	      call read1i(ncrexcpipe,*999,*317)
	    else
	      ncrexcpipe=nfr
	    end if
	    write(6,418) ncrexcpipe
	  else if (var.eq.'isatpixpipe') then
	    if (str1.eq.' ') then
	      write(6,418) isatpixpipe
 318	      write(6,400)
	      call read1i(isatpixpipe,*999,*318)
	    else
	      isatpixpipe=nfr
	    end if
	    write(6,418) isatpixpipe
	  else if (var.eq.'adulow1'.or.var.eq.'adulowfil1') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(1)
 319	      write(6,400)
	      call read1r(adulowfil(1),*999,*319)
	    else
	      adulowfil(1)=rn
	    end if
	    write(6,419) adulowfil(1)
	  else if (var.eq.'adulow2'.or.var.eq.'adulowfil2') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(2)
 320	      write(6,400)
	      call read1r(adulowfil(2),*999,*320)
	    else
	      adulowfil(2)=rn
	    end if
	    write(6,419) adulowfil(2)
	  else if (var.eq.'adulow3'.or.var.eq.'adulowfil3') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(3)
 321	      write(6,400)
	      call read1r(adulowfil(3),*999,*321)
	    else
	      adulowfil(3)=rn
	    end if
	    write(6,419) adulowfil(3)
	  else if (var.eq.'adulow4'.or.var.eq.'adulowfil4') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(4)
 322	      write(6,400)
	      call read1r(adulowfil(4),*999,*322)
	    else
	      adulowfil(4)=rn
	    end if
	    write(6,419) adulowfil(4)
	  else if (var.eq.'adulow5'.or.var.eq.'adulowfil5') then
	    if (str1.eq.' ') then
	      write(6,419) adulowfil(5)
 323	      write(6,400)
	      call read1r(adulowfil(5),*999,*323)
	    else
	      adulowfil(5)=rn
	    end if
	    write(6,419) adulowfil(5)
	  else if (var.eq.'colapert') then
	    if (str1.eq.' ') then
	      write(6,421) colapert
 331	      write(6,400)
	      call read1r(colapert,*999,*331)
	    else
	      colapert=rn 
	    end if
	    write(6,421) colapert
	  else if (var.eq.'ichipsh') then
	    if (str1.eq.' ') then
	      write(6,422) ichipsh
 332	      write(6,400)
	      call read1i(ichipsh,*999,*332)
	    else
	      ichipsh=nfr
	    end if
	    write(6,422) ichipsh
	  else if (var.eq.'adusatpipe') then
	    if (str1.eq.' ') then
	      write(6,423) adusatpipe
 333	      write(6,400)
	      call read1r(adusatpipe,*999,*333)
	    else
	      adusatpipe=rn
	    end if
	    write(6,423) adusatpipe
	  else if (var.eq.'iccdgap') then
	    if (str1.eq.' ') then
	      write(6,424) icccdgap
 334	      write(6,400)
	      call read1i(iccdgap,*999,*334)
	    else
	      iccdgap=nfr
	    end if
	    write(6,4241) iccdgap
	  else if (var.eq.'jccdgap') then
	    if (str1.eq.' ') then
	      write(6,4241) jcccdgap
 3341	      write(6,400)
	      call read1i(jccdgap,*999,*3341)
	    else
	      jccdgap=nfr
	    end if
	    write(6,4241) jccdgap
	  else if (var.eq.'gappercent') then
	    if (str1.eq.' ') then
	      write(6,425) gappercent
 335	      write(6,400)
	      call read1r(gappercent,*999,*335)
	    else
	      gappercent=rn
	    end if
	    write(6,425) gappercent
	  else if (var.eq.'imaskout') then
	    if (str1.eq.' ') then
	      write(6,426) imaskout
 336	      write(6,400)
	      call read1i(imaskout,*999,*336)
	    else
	      imaskout=nfr
	    end if
	    write(6,426) imaskout
	  else if (var.eq.'igetstar') then
	    if (str1.eq.' ') then
	      write(6,427) igetstar
 337	      write(6,400)
	      call read1i(igetstar,*999,*337)
	    else
	      igetstar=nfr
	    end if
	    write(6,427) igetstar
	  else
	    write(6,*) '  *** no such rcspipe set parameter '
	  end if
	end if
 400	format(' Enter new value: ',$)
 401	format(' ircspipe:',i2,' - photometry pipeline switch ('
     *         ,'turned on when called')
 402	format(' irplim:',i5,' - lower limit for replacement for rple')
 403	format(' irprep:',i6,' - replacement value for rple; set to 0 ',
     *      'for auto (adusat)')
 404	format(' iinitpipe:',i3,' - initial i shift for auto matching')
 405	format(' jinitpipe:',i3,' - initial j shift for auto matching')
 4051	format(' iwcssh:',i2,' - switch for auto wcs rough shift (1=on)')
 406	format(' imaskpipe:',i2,' - mask frame number (likely hardwired)')
 407	format(' npadpipe: ',i2,' - padding layer for mask for padd')
 408	format(' ismtypepipe: ',i2,' - tsmo smoothing algorithm: 1=tapered, 
     *               0=box')
 4081	format(' ismthlenp: ',i2,' - tsmo smoothing length, 3 or 5')
 409	format(' iaprefpipe:',i3,' - max aperture for autoref phot')
 410	format(' iapmaxpipe:',i3,' - max aperture diameter for photometry')
 411	format(' ciadmpipe:',f5.1,' - diameter for centroiding routine')
 412	format(' ishmaxpipe:',i2,' - max pixel shift allowed for centroiding')
 413	format(' nsmthlenp:',i2,' - smoothing box size for obj find check')
 414	format(' isbpipe:',i2,' - box x size for median filtering for crm')
 415	format(' jsbpipe:',i2,' - box y size for median filtering for crm')
 416	format(' crsogpipe:',f6.2,' - sigma crit for crm')
 417	format(' ncrexcpipe:',i2,' - mask frame number for exclusion in crm')
 418	format(' isatpixpipe:',i6,' - saturated pix value to be replaced')
 419	format(' adulowfil(n):',f7.4,' - low adu criterion for filter')
 420	format(' adulowfil(1-5):',5f7.4,/
     *               '        low adu criterion for filters 1 to 5')
 421	format(' colapert:',f6.2,' - color aperture in " for mspos ')
 422	format(' ichipsh:',i2,' - algorithm for finding chip shifts'/
     *   ' 1: shifts computed/fixed first; 2: use WCS on pos file;/',
     *   '    3: shifts computed as needed')
 423	format(' adusatpipe:',f7.0,' - overide header saturation value',
     *         ' if set to negative')
 424	format(' iccdgap:',i3,' - ccd gap in pixels, in x/I direction')
 4241	format(' jccdgap:',i3,' - ccd gap in pixels, in y/J direction')
 425	format(' gappercent:',f6.2,' - limit of acceptable % of area ',
     *         ' lost between filters')
 426	format(' imaskout:',i2,' - output of chip mask: 0=ascii'
     *        ,',1=single fits;2=MEF,-1=none')
 427	format(' igetstar:',i2,' - star catalog choice: 1=Tycho;'
     *        ,' 2=USNO')
 999	return1
	end
c/////////////////////////////////////////////////////////////////////
c  subroutine to save fits header info onto filinfo common block for mkhdr
c   the image is always image #1
c    jfil is the filter number
c
	subroutine smspars(jfil,phmnamein)
	character phmnamein*60
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'picdir.inc'
c 	write(6,*) ' filter ',jfil,nfilref
	phmname(jfil)=phmnamein(1:lnblnk(phmnamein))
	fzpt(jfil)=zeropt(1)
	fcolterm(jfil)=clrterm(1)
	fgain(jfil)=ccdgn(1)
	fexptime(jfil)=exptime(1)
	fairmterm(jfil)=airmterm(1)
	fscale(jfil)=pixscale(1)
	frlat(jfil)=rlat(1)
	frlong(jfil)=rlong(1)
	ffiltn(jfil)=filter(1)
c  get obs date to the right format
        fobsdate(jfil)=obsdate(1)(9:10)//'/'//obsdate(1)(6:7)//
     *          '/'//obsdate(1)(3:4)
c  get obs time to the right format
c    determine if the hr is one or two digit
	do 120 i=1,20
	  if (obstime(1)(i:i).eq.':') goto 121
 120	continue
 121	continue
	if (i.eq.2) then 
	  fobstime(jfil)=' '//obstime(1)(1:4)
	else
	  fobstime(jfil)=obstime(1)(1:5)
	end if
c get observatory name based on longitude, reverse engineer the name
c	write(6,*) '  observatory longitude ',frlong(jfil)
	if (frlong(jfil).lt.-154..and.frlong(jfil).gt.-157.) then
	  fobser(jfil)='cfht'
	else if (frlong(jfil).gt.110..and.frlong(jfil).lt.113.) then
	  fobser(jfil)='kpno'
	else if (frlong(jfil).gt.70..and.frlong(jfil).lt.70.75) then
	  fobser(jfil)='lco'
	else if (frlong(jfil).gt.70.75.and.frlong(jfil).lt.71.00) then
	  fobser(jfil)='ctio'
	end if
c	if (fobser(jfil).eq.' ') fobser(jfil)='cfht'
c	write(6,*) ' observatory assigned: ',fobser(jfil)
	if (jfil.eq.nfilref) then
c  for filter 1, save the general pointing info
c   get RA and Dec string properly
	  frastr=rastr(1)
c replace : with blanks
	  do 110 i=1,lnblnk(frastr)
	    if (frastr(i:i).eq.':') frastr(i:i)=' '
 110	  continue
c check if hour is single digit, add 0 if it is
	  ntemp=index(frastr,' ')
	  if (ntemp.eq.2) frastr='0'//frastr
	  fdecstr=decstr(1)
	  do 111 i=1,lnblnk(fdecstr)
	    if (fdecstr(i:i).eq.':') fdecstr(i:i)=' '
 111	  continue
	end if
c copy wcs solution
	fcrpix1(jfil)=crpix1(1)
	fcrpix2(jfil)=crpix2(1)
	fcrval1(jfil)=crval1(1)
	fcrval2(jfil)=crval2(1)
	fcd1_1(jfil)=cd1_1(1)
	fcd1_2(jfil)=cd1_2(1)
	fcd2_1(jfil)=cd2_1(1)
	fcd2_2(jfil)=cd2_2(1)
	ifwcs=iwcs
	fra11(jfil)=ra11(1)
	fdec11(jfil)=dec11(1)
	return
	end
c////////////////////////////////////////////////////////////
c driving routine to make .hdr file, mspos2 batch jobs, 
c  ipar=0: make .hdr file 
c  ipar=1: also make msposin file 
c  ipar=2: run the msposin file
c    ix is dummy,  only for calling rdkfits
c
	subroutine msposdr(*,nch1,nch2,ipar,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,dumstr*8,imdir*55
c char variable declar''s are temp,  need to get rootnmae,cext
	character errmsg*30,rootname*60,cext*2,filent*60,primname*60
	integer status,ib(6),ie(6),rmode
	integer ix(nu,nv,kf)
	common/imagedir/imdir
	common/hdrfoder/nfilh,jforder(6),jrefcol(6)
	include 'pipeparm.inc'
	nfilref=1
 600	write(6,1) 
 1	format('  ++ warning: frame 1 will be overwritten'/
     *    '  Enter fits file names (.fits.*/fz implicit), max 6:'
     *          /': ',$)
	read(5,'(a60)',end=999) filestr
c parse file string
	call fstr(filestr,60,ib,ie,nfilt)
c determine the number of chips	using the first image (all others
c    must have the same number of chips!)
	filen=filestr(ib(1):ie(1))
	call imageex(filen,filenc,iimex)
	if (iimex.lt.0) then
	  ircspipe=0
	  return1
	end if
c  attempt to open file  (cfitsio rwmode: 0=read, 1=write)
	rmode=0
c	wmode=1
	status=0
	call ftgiou(lun,status)    
	call ftnopn(lun,imdir(1:lnblnk(imdir))//
     *      filenc(1:lnblnk(filenc))//'[0]',rmode, status)
	call ftgkyj(lun,'NEXTEND',nextendt,dumstr,status)
c  if not found, set nextend to 1
	if (status.ne.0) nextendt=1
	write(6,*) ' # of extension: ',nextendt,status
	call ftgerr(status,errmsg)
	call ftclos(lun, status)
	call ftfiou(lun, status)
	if (nch1.eq.0) then
	  jc1=1
	  jc2=nextendt
	else if (nch2.eq.0) then
	  jc1=nch1
	  jc2=nch1
	else
	  jc1=nch1
	  jc2=nch2
	end if
	rootname=filen(1:index(filen,'_')-1)
	do 100 jj=jc1,jc2
c extension string
	istatus=encode(cext,float(jj),1,0,idum)
c read header for each filter
	  do 101 i=1,nfilt
c make fits file name for chip jj, filter i
	    filen=filestr(ib(i):ie(i))
	    primname=filen(1:index(filen,'_')-1)
	    filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	    write(6,*) '  ^^^^ reading image: ',filent
	    call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,-1)
c save header block
 800	    call headtst(1)
c get values of key words from header
	    call getkeyval(*309,1,kf)
 309	    call mkoutname(primname,1,cext,'ph')
c *** need to figure out phmname
 	    call smspars(i,poutname)
 101	  continue
c make header 
	  call mkhdr(rootname,cext)
 100	continue
c 
 999	return1
	end 
c///////////////////////////////////////////////////////////////
c 
c  subroutine to make .hdr files
c    almost all information comes from .inc blocks
c
c    assignment of parameters  are made in smspar to the
c              common block filinfo for use in making header.
	subroutine mkhdr(rootname,cext)
	character filestr*60,rootname*60,hdrname*24,decsign*1,
     *   filterstr*20,cext*2,ncostr*1,vstring*16,fstr*60,
     *    nfitstr*1,nskipstr*2,nfilstr*1,fieldname*60,ds*1
 	character phmstr*60
	common/hdrfoder/nfilh,jforder(6),jrefcol(6)
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'objcat.inc'
	include 'picdir.inc'
c temporarily fix the filter order (z,r,g), assuming input as r,z,g, or r z i g
c  set to default if there is no info for constructing hdr
	if (jforder(1).eq.0.and.nfilh.ge.4) then
 	  jforder(4)=4
	  jrefcol(4)=2
	end if
	if (jforder(1).eq.0.and.nfilh.ge.3) then
 	  jforder(3)=3
	  jrefcol(3)=2
	end if
	if (jforder(1).eq.0.and.nfilh.ge.2) then
 	  jforder(1)=2
 	  jforder(2)=1
	  jrefcol(1)=2
	  jrefcol(2)=1
	end if
	fieldname=rootname(1:min0(18,lnblnk(rootname)))//'_'//
     *        cext(1:lnblnk(cext))
	hdrname=fieldname(1:lnblnk(fieldname))//'.hdr'
	write(6,*) '  ++ making .hdr file: ',hdrname
	call opennew(29,1,24,hdrname,*999)
c write name
	write(29,1) fieldname
 1	format(1X,A16,6X,'CONTROL',16X,'// object name, z')
c   parse  RA string
c	write(6,*) frastr, fdecstr 
	itemp=0
	if (itemp.eq.0) goto 888
	n1=index(frastr,':')
	read(frastr(1:n1-1),*) irh
	n2=index(frastr(n1+1:lnblnk(frastr)),':')
	read(frastr(n1+1:n2-1),*) irm
	read(frastr(n2+1:lnblnk(frastr)),*) rs
c parse dec string
	ds=fdecstr(1:1)
	if (ds.ne.'-') ds='+'
	n1=index(fdecstr,':')
	read(fdecstr(1:n1-1),*) idd
	idd=iabs(idd)
	n2=index(fdecstr(n1+1:lnblnk(fdecstr)),':')
	read(fdecstr(n1+1:n2-1),*) idm
	read(fdecstr(n2+1:lnblnk(fdecstr)),*) ds
c  compute BII (temp)
	bii=90.
c write RA and dec
	write(29,2) irh,irm,rs,sd,idd,idm,ds,bii
 2	format(1X,I2,1X,I2,1X,F5.2,3X,A1,I2,1X,I2,1X,F5.2,10X,F5.1,4X,
     *          '// RA and DEC (2000), BII')
 888	continue
	aphat=fcrval1(1)
	call galactic1(fcrval1(1),fcrval2(1),reall,bii,0)
 	write(29,22) frastr(1:12),fdecstr(1:12),bii
 22	format(1X,a12,2X,a12,9x,f5.1,5x,
     *          '// RA and DEC (2000), BII')
c write filter number and names
        istat=encode(ncostr,float(nfilt),1,0,idum)
        istat=encode(vstring,float(42-nfilt*2),1,0,idum)
c	write(6,*) ncostr,vstring
        fstr='(1X,I1,2X,'//ncostr//'(A1,1X),'//
     *       vstring(1:lnblnk(vstring))//
     *       'X,''// # of colors and name(s)'')'
        WRITE(29,fstr) nfilt, (ffiltn(jforder(k)),K=1,nfilt)
c write .phm names for nfilt filters  (should really be nfilh, but
c            it is okay for now, the two are always equal
	do 100 kk=1,nfilt
c take care of filter order
	  k=jforder(kk)
c	write(6,*) ' writing phm name ',k,phmname(k)
	  write(29,3) phmname(k),ffiltn(k)
 3	format(1X,A24,21x,'// data file name for ',A1)
 100	continue
c for each filter, write obs, date and integration time (2 lines)
	do 101 kk=1,nfilt
c	write(6,*) ' writing obs name ',k,fobser(k)
	  k=jforder(kk)
	  write(29,4) fobser(k),frlong(k),frlat(k), ffiltn(k)
 4	format(1X,A4,3X,F8.3,2X,F7.3,21X,'// Obs,longitude,'
     *          ,'latitude: ',a1)
c need to interpret obsdate, obstime ****
	  write(29,5) fobsdate(k)(1:8),fobstime(k)(1:5),
     *           fexptime(k),ffiltn(k)
 5	format(1X,A8,1x,a5,1x,f8.2,22x,
     *    '// UT date, time, & int time: ',a1)
 101	continue  
c default color aperture in arcsec
	write(29,6) colapert
 6	format(1X,f5.2,40X,'// default colour aperture in "')
c reference colors for nfilt (temp for 2 filters)
	istatus=encode(nfilstr,float(nfilt),1,0,idum)
c	nfilstr='2'
	ntem=46-nfilt*2
	istatus=encode(nskipstr,float(ntem),1,0,idum)
c	nskipstr='42'
	fstr='('//nfilstr//'i2,'//nskipstr//
     *    'x,''//ref color for'','//nfilstr//'(1x,a1))'
c  determine the right order of reference filter
	write(29,fstr) (jrefcol(j),j=1,nfilt),
     *           (ffiltn(jforder(k)),k=1,nfilt)
c  zero pt, airmass and color term
	do 102 kk=1,nfilt
	  k=jforder(kk)
c currently set all the color terms to zero
	  fcolterm(k)=0.0
	  write(29,7) fzpt(k),fairmterm(k),fcolterm(k),ffiltn(k)
 7	format(3f7.3,25x,'// 0 pt, airmass & color term: ',a1)
 102	continue
c  if nfilt>2, write complementary terms
	if (nfilt.gt.2) then
	  do 103 kk=1,nfilt
	    k=jforder(kk)
	    comzeropt=fzpt(jforder(jrefcol(kk)))
c 	    compcolor=-0.5*fcolterm(k)
 1032	    compcolor=0.
	    write(29,9) comzeropt,compcolor,ffiltn(k)
 9	format(2f7.3,32x,'// compl 0 pt & color term: ',a1)
 103	  continue
	else if (nfilt.eq.1) then
c single filter, enter average color
	end if
c  scale,  gain and size
	do 104 kk=1,nfilt
	  k=jforder(kk)
	  write(29,8) fscale(k),fgain(k),ffiltn(k)
  8     format(1x,f7.5,1x,f5.2,32x,'//  "/pix and e-/ADU for ',a1)
 104    continue
c orientation now always fixed at 1, rot angle=0.
c  final version should try to read a .pos file to see the limits
c   otherwise use nii and njj
	itempp=0
        write(29,14) itempp,itemp,itempp,itemp
 14     FORMAT('     0.00',1x,' 1',1x,4i5,13X,'// rot:deg, (1,1):'
     *        ,'0=NW,1=ES, pix limits')
c center object is taken to be 1/2 of total object
c   this requires reading the .pos file if exists
c	ncenter=nint(0.5*nob(1))
	ncenter=-10
c  setting ncenter to <-1 will make mspos find the nearest object
c   to the center of the chip of filter 1 to be the center
        write(29,10) ncenter
 10     format(i6,40x,'// reference object number')
	close(unit=29)
 999	return
	end
c////////////////////////////////////////////////////////////////
c  subroutine to perturb the single value saturation pixels by +/- 10
c    to make allow object finding to deal with too many pixeles
c    having the same value
	subroutine fixsatpix(*,nfr,isatpix,ix,nu,nv,kf)
	integer ix(nu,nv,kf)
	include 'picdir.inc'
	call querynfr(nfr,kf,0,*999,*2000)
 2000	if (isatpix.le.0) then
 600	  write(6,1) isatpix
 1	format('  Enter top pixel value that needs to be replaced [',
     *         i6,']: ',$)
	  call read1i(isatpix,*999,*600)
	end if
	iseed=863 
	ifixed=0
	do 100 j=1,njj(nfr)
	  do 101 i=1,nii(nfr)
	    if (ix(i,j,nfr).ge.isatpix) then
c compute a random number with amplitude 10 adu
	      randn=gasdev(iseed)
c this is to prevent a really big change!
	      randn=amin1(1.,randn)
	      randn=amax1(-1.,randn)
	      ix(i,j,nfr)=ix(i,j,nfr)+NINT(10.*randn)
	      ifixed=ifixed+1
	    end if
 101	  continue
 100	continue
	write(6,*) '  ... number of pixels perturbed by +/-10: ',ifixed
	nfl(6,nfr)=1
 999	return1
	end 
c/////////////////////////////////////////////////////////////
c  subroutine to check if an image actually contain non-zero pixles
c    return by *, *, or ipar=0 for okay, ipar=1 bad image
c
	subroutine chzeros(*,*,nfr,ix,nu,nv,kf,ipar)
	integer ix(nu,nv,kf)
	include 'picdir.inc'
c   spot check mask to make sure it isn''t all zeros
 	nzero=0
	ipar=0
	DO 100 j=200,njj(nfr)-200,50
	  DO 101 i=200,nii(nfr)-200,50
c	    if (ix(i,j,nfr).gt.0) goto 110
c changed Sep 15, 2008, to counts all 1''s as bad pixels also
	    if (ix(i,j,nfr).gt.1) goto 110
	    nzero=nzero+1
 101	  continue
 100	continue
c all checked pixels are zeros, not an image
	ipar=1
	write(6,1) nzero
 1	format(' **** zero check: all pixels checked (',i6,
     *         ') are zero.'/
     *  '   Assume this is not a proper image, process aborted.')
	return2
 110	return1
	end
c////////////////////////////////////////////
c   subroutine to return the shift in pixel of 2 images , using wcs
c
c  ipar=1 use ppp buffer image headers  (not implemented yet)
c  ipar=2 use pipeline filter image info headers
c
c  nb is the current image (in ppp buffer), 
c  nf is the reference image:center RA and Dec already computed,
c     the data for this is common block fwcpar
c
c  - compute the pixel co-ordinate (in nf1) of (1,1) RA and Dec of nf2
c  - compute shifts needed in pixels to put center of nf1 onto the pixels
c         and store the result
c
c  NOTE this assumes the same pixel scale for nb and nf
	subroutine wcsshift(nb,nf,ishift,jshift,ipar)
	real*8 ratemp,dectemp
	include 'imagepar.inc'
	include 'pipeparm.inc'
	if (ipar.eq.1) then
	  write(6,*) '  ** using buffer frames not implemented yet *'
	  return
	else
c comptute the pixel co-ord in nb of the center RA and Dec of nf
c   the wcs solution is that of nf in the ppp buffer
	  ratemp=fra11(nf)*15.
	  dectemp=fdec11(nf)
	write(6,*) ' (1,1) RA, dec of ref',fra11(nf),fdec11(nf)
c  (1,1) in nf is equal to (xc,yc) in nb
	  call xypixpos(nb,xc,yc,ratemp,dectemp)
	write(6,*) ' xy of RA Dec in new frame ',xc,yc
	  ishift=nint(xc)
	  jshift=nint(yc)
	end if
	return
	end 
c///////////////////////////////////////////////////////////////
c  subroutine to convert the xy position of posfile to a filter
c    via the wcs transformation
c          
	subroutine pfshiftwcs(nfr,ifilref)
	real*8 ratemp,dectemp
	real*8 crpix1,crpix2,crval1,crval2
        common/wcspar/crpix1(15),crpix2(15),crval1(15),crval2(15),
     *       cd1_1(15),cd1_2(15),cd2_1(15),cd2_2(15),iwcs
     *       ,ra11(15),dec11(15)
	include 'objcat.inc'
	include 'pipeparm.inc'
	crpix1(15)=fcrpix1(ifilref)
	crpix2(15)=fcrpix2(ifilref)
	crval1(15)=fcrval1(ifilref)
	crval2(15)=fcrval2(ifilref)
	cd1_1(15)=fcd1_1(ifilref)
	cd1_2(15)=fcd1_2(ifilref)
	cd2_1(15)=fcd2_1(ifilref)
	cd2_2(15)=fcd2_2(ifilref)
	write(6,1) ifilref,crpix1(15),crpix2(15),cd1_1(15),cd1_2(15)
 1	format('  reference WCS, crpix1/2;ccd1_1/2: ',i1,2f10.3,2g10.2)
	write(6,2) crpix1(nfr),crpix2(nfr),cd1_1(nfr),cd1_2(nfr)
 2	format('  filter WCS, crpix1/2;ccd1_1/2 :',2f10.3,2g10.2)
	do 100 i=1,nob(nfr)
c  convert pos file positions in reference frame to RA and Dec
c   put the reference image wcs in position 15 for this purpose
	  call astrom(15,float(ijoi(i,nfr)),float(ijoj(i,nfr)),
     *                 ratemp,dectemp)
	  ratemp=ratemp*15.000000000000
c  convert RA and Dec to xy using the wcs of the filter
	  call xypixpos(nfr,xc,yc,ratemp,dectemp)
	  ijoi(i,nfr)=nint(xc)
	  ijoj(i,nfr)=nint(yc)
 100	continue
	return
	end
c//////////////////////////////////////////////////////////////
c  subroutine to redo chips with bad WCS shifts by using average
c    shift of the filter
c
c   *** need to deal with making headers
c
	subroutine photbadchips(jc1,jc2,
     *         filestr,maskstr,nfilm,ie,ib,iem,ibm,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,hdrname*60,
     *       maskstr*60,cext*2 
c	integer ib(6),ie(6),ibm(6),iem(6)
	integer ix(nu,nv,kf)
	dimension avxshif(10),avyshif(10),naver(10)
	include 'pppcore.inc'
	include 'ppparm.inc'
	include 'pipeparm.inc'
c  first check which are the bad chips and compute averages
	write(6,*) ' RRRR: rephotometry; checking chips',jc1,jc2
 	call avshifts(avxshif,avyshif,jc1,jc2,naver,irerun)
c  print out average for debugging
	do 100 i=nfilt
	  write(6,700) i,avxshif(i),avyshif(i),naver(i)
 700	format(' filter ',i2,' av x/y shifts ',2f7.2,' N =',i3)
	  if (naver(i).eq.0) then
	     write(6,*) ' **** no data for computing average shifts'
     *            ,', aborting re-photometry'
	     return
	  end if
 100	continue
c  
	if (irerun.eq.0) then
	  write(6,*)  '   --  no bad WCS shift; photometry done!'
	  return
	end if
c now redo bad chips
	do 110 i=1,nfilt
	  if (i.eq.nfilref) goto 110
	  do 111 j=jc1,jc2
	    if (imatch(i,j).le.-1) then
	      jchip=j
c redo the bad chip
	      istatus=encode(cext,float(j),1,0,idum)
	      call fixphotpipe(avxshift,avyshift,i,filestr,maskstr,
     *               nfilm,ix,nu,nv,kf,ie,ib,iem,ibm,cext)
	    end if
 111	  continue
 110	continue
	return
	end
c////////////////////////////////////////////////////////////////
c subroutine to average x y shifts 
c   the assumption is that the chips are read differently 
c     such that the first half of the chips are different from
c     the 2nd half (i.e., where [1,1] is).  Note that this seems 
c    to work for MegaCam, but CFH12k is not so simply dividied`
c 
 	subroutine avshifts2(avxshif1,avyshif1,
     *      avxshif2,avyshif2,jc1,jc2,naver1,naver2,nextendt)
c the running index in the array is for filter number
	dimension avxshif1(*),avyshif1(*),naver1(*),
     *            avxshif2(*),avyshif2(*),naver2(*)
	include 'pipeparm.inc'
	include 'imagepar.inc'
c  nsplit is where the chips are divided on their orientation
	nsplit=nint(nextendt*0.5)
	do 100 i=1,nfilt
c  don''t bother with the reference filter
	  if (i.eq.nfilref) goto 100
	  xshiftaver1=0.
	  yshiftaver1=0.
	  xshiftaver2=0.
	  yshiftaver2=0.
	  nav1=0
	  nav2=0
	  do 101 j=jc1,jc2
	    if (imatch(i,j).ge.1) then
	      if (j.le.nsplit) then
	        xshiftaver1=xshiftaver1+pmxshift(i,j)
	        yshiftaver1=yshiftaver1+pmyshift(i,j)
	        nav1=nav1+1
	      else
	        xshiftaver2=xshiftaver2+pmxshift(i,j)
	        yshiftaver2=yshiftaver2+pmyshift(i,j)
	        nav2=nav2+1
	      end if
c	    else if (imatch(i,j).le.-1) then
c  re running (for any one of the chips) required
c	      irerun=1
	    end if
 101	  continue
	  if (nav1.eq.0) then
	    avxshif1(i)=999.9
	    avyshif1(i)=999.9
	    write(6,*)' *** no data for computing aver shifts',i
	  else
	    avxshif1(i)=xshiftaver1/float(nav1)
	    avyshif1(i)=yshiftaver1/float(nav1)
	  end if
	  naver1(i)=nav1
	  if (nav2.eq.0) then
	    avxshif2(i)=999.9
	    avyshif2(i)=999.9
	    write(6,*)' *** no data for computing aver shifts',i
	  else
	    avxshif2(i)=xshiftaver2/float(nav2)
	    avyshif2(i)=yshiftaver2/float(nav2)
	  end if
	  naver2(i)=nav2
 100	continue
	return
	end
c////////////////////////////////////////////////////////////////
c subroutine to average x y shifts
c
        subroutine avshifts(avxshif,avyshif,jc1,jc2,naver,irerun)
        dimension avxshif(*),avyshif(*),naver(*)
        include 'pipeparm.inc'
        irerun=0
        do 100 i=1,nfilt
c  don''t bother with the reference filter
          if (i.eq.nfilref) goto 100
          xshiftaver=0.
          yshiftaver=0.
          naver(i)=0
          do 101 j=jc1,jc2
            if (imatch(i,j).ge.1) then
              xshiftaver=xshiftaver+xchshift(i,j)
              yshiftaver=yshiftaver+ychshift(i,j)
              naver(i)=naver(i)+1
            else if (imatch(i,j).le.-1) then
c  re running required
              irerun=1
            end if
 101      continue
          if (naver(i).eq.0) then
             avxshif(i)=999.9
             avyshif(i)=999.9
          if (naver(i).eq.0)
     *    write(6,*) ' **** no data for computing aver shifts',i
          else
            avxshif(i)=xshiftaver/float(naver(i))
            avyshif(i)=yshiftaver/float(naver(i))
          end if
 100    continue
        return
        end
c////////////////////////////////////////////////
c  subroutine to do bad chip j, filter i
c
	subroutine fixphotpipe(avxshift,avyshift,ifilter,
     *       filestr,maskstr,nfilm,ix,nu,nv,kf,ie,ib,iem,ibm,cext)
	integer ix(nu,nv,kf)
	integer nfra(5),ib(6),ie(6),ibm(6),iem(6)
	dimension avxshift(*),avyshift(*)
	character filestr*60,maskstr*60,maskname*60,dumstr*60,
     *        filen*24,filent*60,cext*2,temname*60,primname*60,
     *        posfname*60,satposname*60
	include 'pppcore.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'picdir.inc'
	include 'ppparm.inc'
        common/imagestats/imstatspar(50),ich_id,ch_rms(50),ch_sky(50),
     *   nbstar_ch(50),ch_bsmag(15,50),ch_bsx(15,50),ch_bsy(15,50)
	dumstr='dummy'
	idum=0
	imaskpipe=2
	iastromtemp=iastrom
	nfra(1)=1
	do 777 jj=2,5
	  nfra(jj)=0
 777	continue
	write(6,*) ' #### repeating photometry for chip ',
     *    cext,' filter ',ifilter
	read(cext,*,err=901) nchip
	ich_id=nchip
c  first refind the reference position stars from reference filter
c  (copy filter number to "i" to make copying from pipephot easier...)
	i=nfilref
	filen=filestr(ib(i):ie(i))
	primname=filen(1:index(filen,'_')-1)
	filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	call mkoutname(primname,1,cext,'of')
	posfname=poutname(1:lnblnk(poutname))
c read image
	write(6,*) '    +++ first: find pos ref from filter ',i
	write(6,*) '  #### reading image: ',filent
	call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,0)
 800	call chzeros(*8001,*999,1,ix,nu,nv,kf,ichzero)
c read mask
 8001	call getadusat(adusat,saturate(1),adusatpipe)
	if (nfilm.ge.1) then
	  call getmasknam(i,ibm,iem,maskstr,maskname)
	  write(6,*) '  ^^^^ reading mask: ',maskname
	  call rdkfits(*801,2,maskname,irep,ix,nu,nv,kf,ir,0)
	else if (nfilm.eq.0) then
	  dumstr='dummymask'
	  call rdkfits(*801,2,dumstr,irep,ix,nu,nv,kf,ir,0)
	else if (nfilm.eq.-1) then
c create mask from image
	  call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	  write(6,*) ' #### created mask from image'
	end if
c  fill zeros
 801	irplim=-10
	irprep=nint(saturate(1))
	icomp=0
	call searchpix(*802,1,icomp,' ',' ',1,ix,nu,nv,kf)
c create the ref position file (assume the ref image is always the 1st one
c   this uses the reference image and save reference objs positions
c    in common/refobj/. (No shift is done here)
 802	    write(6,*) ' ^^^ qfnd for primary filter'
	    call FINOBD(*803,1,dumstr,dumstr,0.00,7,IX,NU,NV,KF)
 803	    write(6,*) ' #### arf for position reference '
	    call autoref(1,ix,nu,nv,kf,0,*804)
c  now do the bad filter
c  (copy filter number to "i" to make copying from pipephot easier...)
 804	i=ifilter
c do for each filter
	write(6,*) '  ### now photometry for the bad filter +++ ',
     *            ' chip ',cext,' filter ',i
	filen=filestr(ib(i):ie(i))
	primname=filen(1:index(filen,'_')-1)
	filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
c read image
	write(6,*) '  ### reading image: ',filent
	call rdkfits(*890,1,filent,irep,ix,nu,nv,kf,ir,0)
 890	call chzeros(*8901,*999,1,ix,nu,nv,kf,ichzero)
c read mask
 8901	call getadusat(adusat,saturate(1),adusatpipe)
	if (nfilm.ge.1) then
	  call getmasknam(i,ibm,iem,maskstr,maskname)
	  write(6,*) '  ###  reading mask: ',maskname
	  call rdkfits(*891,2,maskname,irep,ix,nu,nv,kf,ir,0)
	else if (nfilm.eq.0) then
	  dumstr='dummymask'
	  call rdkfits(*891,2,dumstr,irep,ix,nu,nv,kf,ir,0)
	else if (nfilm.eq.-1) then
c create mask from image
	  call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	  write(6,*) ' ###  created mask from image'
	end if
c  fill zeros
 891	irplim=-10
	irprep=nint(saturate(1))
	icomp=0
	call searchpix(*892,1,icomp,' ',' ',1,ix,nu,nv,kf)
c  create shift :
 892  write(6,*) ' ### pos matching for filter',i
c  use initial shifts from average
	    iinitpipe=nint(avxshift(i))
	    jinitpipe=nint(avyshift(i))
	    write(6,14) iinitpipe,jinitpipe
 14	format('  ### shifts estimated from average: ',2i5)
c  determine shift between filter i and primary
	    call simposmat(1,iir,iret,1,1,ix,nu,nv,kf,*8041) 
 8041	    imatch(i,nchip)=iret
	    if (imatch(i,nchip).lt.0) then
	      write(6,8042) nchip,i
 8042	format('   ***** STILL matching problem for Chip ',i2,
     *         ' filter ',i2,' give up!!')
	      goto 100
	    end if
c  make bleeding column mask, first read in saturation pos file
c     first make name, and save for other filters
 	  write(6,*) ' ### reading saturated pos file ',satposname
 	  call rpof(*805,1,satposname,idum,0)
c  if not the primary file, shift the pos file
 805	  if (i.ne.nfilref) call pfshift(*806,1,0,0,dumstr,1)
 806	  write(6,*) '  ###  making bleeding mask, added to bp mask'
c first remove diffraction spike parameters
	  dsct1=dsc(1)
	  dsct2=dsc(2)
	  dsct3=dsc(3)
	  dsc(1)=-999
	  dsc(2)=0.
	  dsc(3)=0.
 	  call bleedmap(*807,1,2,ix,nu,nv,kf,0,0,0)
c fix bad column, both bad columns and bleed columns
 807	  call bpfix(*808,1,2,0,ix,nu,nv,kf)
 808	  dsc(1)=dsct1
	  dsc(2)=dsct2
	  dsc(3)=dsct3
c  read position file
	  write(6,*) ' ### reading actual pos file ',posfname
 	  call rpof(*809,1,posfname,idum,0)
c     do shift if needed  (dumstr here is just a dummy, can be anything)
 809      if (i.ne.nfilref) call pfshift(*810,1,0,0,dumstr,1)
c create cosmic ray map and fix
 810	write(6,*) '  ###  creating cosmic ray map and fix '
	  call cosraymap(*8101,1,2,' ',' ',nscrit,0,0,rwd,ix,nu,nv,kf)
c pad the CR map
 8101	  call padmap(*8102,2,ix,nu,nv,kf,0)
 8102	  call bpfix(*8103,1,2,0,ix,nu,nv,kf)
c create classification reference star file
c   first make name
 8103	  call mkoutname(primname,1,cext,'rf')
c  save the name for reference
	  arefname=poutname
	  write(6,*) '  ###  aref: finding psfs'
	  adulowpipe=adulowfil(i)
 	  call autoref(1,ix,nu,nv,kf,1,*811)
c do photometry
c  create .phm file output name and comment
 811	  call mkoutname(primname,1,cext,'ph')
	  call mkoutcomm(primname,1,cext,'ph')
 	  temname=poutname(1:lnblnk(poutname))
	  write(6,*) '   ###  running ppphot ',temname
	  if (i.eq.nfilref) then
c get astrometry
	    iastrom=iastromtemp
	  else
	    iastrom=-1
	  end if
	  call pppdri(*812,nfra,temname,poutcomm,0,ix,nu,nv,kf)
c save mspos hdr parameters
 812	  continue
c812	  call smspars(i,temname)
 100	continue
	iastrom=iastromtemp
 999	return
 901	write(6,*) ' ***chip number text read error ',cext,
     *             '  *** skipping to next chip'
	return
	end
c///////////////////////////////////////////////////////////////
c  quick and dirty bleeding map for very bright stars picked
c    up from USNO catalog
c
c  this simply makes a bleeding column through the whole vertical
c    direction of the chip of 21 pixels in width
c
c   ipar=0  start a new frame
c   ipar=1  put on existing frame
	subroutine qkbsmask(ni,nj,nfr,cext,ix,nu,nv,kf,ipar)
	integer ix(nu,nv,kf)
	character cext*2,tempoutname*24
	include 'picdir.inc'
	include 'objcat.inc'
	common/bstarxy/nbstar,bstx(20),bsty(20),bsrad(20),bsmag(20)
	nii(nfr)=ni
	njj(nfr)=nj
	nfl(6,nfr)=1
c  chip number
	read(cext,*) nchip
	if (ipar.eq.0) then
c  fill array with 0''s
	  do 100 j=1,nj
	    do 101 i=1,ni
	      ix(i,j,nfr)=0
 101	    continue
 100	  continue
	end if
c for each bright star, make a column of 1''s 21 pixel wide, but
c  do not fill the +/- 15 pix around the bright star center.
c
c Mar 9, 2007
c  for special chip(s?) on megacam, the bleeding is in the
c   other direction and broader
	do 110 k=1,nbstar
	  if (bsmag(k).le.9.0) then
	    ibs=nint(bstx(k))
	    jbs=nint(bsty(k))
	    write(6,*) '   Bright star mag: ',bsmag(k),' at ',
     *                bstx(k),bsty(k)
	    if (nchip.eq.3.or.nchip.eq.34.or.nchip.eq.12) then
	      if (bsmag(k).gt.7.5) then
	        write(6,*) '  >>> special horizontal bleeding chip '
	        write(6,*) '  +++ making bleeding columns i, +/-',
     *             ' 25pix , between',  ibs-500,ibs+500,' @ ',ibs,jbs
	        write(6,*) '  ( leaving ',ibs-10,' to ',ibs+10,' free'
c leave the edge at least one pixel in i to avod bad interpolation
c   use the pos file limit, as the megacam images have edges without data
c     assume frame 1 is the image with the posfile
	        do 211 j=max(jlpos(1)+1,jbs-25),min(jupos(1)-1,jbs+25)
	           do 212 i=max(ilpos(1)+1,ibs-500),
     *                            min(iupos(1)-1,ibs+500)
	            if (i.lt.ibs-10.or.i.gt.ibs+10) ix(i,j,nfr)=1
 212	          continue
 211	        continue
	      else
c  very bright star, do the mask across the whole chip, with differen widths
	        if (bsmag(k).gt.7.0) then
	          imaskhwid=40
	        else if (bsmag(k).gt.6.5) then
	          imaskhwid=55
	        else if (bsmag(k).gt.6.0) then
	          imaskhwid=75 
	        else if (bsmag(k).gt.5.7) then
	          imaskhwid=150
	        else
	          imaskhwid=250
	        end if
	        write(6,*) '  +++bleeding columns i, +/- ',
     *             imaskhwid,' pix across whole chip; ',ibs,jbs
	        write(6,*) '  ( leaving ',ibs-10,' to ',ibs+10,' free)'
c leave the edge at least one pixel in j to avod bad interpolation
	        do 221 j=max(jlpos(1)+1,jbs-imaskhwid),
     *                 min(jupos(1)-1,jbs+imaskhwid)
	           do 222 i=ilpos(1)+1,iupos(1)-1
	            if (i.lt.ibs-10.or.i.gt.ibs+10) ix(i,j,nfr)=1
 222	          continue
 221	        continue
	      end if
	    else
c regular chips, bleeding is up and down j direction
	      write(6,*) '  +++ making bleeding columns i between',
     *               ibs-10,ibs+10
	      write(6,*) '  ( leaving ',jbs-15,' to ',jbs+15,' free'
	      do 111 j=1,nj
	        if (j.le.jbs-15.or.j.ge.jbs+15) then
	          do 112 i=max(1,ibs-10),min(ni,ibs+10)
	            ix(i,j,nfr)=1
 112	          continue
	        end if
 111	      continue
	    end if
c  out put mask image
c	    tempoutname='masktemp'//cext(1:lnblnk(cext))//'.fits'
c	    call qkimageout(nfr,ni,nj,ix,nu,nv,kf,tempoutname)
	  end if
 110	continue
	return
	end

c/////////////////////////////////////////////
c  quick and dirty output image subroutine for quick check  for debugging
c
c  not quit sure if this is working...
c
	subroutine qkimageout(nfr,ni,nj,ix,nu,nv,kf,fname)
	character  fname*24,objnam*35,headblkt*2880,
     *          ncomc*36,fitsname*24,headbst*2880
	integer ix(nu,nv,kf)
	COMMON/PICC/ NCOMC(15),fitsname(15)
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
 310        write(6,*) ' -- writing quick temporary fitsfile: ',fname
            open(unit=20,file=fname,status='new',
     *          access='direct',recl=2880,err=9001)
c           ncomc(nfr)=' temporary file'
c           bzest(nfr)=0.
c           bscst(nfr)=1.
c           call wdkfits(*999,2,fname,-1,ix,nu,nv,kf,idum)
            objnam=' temporary file'
	    bzt=0.
	    bsct=1.
            call makefhd1(ni,nj,objnam,headblkt,bzt,bsct)
            ihb=1
            write(20,rec=1) headblkt
C absoultely not clear why byte switch has to be turned off when
c   the display is saoimage, but not when it is ds9. (Nov 19 2004)
            ibytet=1
            call fitsdataout(20,ihb,ix(1,1,nfr),nu,nv,
     *                      ni,nj,ibytet,bscale,bzero)
	go to 999
 9001	write(6,*) '  ++ quick image output error: name problem! '
 999	return
	end
c////////////////////////////////////////////////
c  subroutine to fix adusat to be slightly below the saturation
c    value given in a  fits header, 
c
	subroutine getadusat(adusat,saturate,adusatpipe)
c  over ride fits header saturate parameter if adusatpipe is negative 
	if (adusatpipe.lt.0.) then
	  adusat=abs(adusatpipe)
	  return
	end if
	if (saturate.gt.0.001) then
	  if (saturate.lt.33000.) then
	    adusat=saturate-1000
	  else if (saturate.lt.50000.) then
	    adusat=saturate-2000.
 	  else if (saturate.lt.60000.) then
 	    adusat=saturate-2500.
	  else if (saturate.lt.65000.) then
	    adusat=saturate-3000.
	  else  
	    adusat=saturate-4000.
	  end if
	else
c  if no value saturate is given, assume short integer saturation
	  adusat=32000.
	end if
	return
	end
c///////////////////////////////////////////////////////
c subroutine to update wcs solution from the .fits file
c   using .head file which has a more accurrate wcs solution
c note chkwcshead uses format name+filter_chip_qk.head; the primename
c    uses _ to separate filter and name.
c    the routine tries to figure what the name should be
c
	subroutine chkwcshead(nfr,filent,cext)
	logical ex
	character filent*60,cext*2,headname*60,filenthead*60,
     *        filenthead2*60
c check to make sure .fits and .fz are not attached
	l_dot=index(filent,'.')-1
	l_bar=index(filent,'_')-1
	l_hat=index(filent,'^')-1
c strip the .fz or .fits
	if (l_dot.gt.0) then
	  if (filent(l_dot+2:l_dot+2).eq.'f') then
	    filenthead=filent(1:l_dot)
	  else
	    filenthead=filent
	  end if
	else
	  filenthead=filent
	end if
c strip ^ from the name
	if (l_hat.gt.0.) filenthead=filenthead(1:l_hat)
	if (l_bar.gt.0) then
c assume the bar preceeds the filter
	  filenthead2=filenthead(1:l_bar)//filenthead(l_bar+2:l_bar+2)
	  filenthead=filenthead(1:l_bar)
	end if
	headname=filenthead2(1:lnblnk(filenthead2))//'_'//
     *             cext(1:lnblnk(cext))//'_qk.head'
c    first check for the existence of .head file, if not found,
c          check for _quick_head file
c   
 	write(6,*) ' ... searching for ',headname
	inquire(file=headname,exist=ex)
	if (ex.eqv..true.) then
c read wcs from the .head file
c	  write(6,*) ' --- reading quick wcs header ',
c    *         headname
	  call readheadwcs(nfr,headname)
	else
 	  headname=filenthead(1:lnblnk(filenthead))//'_'//
     *      cext(1:lnblnk(cext))//'.head'
 	write(6,*) ' ... searching for ',headname
	  inquire(file=headname,exist=ex)
	  if (ex.eqv..true.) then
	    write(6,*) ' --- reading replacement wcs header ',
     *         headname
	    call readheadwcs(nfr,headname)
	  else
	    write(6,*) ' -- no quick wcs header found -- '
	  end if
	end if
	return
	end
c///////////////////////////////////////////
c subroutine to read wcs from .head file
c
	subroutine readheadwcs(nfr,headname)
	character headname*60,hdr*80
	include 'imagepar.inc'
	open(unit=41,file=headname,status='old',err=999)
 100	read(41,'(a80)',end=998) hdr
        if (hdr(1:6).eq.'CRPIX1') then
          read(hdr(11:31),*,err=990) crpix1(nfr)
          iwcs=1
          write(6,*) ' .head wcs found;  CRPIX1 ',crpix1(nfr),iwcs
        else if (hdr(1:6).eq.'CRPIX2') then
          read(hdr(11:31),*,err=990) crpix2(nfr)
        else if (hdr(1:6).eq.'CRVAL1') then
          read(hdr(11:31),*,err=990) crval1(nfr)
        else if (hdr(1:6).eq.'CRVAL2') then
          read(hdr(11:31),*,err=990) crval2(nfr)
        else if (hdr(1:5).eq.'CD1_1') then
          read(hdr(11:31),*,err=990) cd1_1(nfr)
        else if (hdr(1:5).eq.'CD1_2') then
          read(hdr(11:31),*,err=990) cd1_2(nfr)
        else if (hdr(1:5).eq.'CD2_1') then
          read(hdr(11:31),*,err=990) cd2_1(nfr)
        else if (hdr(1:5).eq.'CD2_2') then
          read(hdr(11:31),*,err=990) cd2_2(nfr)
	end if
	goto 100
 999	write(6,*) '  *** .head file open error ***'
	return
 990	write(6,*) ' *** header reading error, at ',hdr(11:31)
 998	return
	end
c///////////////////////////////////////////////////////////////
c   subroutine to write out the mask file to a fits or MEF file
c      after object finding.  
c    the mask frame from objfinding is assumed to have
c          bleedmap and bstmask.
c      read the image back, re-make the bad-pixel map, add missed
c     saturated pixels (leave a hole where there are saturated objects
c         and output.
c
c          nfm is the mask frame from object finding
c
	subroutine wrtmask(nfr,nfm,filen,cext,adusat,ix,nu,nv,kf)
	integer ix(nu,nv,kf)
	integer status,wmode,rmode,naxes(2)
	logical simple,extend
 	character filen*60,cext*2,satposname*60,filent*60,
     *            primname*60,fileout*60,outname*60,objcom*60,
     *           filtername*1
	character ncomc*36,fitsname*24,headbst*2880,headblk*2880
	COMMON/PICC/NCOMC(15),fitsname(15)
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	common/header/bzero,bscale,headblk(20),nhb
	include 'objcat.inc'
	include 'pipeparm.inc'
	include 'picdir.inc'
c read the actual image again, to put in the bad pixel map
	filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
	write(6,*) ' writing mask image; re-reading  ',filent
	call rdkfits(*800,nfr,filent,irep,ix,nu,nv,kf,ir,0)
 800    continue
c800	call createmask(ix(1,1,nfr),ix(1,1,nfm),nu,nv,
c    *         nii(nfr),njj(nfr),nii(nfm),njj(nfm),1)
c creat mask for all saturated pixels
	call satumap(nfr,nfm,adusat,ix,nu,nv,kf,1)
c read saturation position file, and make holes on the saturation pixel
	primname=filent(1:index(filen,'_')-1)
	filtername=filent(index(filen,'_')+1:index(filen,'_')+1)
	call mkoutname(primname,1,cext,'ss')
	satposname=poutname(1:lnblnk(poutname))
	write(6,*) '  -- reading saturated pos file ',satposname
	call rpof(*802,1,satposname,idum,0)
 802	continue
	do 100, k=1,nob(nfr)
c  make sure there is a  9x9 opening around each saturated star
	  do 101 j=ijoj(k,nfr)-5,ijoj(k,nfr)+5
	    do 102 i=ijoi(k,nfr)-5,ijoi(k,nfr)+5
	      ix(i,j,nfm)=0
 102	    continue
 101	  continue
 100	continue
c out put; create name; check if it exists
c           if not, create mother frame
c           if so, add HUD
c  temperary check just put out regular file
c  write out mask, and compress  (this version for indivdual chips)
	if (imaskout.eq.1) then
c write individual fits image, need to make names with chip number
	   call mkoutname(primname,nfr,cext,'mk')
	   outname=poutname(1:lnblnk(poutname))//'.fits'
	   objcom=primname(1:lnblnk(primname))//' chip '//cext//' '
     *       //filtername//' mask by ppp'
c	   ncomc(nfm)=ncomc(nfr)(1:lnblnk(ncomc(1)))//' '//'mask'
c    *            filtername//' mask'
c copy header from 1 to for the output
 	write(6,*) ' -- individual chip mask fits outname: ',outname
	   bzest(nfm)=0.
	   bscst(nfm)=1.
	   call wrtcfits(nfm,outname,objcom,nii(nfm),njj(nfm),
     *                ix(1,1,nfm),nu,nv)
c compress the file
	   call system('gzip '//outname//' &')
c	      call wdkfits(*200,nfm,outname,1,ix,nu,nv,kf,idum)
 200	     continue
	else if (imaskout.eq.2) then
c   write into MEF file (does not work yet)
c	  outname=imdir(1:lnblnk(imdir))//
	  outname=
     *              primname(1:lnblnk(primname))//'_mk'
	write(6,*) ' - MEF mask outname ',outname
	  call wrtmefmask(outname,primname,nhud,nii(nfm),
     *        njj(nfm),ix(1,1,nfm),nu,nv)
	end if 
	return
	end
c//////////////////////////////////////////
c
	subroutine wrtmefmask(outname,primname,nhud,nii,njj,ix,nu,nv)
	integer ix(nu,nv)
	integer status,wmode,rmode,naxes(2)
	logical simple,extend
	character outname*60,filenc*60,primname*60,cext*2
c write MEF file; first check name to see if it exists
c     filenc is the name, if it exists
c write mode
	wmode=1 
	status=0
c define fits  parameters
	ibitpix=16
	naxis=2
	naxes(1)=nii
	naxes(2)=njj
	write(6,*) ' naxes : ',naxes(1),naxes(2)
	call imageex(outname,filenc,imaskex)
	write(6,*) ' outname after imageex', outname
c	outname=outname(1:index(outname,' ')-1)
c	write(6,*) ' outname after fix', outname
c get an I/O unit
c 	call ftgiou(idev,status)
c	call printerror(status)
	iun=47
	if (imaskex.lt.0) then
c  no image, initialize a new MEF file
	  outname=outname(1:lnblnk(outname))//'.fits'
	write(6,*) ' outname added .fits ', outname
	  write(6,*) ' Opening new MEF fits file ',outname
	  call ftinit(iun,outname,1,status)
	  call printerror(status)
	  simple=.true.
	  extend=.true.
	  call ftphpr(iun,simple,ibitpix,naxis,naxes,0,1,extend,
     *                    status)
	  filenc=primname(1:lnblnk(primname))//' ppp mask'
	write(6,*) ' obj name ',filenc
	  call ftpkls(iun,'OBJECT',filenc,' ',status)
c       ---- write FITS header to new file
c       print *,'need to read header here 2 !'
c	  call ftgiou(lun,status)
c	    call ftnopn(lun,imdir(1:lnblnk(imdir))//fitsname(nfr),0,
c     *         status)
c	    call printerror(status)
c	    if (status .eq. 0) then ! file exists, assume it has header!
c	    call ftghsp(lun,nkeys,nspace,status)
c	    call printerror(status)
	  else
c  write into current image to next HUD
c       open the file to the last HUD
	   istatus=encode(cext,float(nhud-1),1,0,idum)
	   filenc=filenc(1:lnblnk(filenc))//'['//cext(1:lnblnk(cext))
     *               //']'
	  write(6,*) ' mask image already exists; open ',filenc
	  call ftnopn(iun,filenc,wmode,status)
 	    call printerror(status)
c  get the number of HUDs
c	  call ftthdu(iun,nhud,status)
c	write(6,*) ' total HUD number ',nhud
c 	    call printerror(status)
c move to the last HUD
c	  call ftmahd(iun,nhud+1,hdutype,status)
c 	    call printerror(status)
c insert a IMAGE extension at the end
	write(6,*) ' creating a new HUD'
 	  call ftiimg(iun,ibitpix,naxis,naxes,status)
 	    call printerror(status)
	end if
c write the data:
        nbuffer=nii
	ipix=1
        do 150 j=1,njj
          call ftpprj(iun,1,ipix,nii,ix(1,j),status)
          ipix=ipix+nii
 150	continue
c close and deallocate units (what is lun?)
	call ftclos(iun,status)
	call ftfiou(iun,status)
c	call ftclos(lun,status)
c	call ftfiou(lun,status)
	return
	end
c//////////////////////////////////////////
c
	subroutine wrtmefmask2(iout,ichip,nii,ix,nu,nv)
	integer ix(nu,nv)
	integer status,wmode,rmode,naxes(2)
	logical simple,extend
	character outname*60,filenc*60,primname*60,cext*2
c write MEF file; first check name to see if it exists
c     filenc is the name, if it exists
c write mode
	wmode=1 
	status=0
c define fits  parameters
	ibitpix=16
	naxis=2
	naxes(1)=nii
	naxes(2)=njj
c move to the correct  HUD
 	write(6,*) ' move to HUD number ',ichip
	call ftmahd(iout,ichip,hdutype,status)
  	call printerror(status)
c write the data:
        nbuffer=nii
	ipix=1
        do 150 j=1,njj
          call ftpprj(iun,1,ipix,nii,ix(1,j),status)
          ipix=ipix+nii
 150	continue
c close and deallocate units (what is lun?)
c	call ftclos(iun,status)
c	call ftfiou(iun,status)
c	call ftclos(lun,status)
c	call ftfiou(lun,status)
	return
	end

c//////////////////////////////////////////////////////////////////
c...............................................................
c  Pipeline subroutine to create mask for an MEF file, and save it.
c    this is done for a particular image/fitler, not for all filters
c
c   ipar=0: bad pixels only
c   ipar=1: bad pixels+ bright stars
c   ipar=2: bad pixels+ cosmic ray
c   ipar=3: bad pixels+ cosmic ray + bright stars
c
c   ioutp=0, create ascii list of chip#, i,j of bad pixels
c   ioutp=1, create fits image of bad pixels
c
c  nch1, nch2: chip number range; if 0, all chips
c
c  naming convention:
c    the input name is just the pointing name, the user needs
c     to specify the filter name (from which to make the mask)
c     the image is named   pointing_r.fz, where r=filter name.
c     The image list name does not contain the filter
c
c  ascii pixel list file is name.mkp
c
	subroutine maskpipe(*,nch1,nch2,ipar,ioutp,ix,nu,nv,kf)
	character filestr*60,filen*60,filenc*60,posname*60,
     *            maskstr*60,cext*2,imdir*55,dumstr*8,errmsg*30
     *    ,imagelist(100)*60,outname*24,outname2*60,filtername*1,
     *          ncomc*36,fitsname*24,headbst*2880,headblk*2880
     *          ,primname*60
	integer ib(6),ie(6),ibm(6),iem(6),status,rmode
	integer ix(nu,nv,kf)
	include 'picdir.inc'
	include 'pppcore.inc'
	include 'ppparm.inc'
	include 'pipeparm.inc'
	include 'objcat.inc'
	include 'imagepar.inc'
	COMMON/PICC/ NCOMC(15),fitsname(15)
	common/imagedir/imdir
	common/bstarxy/nbstar,bstx(20),bsty(20),bsrad(20),bsmag(20)
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
	common/header/bzero,bscale,headblk(20),nhb
	ircspipe=1
c current version does not allow explicit input mask
	nfilm=-1
	call initghost
	write(6,1)
 1	format('  Enter pointing name (_r.fits.*/fz implicit)'
     *    /'    or a file containing a list of names         : ',$)
	read(5,'(a60)',end=999) imagelist(1)
	write(6,2)
 2	format('  Enter filter name (dft: r): ',$)
	read(5,'(a1)',end=999) filtername
	if (filtername.eq.' ') filtername='r'
c  figure out whether it is a fits file name or a file of fits names
c    nimage= number of images; =1 if name is a file name
	filestr=imagelist(1)(1:lnblnk(imagelist(1)))//'_'//filtername
c  imageex returns the fits file name, in filstr, as a dummy here
	call imageex(filestr,filenc,iimex)
	if (iimex.ne.-1) then
c the file is an image, 
	  nimage=1
	else
c the file is a list of images
	  open(unit=41,file=filestr,status='old',err=901)
	  do 110 i=1,100
	    read(41,'(a24)',end=111) imagelist(i)
 110	  continue
 111	  nimage=i-1
	  goto 112
 901	  write(6,3) filestr(1:24)
 3	format(' *** READ ERR ***, no such file: ',a24)
	  return1
	end if
 112	continue
c create masks for each image
c  image list is the primary name, always need to add filter if needed
	do 101 k=1,nimage
c determine the number of chips	
	  filestr=imagelist(k)(1:lnblnk(imagelist(k)))//'_'//filtername
	  call imageex(filestr,filenc,iimex)
	  if (iimex.lt.0) then
	    write(6,*) ' **** no image for ',imagelist(k)
	    go to 101
	  end if
c  attempt to open file  (cfitsio rwmode: 0=read, 1=write)
	  rmode=0
	  status=0
	  call ftgiou(lun,status)    
	  call ftnopn(lun,imdir(1:lnblnk(imdir))//
     *        filenc(1:lnblnk(filenc))//'[0]',rmode, status)
	  call ftgkyj(lun,'NEXTEND',nextendt,dumstr,status)
c  if not found, set nextend to 1
	  if (status.ne.0) nextendt=1
	  write(6,*) ' # of extension: ',nextendt,status
	  call ftgerr(status,errmsg)
c if writing MEF, open a file using the image as a template
	  if (imaskout.eq.2) then
	    call ftgiou(iout,status)
	    primname=imagelist(k)(1:lnblnk(imagelist(k)))
	    outname2=primname(1:lnblnk(primname))//'_mk.fz'
	write(6,*) ' ++ Open MEF file, using template ',outname2,
     *             filenc
	    call fttplt(iout,outname2,filenc,status)
	  end if
	  call ftclos(lun, status)
	  call ftfiou(lun, status)
	  if (nch1.eq.0) then
	    jc1=1
	    jc2=nextendt
	  else if (nch2.eq.0) then
	    jc1=nch1
	    jc2=nch1
	  else
	    jc1=nch1
	    jc2=min0(nextendt,nch2)
	  end if
c check ij limit (does reading an image automatically reset ?
	  write(6,*) ' +++ masks for chips ',jc1,' to ',jc2
c
	  if (imaskout.eq.0) then
c open ascii output file for the pointing
	    outname=imagelist(k)(1:lnblnk(imagelist(k)))//'.mkp'
	    call opennew(22,1,24,outname,*820)
 820	    continue 
	  end if
	  do 100 jj=jc1,jc2
c extension string
	    istatus=encode(cext,float(jj),1,0,idum)
c  this is the parameter that is passed to the pipeparm parameter
	    jchip=jj
c read image, make name
	    filen=filestr(1:lnblnk(filestr))//'^'//cext(1:lnblnk(cext))
	    write(6,*) ' reading ',filen
	    call rdkfits(*800,1,filen,irep,ix,nu,nv,kf,ir,0)
c check if there really is data on the chip, by assume a chip
c  with all zeros is an empty chip
 800	    call chzeros(*8001,*8101,1,ix,nu,nv,kf,ichzero)
 8101	      write(6,*) '  *** chip ',cext,' is empty; skipped '
	      goto 100
 8001	    write(6,*) '   >>>>> making mask for chip ',cext,' <<<<'
	    call getadusat(adusat,saturate(1),adusatpipe)
	    if (nfilm.gt.0) then
c	      maskname=maskstr(ibm(nobf1):iem(nobf1))
c 	      call getmasknam(nobf1,ibm,iem,maskstr,maskname)
c 	      if (maskname.ne.' ') then
c	      call rdkfits(*802,2,maskname,irep,ix,nu,nv,kf,ir,0)
 802	      write(6,*) '  ^^^ read in mask for primary image'
	    else if (nfilm.eq.-1) then
	      call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	      write(6,*) ' ^^^^ created mask from primary image '
	    else if (nfilm.eq.0) then
c  read in dummy mask, called dummymask.fits
	      dumstr='dummymask'
	      call rdkfits(*803,2,dumstr,irep,ix,nu,nv,kf,ir,0)
 803	       write(6,*) '  ^^^ read in dummy mask '
	    end if
c now add other bad pixel maps
	    if (ipar.eq.1.or.ipar.eq.3) then
c add bright star, including bleeding columns
c   bright star maskin algorithm:
c  (1) find stars brighter than 9, and create a large region of
c         mask around it
c  (2) to double check, run bleedmap
c  (3) find all pixels within 1000 of saturation and mark them
c
c  find bright stars using star catalog (wcs solution from frame 1)
c      bsmag1 and bsmag2 are global ppparm set parameters.
c      The positions are stored and masks created when doing sky
	      call usnodri(*808,1,kf,bsmag1,bsmag2,igetstar)
 808	      mkbsmask=0
	      if (nbstar.gt.0) then
	        do 167 kj=1,nbstar
c	          if (bsmag(kj).le.9.0) mkbsmask=1
	          if (bsmag(kj).le.12.0) mkbsmask=1
 167	        continue
c make a quick and dirty bright star mask on frame 3
	        if (mkbsmask.eq.1) then
	          write(6,*) '  ++ very bright stars found, making mask'
c	          call bsmaskcir(2,1,ix,nu,nv,kf,cext,1)
	          call bstarmask(2,1,ix,nu,nv,kf,cext,1,1,0)
	        end if
	      end if
c read in pof file, and make bleeding map
	      call mkoutname(imagelist(k),1,cext,'of')
	      posname=poutname(1:lnblnk(poutname))
 	      call rpof(*810,1,posname,idum,0)
c make mask on existing mask image
 810	      call bleedmap(*815,1,2,ix,nu,nv,kf,0,0,1)
c  make sure all saturated pixels are marked
 815	      call satumap(1,2,adusat,ix,nu,nv,kf,1)
	    end if
	    if (ipar.ge.2) then
c  add cosmic ray
	    end if
c  out put
	    if (imaskout.eq.0) then
c  write ascii pixel co-or file
c write the header, with the pos file limit from chip 1 (this
c   assumes all chips have the same pos file limits)
	      if (k.eq.jc1) write(22,14) ilpos(1),jlpos(1),iupos(1),
     *           jupos(1),imagelist(k)(1:20),filtername,jc1,jc2
 14	format(4i6,8x,' Bad pixel list (within box) for ',a20,/
     *    ' , created from filter ',a1,' chips',i3,' to',i3)
	      do 200 jp=jlpos(1),jupos(1)
	        do 201 ip=ilpos(1),iupos(1)
	          if (ix(ip,jp,2).gt.0) write(22,15) jj,ip,jp
 201	        continue
 200	      continue
 15	format(i3,1x,i5,1x,i5)
	    else if (imaskout.eq.1) then
c  output individual mask fits files
	      if (imef.eq.0) then
c  write out mask, and compress  (this version for indivdual chips)
	        call mkoutname(imagelist(k),1,cext,'mk')
	        outname=poutname(1:lnblnk(poutname))
	        ncomc(2)=ncomc(1)(1:lnblnk(ncomc(1)))//' '//
     *            filtername//' mask'
	write(6,*) 'ncomc(1)',ncomc(1)
c copy header from 1 to for the output
c  future improvement: strip all the header except for the
c   bare minimum: +name, odometer number, wcs solution
c	write(6,*) ' outname ',outname,ncomc(2)
	        bzest(2)=0.
	        bscst(2)=1.
	        write(6,*) ' writing mask image file '
	        call wdkfits(*100,2,outname,1,ix,nu,nv,kf,idum)
	      else 
c	        primname=imagelist(k)(1:lnblnk(imagelist(k)))
c	        outname2=primname(1:lnblnk(primname))//'_mk'
c	write(6,*) ' outputing MEF mask file ',outname2
c	        call wrtmefmask(outname2,primname,nhud,nii(2)
	        call wrtmefmask2(iout,jj,nii(2),ix(1,1,2),nu,nv)
	      end if
	    end if
 100	  continue
	  if (imaskout.eq.0) then
	      close(unit=22)
	  else if (imaskout.eq.2) then
 	      call ftclos(iun,status)
 	      call ftfiou(iun,status)
	  end if
 101	continue
	ircspipe=0
 999	return1
	end
c///////////////////////////////////////////////////////////////////
c subroutine to create a reasonable mask for very bright stars
c       (1) locate star from usno
c       (2) using USNO position, locate the true star by looking
c           for saturated pixels; do a connected centroid of
c           staturated pixels to find a center
c       (3) create a circular mass of appropriate size
c   possible future improvement: produce a highly smoothed high-pass
c    filtered image, and then do connected pixels to map out area
c    of high background values.
c
c///////////////////////////////////////////////////////////////
c
c   ipar=0  start a new frame
c   ipar=1  put on existing frame
c     nfm is the mask frame, nfr is the science image frame
c
c   the bright stars are masked using the following metrix of info
c     ghostpolyx(2,2,nchip,nghost):
c        shifts from star center for ghost images 1,2 and 3, for the 36 chips
c     bsmaskrad(0:3): radius of circle for the ghost 
c     bspmag(3,2): boundry mag for applying the ghost
c
c   Indexing of the ghosts:
c   bsmaskrad:    ghost 0: the actual star image
c                 1: smallest, 3: largest
c   bspmag: mag division for desiding on which ghost to use
c               bspmat(1,n): separation between ghost 0 and 1
c                      2                              1 and 2
c                      3                              2 and 3
c               i.e.; for mag brighter than bspmag(3), apply the largest ghost
c
c   2-tier masking: parameter: ntier1,ntier2
c         the tier number is the bspmag criterior
c         if 2 numbers are specified, then the 1st one is given 1 in the mask
c                                              2nd one is given -1 
c  this allows one to distinguish different level of masking required
c           e,g,  1,2 means bspmag(n,1) criteria are used for mask value 1
c                           bspmag(n,2)                       mask value 2
c                a zero means do not produce mask with that mask value
c           e.g. 0,1 means  bspmag(n,1) should be used to produce mask value 2
c
	subroutine bstarmask(nfm,nfr,ix,nu,nv,kf,cext,ipar,ntier1,ntier2)
	integer ix(nu,nv,kf),igh(2)
        dimension xgho(2),ygho(2)
	character cext*2,tempoutname*24
	include 'picdir.inc'
	include 'objcat.inc'
	common/bstarxy/nbstar,bstx(20),bsty(20),bsrad(20),bsmag(20)
	common/bsmaskdata/ghostplx(2,2,36,3),ghostply(2,2,36,3),
     *         bsmaskrad(0:3),bspmag(3,2),ighostread
c  chip number
	read(cext,*) nchip
	write(6,*) ' ++ Bright star masking for chip ',nchip
	if (ipar.eq.0) then
	  nii(nfm)=nii(nfr)
	  njj(nfm)=njj(nfr)
	  nfl(6,nfm)=1
c  fill array with 0''s
	  do 100 j=1,njj(nfm)
	    do 101 i=1,nii(nfm)
	      ix(i,j,nfm)=0
 101	    continue
 100	  continue
	end if
c  do USNO bright stars.
	
	do 120 k=1,nbstar
	  write(6,12) k,bsmag(k),bstx(k),bsty(k)
 12	format(' *',i2,' masking bright star, mag:',f6.2,' @(USNO/Tycho)',
     *               2f7.1)
c  bspmag(1) is for ghost 0 (i,e, the star)
c  first check if the star is inside the image.  If it is more
c   than 200 pix from the edges, subtract 0.5 mag from the bspmag crit
	  if (bstx(k).le.-200..or.bstx(k).ge.float(nii(nfr))+200..or.
     *       bsty(k).le.-200..or.bsty(k).ge.float(njj(nfr))+200.) then
	    delmagc=0.5
	  else 
	    delmagc=0.0
	  end if
          do 121 i=1,2
            if (i.eq.1) then
              nnn=ntier1
            else
              nnn=ntier2
            end if
            if (nnn.ne.0) then
c note that ghost 0 is not used here
	      radmult=1.00000
	      if (bsmag(k).gt.bspmag(2,nnn)-delmagc) then 
	        igh(i)=1
c determine additional masking due to multiple defraction spikes
c   beyond ghost 1
	        if (bsmag(k).lt.11.5) then
	          radmult=1.+2.*(11.7-bsmag(k))/2.5
	        end if
	        write(6,*) '  --- ghost 2 radius multiplier: ',radmult
	      else if (bsmag(k).gt.bspmag(3,nnn)-delmagc) then
	        igh(i)=2
	      else 
	        igh(i)=3
	      end if 
	      call goffset(bstx(k),bsty(k),igh(i),nchip,xgho(i),ygho(i))
            else
c             igh(i)=-1
	      goto 121
            end if
c           if (igh(i).eq.-1) goto 121
	    radsq=(bsmaskrad(igh(i))*radmult)**2.0
	    icen=nint(xgho(i))
	    jcen=nint(ygho(i))
	    write(6,10) bstx(k),bsty(k),bsmag(k),xgho(i)-bstx(k),
     *         ygho(i)-bsty(k),icen,jcen,igh(i),
     *         radmult*bsmaskrad(igh(i))
 10	format(' Bright star @',2f6.0,' mag:',f6.2,' offset xy',
     *       2f7.2,' ghost center:',2i6,' ghost# ',i1,' rad',f5.0)
 	    do 110 jj=1,njj(nfr)
 	      do 111 ii=1,nii(nfr)
c	    do 110 jj=jlpos(nfr),jupos(nfr)
c	      do 111 ii =ilpos(nfr),iupos(nfr)
c compute distance to the centner
	        distq=(ii-icen)**2+(jj-jcen)**2
	        if (distq.lt.radsq) then
                   if (i.eq.1) then
                     ix(ii,jj,nfm)=1
                   else
                     ix(ii,jj,nfm)=-1
                   end if
                end if
 111	      continue
 110	    continue
 121      continue
 120	continue
c go through bright stars from .phm file
c    *** this is not done for the purpose of object finding
c	do 121 k=1,nob(nfr)
c	  if (rmag(k).lt.-16.and.icl(k).ge.3) then
c use integer aperture of diamter 21
c	  end if
c121	continue
	return
	end
c/////////////////////////////////////////////////////////
c  add a ghost 1 mask to the image at position specified
c
	subroutine addbstarmask(nfr,ic,jc,ix,nu,nv)
	integer ix(nu,nv)
	include 'picdir.inc'
	include 'objcat.inc'
	common/bsmaskdata/ghostplx(2,2,36,3),ghostply(2,2,36,3),
     *         bsmaskrad(0:3),bspmag(3,2),ighostread
c  the chip number is passed via ich_id
        common/imagestats/imstatspar(50),ich_id,ch_rms(50),ch_sky(50),
     *     nbstar_ch(50),ch_bsmag(15,50),ch_bsx(15,50),ch_bsy(15,50)
c check if the chip number has been set and ghost data available
	if (ich_id.lt.1.or.ich_id.gt.50.or.ighostread.le.0) then
	  write(6,*) '   *** Chip number or ghost offset data NOT set'
	  return
	end if
	xc=float(ic)
	yc=float(jc)
	call goffset(xc,yc,1,ich_id,xgho,ygho)
	radsq=bsmaskrad(1)*bsmaskrad(1)
	icen=nint(xgho)
	jcen=nint(ygho)
	write(6,10) ich_id,xc,yc,icen,jcen,bsmaskrad(1)
 10	format('  -- Chip ',i2,': adding ghost#1 to RCS2 bright star@'
     *      ,2f6.0, ' ghost center:',2i6,' radius',f5.0)
 	do 110 jj=1,njj(nfr)
 	  do 111 ii=1,nii(nfr)
c compute distance to the centner
	    distq=(ii-icen)**2+(jj-jcen)**2
	    if (distq.lt.radsq) then
c             if (i.eq.1) then
                ix(ii,jj)=1
c             else
c               ix(ii,jj)=-1
c             end if
            end if
 111	  continue
 110	continue
	return
	end
c////////////////////////////////////////////////////////
c   subroutine to compute the bright star ghost images from
c       the star position, using Mike''s polynomial fit for
c        each chip
c  see Mike'' email, Jun 23, 2008 for the polynomial fit format
c
	subroutine goffset(bstx,bsty,igh,nchip,xgho,ygho)
	common/bsmaskdata/ghostplx(2,2,36,3),ghostply(2,2,36,3),
     *         bsmaskrad(0:3),bspmag(3,2),ighostread
	xgho=0.0
	ygho=0.0
	do 101 j=1,2
	  if (j.eq.1) then
	    temp1=1.
	  else
	    temp1=bsty
	  end if
	  do 100 i=1,2
	    if (i.eq.1) then
	      temp=temp1
	    else
	      temp=temp1*bstx
	    end if
	    xgho=xgho+ghostplx(j,i,nchip,igh)*temp
	    ygho=ygho+ghostply(j,i,nchip,igh)*temp
 100	  continue
 101	continue
	return
	end
c///////////////////////////////////////////////////////
c   subroutine to read Mike''s ghost image position data file
c       to be substituted by a data statement later...
c
	subroutine rdghostpoly(*)
	common/bsmaskdata/ghostplx(2,2,36,3),ghostply(2,2,36,3),
     *         bsmaskrad(0:3),bspmag(3,2),ighostread
	open(unit=25,file='starghost.poly.dat',status='old',err=998)
c	do 101 igh=3,1,-1
 	do 101 igh=1,3 
	  do 100 ich=1,36
	    read(25,*,end=999) idum 
	    read(25,*,end=999) ghostplx(1,1,ich,igh),ghostplx(2,1,ich,igh)
	    read(25,*,end=999) ghostplx(1,2,ich,igh),ghostplx(2,2,ich,igh)
	    read(25,*,end=999) ghostply(1,1,ich,igh),ghostply(2,1,ich,igh)
	    read(25,*,end=999) ghostply(1,2,ich,igh),ghostply(2,2,ich,igh)
 100	  continue
 101	continue
	close(unit=25)
c define radii and mag criterior
	bsmaskrad(0)=17.
	bsmaskrad(1)=65.
	bsmaskrad(2)=285.
	bsmaskrad(3)=1030.
        bspmag(1,1)=16.5
        bspmag(2,1)=10.0
        bspmag(3,1)=9.4
c *** need to check the mag division!!!
        bspmag(1,2)=10
        bspmag(2,2)=9.1
        bspmag(3,2)=8.0
	ighostread=1
	write(6,1) ghostplx(1,1,1,1),ghostplx(1,1,2,1)
 1	format(' -- ghost polynomial read successfully --',2g10.3)
	return1
 999	close(unit=25)
	write(6,*) ' *** read error/unexpected end ** @igh,ich ',igh,ich
	return1
 998	write(6,*) '  *** starghost.poly.dat file does not exist,'
     *              ,' read aborted '
	return1
	end
c/////////////////////////////////////////////////////////
c  subroutine to mark all satuarted pixels on a map
c
c   ipar=0  start a new frame
c   ipar=1  put on existing frame
c     nfm is the mask frame, nfr is the science image frame
c
	subroutine satumap(nfr,nfm,adusat,ix,nu,nv,kf,ipar)
	integer ix(nu,nv,kf)
	include 'picdir.inc'
	include 'objcat.inc'
	if (ipar.eq.0) then
	  nii(nfm)=nii(nfr)
	  njj(nfm)=njj(nfr)
	  nfl(6,nfm)=1
c  fill array with 0''s
	  do 100 j=1,njj(nfm)
	    do 101 i=1,nii(nfm)
	      ix(i,j,nfm)=0
 101	    continue
 100	  continue
	end if
	iadusat=nint(adusat)-1000
	write(6,*) '  mapping saturated pixel  > ',iadusat,adusat
	do 110 j=1,njj(nfr)
	  do 111 i=1,nii(nfr)
	    if (ix(i,j,nfr).ge.iadusat) ix(i,j,nfm)=1
 111	  continue
 110	continue
	return
	end
c//////////////////////////////////////////////////////////
c  driving routine to compute chip stats and create masks
c
        subroutine frstdri(*,ix,nu,nv,kf)
	integer ix(nu,nv,kf),status,rmode
        character filen*60,errmsg*60,dumstr*60,filenc*60,imdir*55
	common/imagedir/imdir
 600    write(6,1)
 1      format(' Enter image name: ',$)
        read(5,'(a60)',end=999) filen
c check existence of image and get chip number
        call imageex(filen,filenc,iimex)
        if (iimex.lt.0) then
          write(6,*) ' **** no image for ',filen
          go to 600
        end if
c  attempt to open file  (cfitsio rwmode: 0=read, 1=write)
        rmode=0
        status=0
	call initghost
        call ftgiou(lun,status)
        call ftnopn(lun,imdir(1:lnblnk(imdir))//
     *        filenc(1:lnblnk(filenc))//'[0]',rmode, status)
        call ftgkyj(lun,'NEXTEND',nextendt,dumstr,status)
c  if not found, set nextend to 1
        if (status.ne.0) nextendt=1
        write(6,*) ' # of extension: ',nextendt,status
        call ftgerr(status,errmsg)
        call ftclos(lun, status)
        call ftfiou(lun, status)
	if (jc1.eq.0.or.jc2.eq.0) then
          jc1=1
          jc2=nextendt
	end if
 601    write(6,2) jc1,jc2
 2      format('  Enter chip number range, 0=all [',2i3,']: ',$)
        call read2i(jc1,jc2,*999,*601)
	if (jc1.eq.0) then
	  jc1=1
	  jc2=nextendt
	end if
        if (imaskout.eq.1) imaskout=0
 602    write(6,3) imaskout 
 3      format('  Do you want to save mask frames? 0=no [',i1,']: ',$)
        call read1i(imaskout,*999,*602)
        call frstats(jc1,jc2,filen,ix,nu,nv,kf)
 999    return1
        end 
c////////////////////////////////////////////////////////////
c   subroutine to compute sky background and rms stats for all chips
c       and save info on bright star locations
c
c    does this for the primary filter, and save result in a common block
c     jc1, jc2: chip range
c     filestr: the string of filter names
c     nextendt: number of extension in fits file (number of chips)
c     nfilm: frame numbe of mask
c
C 
c   Jul 21, 2008; 
c
	subroutine frstats(jc1,jc2,filen,ix,nu,nv,kf)
	integer ix(nu,nv,kf),ib(6),ie(6),ibm(6),iem(6),
     *            ibadr(36),ibads(36)
        real temrms(36),temsky(36)
	character dumstr*60,fitsname*24,ncomc*36,
     *        filen*60,filent*60,cext*2,primname*60,outname*24
	character headbst*2880
	include 'pppcore.inc'
	include 'pipeparm.inc'
	include 'imagepar.inc'
	include 'objcat.inc'
	include 'picdir.inc'
	include 'ppparm.inc'
	common/headerst/bzest(15),bscst(15),headbst(20,15),nhbst(15)
        common/picc/ncomc(15),fitsname(15)
        common/bstarxy/nbstar,bstx(20),bsty(20),bsrad(20),bsmag(20)
        common/imagestats/imstatspar(50),ich_id,ch_rms(50),ch_sky(50),
     *   nbstar_ch(50),ch_bsmag(15,50),ch_bsx(15,50),ch_bsy(15,50)
	dumstr='dummy'
	idum=0
	imaskpipe=2
	ircspipe=1
c  nfilref is assumed to be one: need to update
	nfilref=1
c for each chip, search for USNO bright stars, 
c         
c  to find the positions of reference objects, and saved
	do 100, jj=jc1,jc2
	  istatus=encode(cext,float(jj),1,0,idum)
	  jchip=jj
	  write(6,*) ' @@@ Obtaining bright stars, sky, & rms for chip '
     *              ,cext,' filter ',nfilref
c create name strings  ** need to fix **
	  primname=filen(1:index(filen,'_')-1)
	  filent=filen(1:lnblnk(filen))//'^'//cext(1:lnblnk(cext))
c read image
	  write(6,*) '  ^^^^ reading image: ',filent
	  call rdkfits(*800,1,filent,irep,ix,nu,nv,kf,ir,0)
c check for .head files, which, if found, replaces the wcs solution
c   read from the .fits file (chkwcshead)
c     this is a fix of very badly off wcs solutions
 800	  call chkwcshead(1,filent,cext)
	  call chzeros(*8001,*8101,1,ix,nu,nv,kf,ichzero)
 8101	    write(6,*) ' *** Chip ',cext,' is empty; skipped '
	    ch_rms(jj)=999.9 
	    ch_sky(jj)=9999.9
	    imstatspar(jj)=-1
	    goto 100
c create bad column mask from image, put mask in 2
 8001	  call createmask(ix(1,1,1),ix(1,1,2),nu,nv,
     *                    nii(1),njj(1),nii(2),njj(2),0)
 	  write(6,*) ' ^^^^ created bad column mask from image'
	  call smspars(1,filent)
c fix adusat
	  call getadusat(adusat,saturate(1),adusatpipe)
c  make sure all saturated pixels are marked
	  call satumap(1,2,adusat-500,ix,nu,nv,kf,1)
 	  write(6,*) ' ^^^ find bright stars for chip'
c find the bright star list 
          call usnodri(*802,1,kf,bsmag1,bsmag2,igetstar)
 802      continue
c save bright stars in common block
          do 110 k=1,nbstar
            ch_bsx(k,jj)=bstx(k)
            ch_bsy(k,jj)=bsty(k)
            ch_bsmag(k,jj)=bsmag(k)
 110      continue
          nbstar_ch(jj)=nbstar
c create bright star mask
          call bstarmask(2,1,ix,nu,nv,kf,cext,1,1,0)
c for testing, output mask fits file
	  imaskouttest=0
          if (imaskouttest.eq.1) then
c  write out mask, and compress  (this version for indivdual chips)
	    filent=filen(1:index(filen,'_')-1)
            call mkoutname(filent,1,cext,'mk') 
            outname=poutname(1:lnblnk(poutname))     
            ncomc(2)=ncomc(1)(1:lnblnk(ncomc(1)))//' '//
     *            ' mask'                  
            write(6,*) 'ncomc(1)',ncomc(1)                 
c copy header from 1 to for the output                 
            bzest(2)=0.                              
            bscst(2)=1.                              
            write(6,*) ' writing mask image file '   
            call wdkfits(*180,2,outname,1,ix,nu,nv,kf,idum)
          end if
c compute sky and rms values
 180      call rowskyval(1,2,100,50,nii(1)-50,
     *          rcmed,sigmed,ix,nu,nv,kf,*182)
 182      ch_rms(jj)=sigmed
          ch_sky(jj)=rcmed
          imstatspar(jj)=1
 100    continue
c summarize result in a table
        write(6,11) primname(1:24),bsmag2
 11     format('    --- Summary of chip statististics for: ',a24,' ---'
     *  /' chip#   sky     rms    N(stars brighter than',f6.2,')')
	kp=0
        do 120 i=jc1,jc2
	  if (imstatspar(i).eq.1) then
	    kp=kp+1
	    temsky(kp)=ch_sky(i)
	    temrms(kp)=ch_rms(i)
	  end if
          write(6,10) i,ch_sky(i),ch_rms(i),nbstar_ch(i)
 10     format(i3,2x,f9.2,f8.2,6x,i3)
 120    continue
c compute statistics of sky and rms values, search for outlyers
c  
	call mmm(temsky,kp,nzs,skymean,skymed,skymod,skysig,
     *          zskew,zreject,niter,3.,onemean)
	call mmm(temrms,kp,nzr,rmsmean,rmsmed,rmsmod,rmssig,
     *          zskew,zreject,niter,3.,onemean)
	write(6,12) kp,nzs,skymean,skymed,skymod,skysig,
     *              kp,nzr,rmsmean,rmsmed,rmsmod,rmssig
 12	format(2i4,' mean/med/mod sky:',3f9.2,f7.2/
     *      2i4,' mean/med/mod rms:',3f7.2,f6.2)
c  search for outlyers
	crms=3.*rmssig
	csky=3.*skysig
	iflag=0
	iflagch=0
	ks=0 
	kr=0
	do 121 i=jc1,jc2
	  del=abs(ch_rms(i)-rmsmean)
	  if (del.gt.crms.and.imstatspar(i).eq.1) then
	    write(6,13) i,ch_rms(i),rmsmean,del
 13	format(' chip ',i2,'  rms of sky:',f7.2,'  mean rms',f7.2,
     *          '  sigma off:',f6.2)
	    kr=kr+1
	    ibadr(kr)=i
	    iflag=1
	    if (del.ge.5.0) iflagch=1
	  end if
	  del=abs(ch_sky(i)-skymean)
	  if (del.gt.csky.and.imstatspar(i).eq.1) then
	    write(6,14) i,ch_sky(i),skymean,del
 14	format(' chip ',i2,'  sky value:',f8.2,'  mean sky',f8.2,
     *          '  sigma off:',f6.2)
	    ks=ks+1
	    ibads(ks)=i
	    iflag=1
	    if (del.ge.5.0) iflagch=1
	  end if
 121	continue
c  it is also considered checking required if the rms of the rms
c      is >1/3 of the rms
	if (3.*rmssig.ge.rmsmean) iflag=1
c issue warning
	if (iflag.eq.1) then
	  if (iflatch.eq.1) then
	    dumstr=primname(1:lnblnk(primname))//'.sta.CHECK'
	  else
	    dumstr=primname(1:lnblnk(primname))//'.sta.check'
	  end if
	else
	  dumstr=primname(1:lnblnk(primname))//'.sta'
	end if
	  call opennew(29,1,60,dumstr,*999)
          write(29,11) primname(1:24),bsmag2
          do 130 i=jc1,jc2
            write(29,10) i,ch_sky(i),ch_rms(i),nbstar_ch(i)
 130      continue
	  write(29,12) kp,nzs,skymean,skymed,skymod,skysig,
     *           kp,nzr,rmsmean,rmsmed,rmsmod,rmssig
	  write(29,*) ' chips with outlying stats, or rms too scattered'
	  do 131 i=1,kr
	    del=abs(ch_rms(ibadr(i))-rmsmean)/rmssig
	    write(29,13) ibadr(i),ch_rms(ibadr(i)),rmsmean,del
 131	  continue
	  do 132 i=1,ks
	    del=abs(ch_sky(ibadr(i))-skymean)/skysig
	    write(29,14) ibads(i),ch_sky(ibads(i)),skymean,del
 132	  continue
	  close(unit=29)
 999	return
	end
c///////////////////////////////////////////////////
c   subroutine to read in Mike''s polynomial fit and write them
c        out into a fortran data statement
c
	subroutine polywrt(*)
	character outname*24
	common/bsmaskdata/ghostplx(2,2,36,3),ghostply(2,2,36,3),
     *         bsmaskrad(0:3),bspmag(3,2),ighostread
	outname='polyfortran.out'
	call opennew(28,1,24,outname,*999)
	write(28,1)
 1	format('       data ghostplx/')
	do 100 ig=1,3
	  do 101 ic=1,36
	    write(28,2) ghostplx(1,1,ic,ig),ghostplx(2,1,ic,ig),
     *                  ghostplx(1,2,ic,ig),ghostplx(2,2,ic,ig)
 2	format('     *    ',4(g12.5,','))
 101	  continue
 100	continue
	write(28,3)
 3	format('       data ghostply/')
	do 200 ig=1,3
	  do 201 ic=1,36
	    write(28,2) ghostply(1,1,ic,ig),ghostply(2,1,ic,ig),
     *                  ghostply(1,2,ic,ig),ghostply(2,2,ic,ig)
 201	  continue
 200	continue
	close(unit=28)
 999	return1
	end
c////////////////////////////////////////////////////
c  subroutine to initiate ghost image array
c
	subroutine initghost
	common/bsmaskdata/ghostplx(2,2,36,3),ghostply(2,2,36,3),
     *         bsmaskrad(0:3),bspmag(3,2),ighostread
       data ghostplx/
     *     -4.6503    ,-0.32874E-03, 0.99836    ,-0.11688E-07,
     *     -1.2212    ,-0.30449E-03, 0.99838    ,-0.12399E-07,
     *      2.1960    ,-0.28011E-03, 0.99839    ,-0.10819E-07,
     *      5.5787    ,-0.25237E-03, 0.99839    ,-0.10525E-07,
     *      8.9683    ,-0.21994E-03, 0.99841    ,-0.16998E-07,
     *      12.397    ,-0.20212E-03, 0.99838    ,-0.10483E-07,
     *      15.831    ,-0.17122E-03, 0.99838    ,-0.16218E-07,
     *      19.303    ,-0.14294E-03, 0.99836    ,-0.13557E-07,
     *      22.880    ,-0.12721E-03, 0.99831    ,-0.81649E-08,
     *     -6.3074    ,-0.35009E-03, 0.99830    ,-0.15957E-07,
     *     -2.7562    ,-0.31647E-03, 0.99833    ,-0.18812E-07,
     *     0.78097    ,-0.28529E-03, 0.99833    ,-0.16146E-07,
     *      4.3082    ,-0.25233E-03, 0.99834    ,-0.16638E-07,
     *      7.8431    ,-0.22080E-03, 0.99833    ,-0.14961E-07,
     *      11.399    ,-0.19247E-03, 0.99832    ,-0.12046E-07,
     *      14.956    ,-0.15554E-03, 0.99831    ,-0.14738E-07,
     *      18.591    ,-0.13000E-03, 0.99828    ,-0.92689E-08,
     *      22.237    ,-0.84800E-04, 0.99827    ,-0.21454E-07,
     *      13.565    ,-0.40630E-03, 0.99815    , 0.17444E-07,
     *      9.6411    ,-0.36876E-03, 0.99816    , 0.19588E-07,
     *      5.7555    ,-0.33029E-03, 0.99817    , 0.18809E-07,
     *      1.8888    ,-0.29245E-03, 0.99818    , 0.19042E-07,
     *     -1.9673    ,-0.25243E-03, 0.99818    , 0.15243E-07,
     *     -5.8294    ,-0.21761E-03, 0.99817    , 0.19199E-07,
     *     -9.6932    ,-0.17877E-03, 0.99816    , 0.20394E-07,
     *     -13.602    ,-0.13632E-03, 0.99815    , 0.13773E-07,
     *     -17.548    ,-0.97052E-04, 0.99815    , 0.10166E-07,
     *      15.763    ,-0.43860E-03, 0.99802    , 0.24771E-07,
     *      11.581    ,-0.38165E-03, 0.99806    , 0.17949E-07,
     *      7.4622    ,-0.33855E-03, 0.99807    , 0.18102E-07,
     *      3.3647    ,-0.29356E-03, 0.99808    , 0.19085E-07,
     *    -0.69703    ,-0.25093E-03, 0.99808    , 0.17567E-07,
     *     -4.7597    ,-0.21413E-03, 0.99807    , 0.23328E-07,
     *     -8.8623    ,-0.15933E-03, 0.99807    , 0.12571E-07,
     *     -12.953    ,-0.12471E-03, 0.99805    , 0.18189E-07,
     *     -17.098    ,-0.79720E-04, 0.99804    , 0.16637E-07,
     *     -40.756    , 0.10310E-03, 0.99393    , 0.35098E-07,
     *     -28.061    , 0.20133E-05, 0.99400    , 0.49299E-07,
     *     -15.492    ,-0.48940E-04, 0.99406    , 0.27426E-07,
     *     -2.9624    ,-0.15165E-03, 0.99408    , 0.44032E-07,
     *      9.5135    ,-0.21570E-03, 0.99413    , 0.24937E-07,
     *      22.083    ,-0.32280E-03, 0.99407    , 0.44808E-07,
     *      34.679    ,-0.39941E-03, 0.99406    , 0.34689E-07,
     *      47.346    ,-0.47332E-03, 0.99400    , 0.36771E-07,
     *      60.301    ,-0.58309E-03, 0.99388    , 0.48875E-07,
     *     -40.233    , 0.43767E-04, 0.99411    , 0.31451E-07,
     *     -28.001    ,-0.21301E-04, 0.99424    , 0.20030E-07,
     *     -15.849    ,-0.84101E-04, 0.99426    , 0.21596E-07,
     *     -3.7248    ,-0.15515E-03, 0.99429    , 0.26685E-07,
     *      8.3636    ,-0.22083E-03, 0.99429    , 0.28318E-07,
     *      20.507    ,-0.29799E-03, 0.99426    , 0.37495E-07,
     *      32.650    ,-0.35563E-03, 0.99425    , 0.31666E-07,
     *      44.976    ,-0.43771E-03, 0.99417    , 0.44920E-07,
     *      57.352    ,-0.47465E-03, 0.99413    , 0.16550E-07,
     *      51.923    , 0.51321E-04, 0.99443    ,-0.31265E-07,
     *      40.096    ,-0.10999E-04, 0.99445    ,-0.20217E-07,
     *      28.391    ,-0.61517E-04, 0.99452    ,-0.28984E-07,
     *      16.777    ,-0.12021E-03, 0.99452    ,-0.20078E-07,
     *      5.1975    ,-0.16383E-03, 0.99456    ,-0.28187E-07,
     *     -6.3648    ,-0.22002E-03, 0.99453    ,-0.21505E-07,
     *     -17.914    ,-0.26965E-03, 0.99450    ,-0.17651E-07,
     *     -29.583    ,-0.30727E-03, 0.99448    ,-0.30796E-07,
     *     -41.344    ,-0.35064E-03, 0.99447    ,-0.41844E-07,
     *      52.139    ,-0.45648E-04, 0.99445    ,-0.10619E-07,
     *      40.402    ,-0.48384E-04, 0.99458    ,-0.30007E-07,
     *      28.876    ,-0.90623E-04, 0.99461    ,-0.28048E-07,
     *      17.418    ,-0.12921E-03, 0.99463    ,-0.19261E-07,
     *      6.0479    ,-0.16377E-03, 0.99464    ,-0.24410E-07,
     *     -5.3018    ,-0.21353E-03, 0.99461    ,-0.10281E-07,
     *     -16.732    ,-0.22270E-03, 0.99460    ,-0.29430E-07,
     *     -28.171    ,-0.26992E-03, 0.99454    ,-0.18519E-07,
     *     -39.737    ,-0.29172E-03, 0.99451    ,-0.24559E-07,
     *     -184.71    , 0.51280E-03, 0.97739    , 0.97516E-08,
     *     -137.41    , 0.38505E-03, 0.97764    , 0.78639E-07,
     *     -90.710    , 0.55783E-03, 0.97797    ,-0.83641E-07,
     *     -43.939    , 0.34161E-03, 0.97797    , 0.42584E-07,
     *      2.4356    , 0.36123E-03, 0.97819    ,-0.30055E-07,
     *      49.112    , 0.22519E-03, 0.97795    , 0.44119E-07,
     *      95.931    , 0.17959E-03, 0.97791    , 0.21281E-07,
     *      142.93    , 0.15982E-03, 0.97773    , 0.23539E-07,
     *      190.72    , 0.79799E-04, 0.97743    , 0.21279E-07,
     *     -181.96    , 0.26516E-03, 0.97735    ,-0.13370E-08,
     *     -135.10    , 0.26049E-03, 0.97783    ,-0.20466E-07,
     *     -88.671    , 0.33395E-03, 0.97806    ,-0.56390E-07,
     *     -42.210    , 0.31556E-03, 0.97814    ,-0.24257E-07,
     *      4.0010    , 0.33717E-03, 0.97818    ,-0.24060E-07,
     *      50.401    , 0.31350E-03, 0.97808    , 0.91356E-08,
     *      96.812    , 0.34893E-03, 0.97805    ,-0.69246E-08,
     *      143.80    , 0.31534E-03, 0.97776    , 0.37169E-07,
     *      191.02    , 0.42188E-03, 0.97764    ,-0.57102E-07,
     *      228.43    , 0.35555E-04, 0.97751    ,-0.51762E-08,
     *      180.66    , 0.34571E-04, 0.97752    , 0.66241E-07,
     *      133.22    , 0.14452E-03, 0.97788    ,-0.10243E-07,
     *      86.291    , 0.16509E-03, 0.97787    , 0.49031E-07,
     *      39.489    , 0.26168E-03, 0.97800    , 0.25838E-07,
     *     -7.2270    , 0.32315E-03, 0.97793    , 0.39545E-07,
     *     -53.827    , 0.39727E-03, 0.97783    , 0.51484E-07,
     *     -100.87    , 0.51709E-03, 0.97773    , 0.16900E-07,
     *     -148.27    , 0.63273E-03, 0.97773    ,-0.34595E-07,
     *      230.13    ,-0.32321E-03, 0.97695    , 0.56852E-07,
     *      181.36    ,-0.10652E-03, 0.97742    , 0.15801E-07,
     *      133.36    , 0.25058E-05, 0.97757    , 0.13747E-07,
     *      85.698    , 0.10471E-03, 0.97760    , 0.63468E-07,
     *      38.311    , 0.25782E-03, 0.97773    , 0.28165E-07,
     *     -8.8964    , 0.32350E-03, 0.97760    , 0.84193E-07,
     *     -56.386    , 0.55390E-03, 0.97756    , 0.17566E-07,
     *     -103.94    , 0.64009E-03, 0.97736    , 0.62936E-07,
     *     -151.97    , 0.85819E-03, 0.97728    , 0.12238E-07/
       data ghostply/
     *      22.790    , 0.99762    , 0.40703E-03,-0.38771E-07,
     *      21.921    , 0.99771    , 0.40510E-03,-0.38445E-07,
     *      21.082    , 0.99779    , 0.40383E-03,-0.41678E-07,
     *      20.278    , 0.99787    , 0.39068E-03,-0.38457E-07,
     *      19.476    , 0.99794    , 0.37071E-03,-0.32749E-07,
     *      18.695    , 0.99802    , 0.36429E-03,-0.31648E-07,
     *      17.922    , 0.99809    , 0.36130E-03,-0.34616E-07,
     *      17.183    , 0.99817    , 0.34989E-03,-0.34626E-07,
     *      16.450    , 0.99824    , 0.35503E-03,-0.37914E-07,
     *      10.848    , 0.99766    , 0.20790E-03,-0.39493E-07,
     *      10.406    , 0.99774    , 0.20325E-03,-0.37840E-07,
     *      9.9775    , 0.99782    , 0.19813E-03,-0.36673E-07,
     *      9.5594    , 0.99791    , 0.19798E-03,-0.37557E-07,
     *      9.1517    , 0.99798    , 0.19527E-03,-0.36565E-07,
     *      8.7549    , 0.99805    , 0.18126E-03,-0.31646E-07,
     *      8.3515    , 0.99813    , 0.19226E-03,-0.36226E-07,
     *      7.9682    , 0.99821    , 0.18092E-03,-0.36852E-07,
     *      7.5813    , 0.99827    , 0.18377E-03,-0.33909E-07,
     *      11.196    , 0.99757    ,-0.16349E-03, 0.40743E-07,
     *      10.867    , 0.99765    ,-0.16982E-03, 0.42505E-07,
     *      10.515    , 0.99774    ,-0.15634E-03, 0.39429E-07,
     *      10.193    , 0.99782    ,-0.15471E-03, 0.38728E-07,
     *      9.8661    , 0.99790    ,-0.14266E-03, 0.35465E-07,
     *      9.5458    , 0.99798    ,-0.14427E-03, 0.36807E-07,
     *      9.2434    , 0.99806    ,-0.13189E-03, 0.32521E-07,
     *      8.9764    , 0.99812    ,-0.14288E-03, 0.41590E-07,
     *      8.7183    , 0.99819    ,-0.14581E-03, 0.40980E-07,
     *      23.598    , 0.99754    ,-0.33658E-03, 0.30178E-07,
     *      22.823    , 0.99762    ,-0.34628E-03, 0.35916E-07,
     *      22.077    , 0.99770    ,-0.35498E-03, 0.41043E-07,
     *      21.354    , 0.99777    ,-0.36063E-03, 0.44531E-07,
     *      20.610    , 0.99786    ,-0.34265E-03, 0.41292E-07,
     *      19.900    , 0.99794    ,-0.32716E-03, 0.40192E-07,
     *      19.200    , 0.99802    ,-0.29847E-03, 0.30541E-07,
     *      18.555    , 0.99809    ,-0.30088E-03, 0.34770E-07,
     *      17.903    , 0.99817    ,-0.27550E-03, 0.28607E-07,
     *      61.237    , 0.99385    , 0.51195E-03,-0.24015E-07,
     *      60.128    , 0.99390    , 0.51631E-03,-0.22005E-07,
     *      59.157    , 0.99395    , 0.49490E-03,-0.28346E-07,
     *      58.316    , 0.99396    , 0.43378E-03,-0.16989E-07,
     *      57.484    , 0.99398    , 0.38028E-03,-0.39028E-08,
     *      56.711    , 0.99399    , 0.35314E-03, 0.84927E-09,
     *      55.976    , 0.99400    , 0.33980E-03,-0.38508E-08,
     *      55.350    , 0.99401    , 0.29881E-03,-0.80671E-09,
     *      54.744    , 0.99401    , 0.30707E-03,-0.89165E-08,
     *      30.356    , 0.99396    , 0.38003E-03,-0.16901E-07,
     *      29.522    , 0.99401    , 0.36822E-03,-0.12815E-07,
     *      28.705    , 0.99404    , 0.37031E-03,-0.11801E-07,
     *      27.978    , 0.99407    , 0.35171E-03,-0.13582E-07,
     *      27.256    , 0.99409    , 0.34534E-03,-0.11065E-07,
     *      26.554    , 0.99410    , 0.31433E-03, 0.76886E-09,
     *      25.869    , 0.99411    , 0.32615E-03,-0.53387E-08,
     *      25.224    , 0.99413    , 0.30008E-03,-0.67102E-08,
     *      24.560    , 0.99411    , 0.31202E-03,-0.12027E-08,
     *      25.163    , 0.99392    , 0.18576E-03, 0.25390E-07,
     *      25.647    , 0.99395    , 0.15620E-03, 0.33105E-07,
     *      26.029    , 0.99400    , 0.20945E-03, 0.23475E-07,
     *      26.511    , 0.99403    , 0.22381E-03, 0.18822E-07,
     *      26.991    , 0.99407    , 0.25860E-03, 0.81124E-08,
     *      27.511    , 0.99409    , 0.25671E-03, 0.99534E-08,
     *      28.064    , 0.99411    , 0.28841E-03, 0.21031E-09,
     *      28.706    , 0.99408    , 0.27240E-03, 0.18660E-07,
     *      29.387    , 0.99409    , 0.26002E-03, 0.19963E-07,
     *      56.118    , 0.99386    , 0.18169E-03,-0.10624E-07,
     *      56.333    , 0.99388    , 0.16091E-03, 0.54345E-08,
     *      56.644    , 0.99389    , 0.14070E-03, 0.22175E-07,
     *      57.018    , 0.99391    , 0.13438E-03, 0.28408E-07,
     *      57.350    , 0.99395    , 0.18549E-03, 0.19278E-07,
     *      57.762    , 0.99398    , 0.23791E-03, 0.10986E-07,
     *      58.243    , 0.99399    , 0.29696E-03,-0.71347E-08,
     *      58.838    , 0.99398    , 0.29617E-03, 0.28552E-08,
     *      59.428    , 0.99402    , 0.35680E-03,-0.12498E-07,
     *      220.04    , 0.97642    , 0.75447E-03,-0.15122E-07,
     *      218.09    , 0.97654    , 0.89581E-03,-0.45380E-07,
     *      216.82    , 0.97656    , 0.76347E-03,-0.41023E-07,
     *      216.00    , 0.97647    , 0.53281E-03,-0.46806E-08,
     *      215.21    , 0.97641    , 0.34081E-03, 0.39830E-07,
     *      214.58    , 0.97631    , 0.24874E-03, 0.56388E-07,
     *      214.12    , 0.97624    , 0.19902E-03, 0.45691E-07,
     *      214.05    , 0.97610    , 0.28044E-04, 0.71703E-07,
     *      214.07    , 0.97599    , 0.38335E-04, 0.42471E-07,
     *      101.84    , 0.97684    , 0.55761E-03, 0.26499E-07,
     *      100.32    , 0.97690    , 0.56415E-03, 0.16505E-07,
     *      98.839    , 0.97689    , 0.62623E-03, 0.18530E-07,
     *      97.815    , 0.97685    , 0.52355E-03, 0.15811E-07,
     *      96.732    , 0.97680    , 0.51812E-03, 0.17758E-07,
     *      95.704    , 0.97671    , 0.42525E-03, 0.56733E-07,
     *      94.806    , 0.97663    , 0.43088E-03, 0.49005E-07,
     *      94.041    , 0.97653    , 0.32377E-03, 0.52326E-07,
     *      93.120    , 0.97637    , 0.41577E-03, 0.57048E-07,
     *      111.68    , 0.97680    , 0.45903E-03, 0.49449E-07,
     *      113.01    , 0.97679    , 0.37628E-03, 0.65657E-07,
     *      113.99    , 0.97685    , 0.58527E-03, 0.30535E-07,
     *      115.36    , 0.97684    , 0.64248E-03, 0.12376E-07,
     *      116.76    , 0.97685    , 0.76212E-03,-0.25268E-07,
     *      118.31    , 0.97679    , 0.77119E-03,-0.26586E-07,
     *      119.97    , 0.97673    , 0.89327E-03,-0.59996E-07,
     *      121.99    , 0.97649    , 0.84482E-03, 0.65509E-08,
     *      124.19    , 0.97635    , 0.77575E-03, 0.25898E-07,
     *      229.99    , 0.97651    , 0.68374E-03,-0.81965E-07,
     *      230.87    , 0.97647    , 0.67589E-03,-0.31642E-07,
     *      232.16    , 0.97642    , 0.65306E-03, 0.19145E-07,
     *      233.78    , 0.97638    , 0.63548E-03, 0.40988E-07,
     *      235.30    , 0.97639    , 0.82915E-03, 0.98979E-08,
     *      237.11    , 0.97635    , 0.10518E-02,-0.32224E-07,
     *      239.23    , 0.97626    , 0.12679E-02,-0.95243E-07,
     *      241.81    , 0.97606    , 0.12458E-02,-0.35266E-07,
     *      244.40    , 0.97606    , 0.14690E-02,-0.11491E-06/
c define radii and mag criterior
	bsmaskrad(0)=17.
	bsmaskrad(1)=70.
	bsmaskrad(2)=290.
	bsmaskrad(3)=1025.
c  this is the division for masking for statistics, for USNO R
c       bspmag(1,1)=16.5
c       bspmag(2,1)=10.0
c       bspmag(3,1)=9.4
c  division in r (from Tycho B and V)
        bspmag(1,1)=16.7
        bspmag(2,1)=10.2
        bspmag(3,1)=9.6
c  this is the division for masking for object finding, for USNO R
c *** need to check the mag division!!!
c       bspmag(1,2)=11.5
c       bspmag(2,2)=8.9
c       bspmag(3,2)=7.9
c  division for r
        bspmag(1,2)=11.7
        bspmag(2,2)=9.1
        bspmag(3,2)=8.1
	ighostread=1
	write(6,*) '  ++ Megacam ghost image data implemented' 
	return
	end
c/////////////////////////////////////////////////////////
c  subroutine to examine the size statistics of the .pc2 files
c      for different chips to provide warning of problem chips
c
	subroutine chkpc2size(*,nchipt)
	character string*80,sizstr*80,filename*40
	dimension pc2size(50,5),ib(20),ie(20),ichip(36)
	include 'pipeparm.inc'
c  read in the size of the pc2 files for each filter
	nchip=nchipt
	if (nchip.le.0) nchip=36
	filename='pc2size.dat'
	do 100 j=1,nfilt
c check existance of file, 
	  string='ls -lat -1 *'//ffiltn(i)//'.pc2  > '//filename
c  .pc2 files do not exist, check .pc2.Z and pc2.gz
c	  string='ls -lat -1 *'//ffiltn(i)//'.pc2.Z  > '//filename
c	  string='ls -lat -1 *'//ffiltn(i)//'.pc2.gz  > '//filename
	  call system(string)
	  ichip(j)=0
	  open(unit=31,file=filename,status='old',err=998)
	  goto 801
 800	  write(6,*) ' ** no pc2 files for filter ',ffiltn(j),string
	  goto 100
 801	  continue
	  do 101 k=1,50
	    read(31,1,end=101) sizstr
 1	format(a80)
c parse the string
	    call fstr(sizstr,80,ib,ie,ns)
	    ichip(j)=ichip(j)+1
	    read(sizstr(ib(5):ie(5)),*,err=810) pc2size(k,j)
	    go to 101
 810	    ichip(j)=ichip(j)-1
 101	  continue
	  close(unit=31)
c do statistics
c	  call mmm(pc2size(1,j),nc
 100	continue
c check for outliers
 998	continue
	return1
	end
